/*
    Advanced Explorer (c) 1999-2006 by D.G. Sureau

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    webmaster@scriptol.org
    http://www.scriptol.org
*/
package common;

/**
   The Canvas3d 3d viewer.
 
   Utilise 
   - Trois Components3d  pour les mondes virtuels (rel, view, clip)
   - Un Virtual3d pour l'cran virtuel associ au Componend3d clip
   - Un viewpoint utilis par makeViewWorld
   - Frustum compos de cinq plane
   - Des vertex

   add:            ajoute  world un component3d
   createFrustrum  sert au clip, aucun effet visible
   rotate:         transforme world selon X,Y,Z
   makeViewWorld:  world est transform en viewWorld selon viewpoint
   paint:          transforme viewWorld en clipWorld selon frustrum
                   paint par clipWorld sur virtualScreen
                   paint virtualScreen sur le canvas 
                   
   Component3d (world, viewworld, clippedworld)
   - Liste de polygon3d
   rotate, translate, drawVirtual (paint sur cran virtuel)
   
   Virtual3d (virtualscreen)
   - Tableau de pixel
      render une texture polygone par polygone
   - Cr et effac par update
   - Rempli dans paint, par la fonction drawVirtual de clipworld
     elle appelle la fonction paint de chaque polygone
     laquelle appelle la fonction fillPoly de Virtual3d 

   l'image est trop petite dans l'cran virtuel
 */   

import java.awt.*;

public class Canvas3d extends Canvas
{
	int width = 0;
	int height = 0;

	private boolean interactive = false;
	private boolean inspect = false;

	private int zBuffer[];
	private boolean m_tAnimate = true;

	private Virtual3d virtualscreen;
	private Viewpoint viewPoint;

	Component3d world = new Component3d();
	Component3d viewworld = new Component3d();
	Component3d clippedWorld = new Component3d();

	int moveX, moveY, moveZ;

	public Canvas3d() {}
	public Canvas3d(Viewpoint viewPoint)
	{
		this.viewPoint = viewPoint;
		this.viewPoint.parent = this;
		setLight(new Light(0, 10, 0)); // futur
		createfrustum();
	}


	public Dimension getPreferredSize() 
	 { return new Dimension(160, 120);}

    /**
       Ajout d'un composant 3d au canvas
       il est ajout au component3d world
    */   
	public void add(Component3d vol)
	{
		world.add(vol);
		vol.parent = this;
	}
    
    /** Nouveau world */   
    public void clearWorld() { world = new Component3d(); }    

    /* Lumire futur */   
       
	Light light;
	Light origlight;

	public void setLight(Light light)
	{
      this.light = light;
      this.origlight = new Light(light);
	}


//-------------------------- Transformations externes 

   boolean running = true;

   public void setX(int x)
    { 
     moveX = x;
     if(moveX < 0) moveX += 360;
     running = true;
    }
   public void setY(int y) 
   {  
     moveY = y;
     if(moveY < 0) moveY += 360;     
     running = true;
   }
   public void setZ(int z)
   { 
     moveZ = z;
     if(moveZ < 0) moveZ += 360;     
     running = true;
   }   
   

   /**
       La commande rotate
       est transmise au Component3d world
   */    
   public void rotate()
   {
    //System.out.println("canvas3d - rotate " + moveX + " " + moveY + " " + moveZ); 
    
	world.rotate(moveY, moveX, moveZ);
	try { Thread.sleep(50);	} catch(InterruptedException e) { }
   }		


   /** Dlivre l'image actuelle */   

    public Image getImage() { return virtualscreen.getImage(); } 


    /**
        Partie visible?
        5 lments dans le tableau de plans
        donc les plans visibles
        Un plan contient un vertex (normal) et une distance
    */    
	Plane frustum[] = new Plane[5];

	private void createfrustum()
	{
		frustum[0] = new Plane();
		frustum[0].normal = new Vertex(0, -Math.sqrt(2), Math.sqrt(2));
		frustum[0].dist = 0;

		frustum[1] = new Plane();
		frustum[1].normal = new Vertex(-Math.sqrt(2), 0, Math.sqrt(2));
		frustum[1].dist = 0;

		frustum[2] = new Plane();
		frustum[2].normal = new Vertex(Math.sqrt(2), 0, Math.sqrt(2));
		frustum[2].dist = 0;

		frustum[3] = new Plane();
		frustum[3].normal = new Vertex(0, Math.sqrt(2), Math.sqrt(2));
		frustum[3].dist = 0;
		
		frustum[4] = new Plane();
		frustum[4].normal = new Vertex(0, 0, 1);
		frustum[4].dist = 2;

	}


    /**
       cr une viewWorld  partir de world
       et opre 
       - translation x, y, z (viewpoint.from.xyz)
       - rotation orientation x, y, z (viewpoint.orientation)
    */   
	private void makeViewWorld(Viewpoint viewPoint)
	{
	    // Cre un vertext selon point de vue
		Vertex diff1 = new Vertex();
		// Cre view world selon world
		viewworld = new Component3d(world);

		diff1.x = -viewPoint.from.x;
		diff1.y = -viewPoint.from.y;
		diff1.z = -viewPoint.from.z;
		
		viewworld.translate(diff1);
		viewworld.rotate(viewPoint.o1);

        // La lumire subit la mme transformation
		light = new Light(origlight);
		light.translate(diff1);
		light.rotate(viewPoint.o1);
	}


   /**
       Dfinit la taille de l'cran virtuel
       et donc du canvas qui le contient
       
       J'ai chang en 3/4
   */    

   int xOffset;
   int yOffset;
   
   public void setVirtualSize(int w, int h)
   {
     width = w;
     height = h;
 	 virtualscreen = new Virtual3d(this, w * 3 / 4, h * 3 / 4);	
     xOffset = w / 8;
     yOffset = h / 8;
   }	    


   /*
     comme l'image est construite et prise sur l'cran virtuel
     c'est de celui-ci que l'on efface le fond
   */
	public void update(Graphics g)
	{
	 //   System.out.println("Canvas3d - update");
	    if(! running) return;
		if (virtualscreen != null)
		{
			virtualscreen.clear(); 
			paint(g); // appel classique mais qui va utiliser virtual3d
		}
		running = false;
	}


	public void paint(Graphics g)
	{
	 int ret1, w, h;
	 int hbr, hbrold;

	 if ((height == 0) || (width == 0))
	 {
	   setVirtualSize(size().width, size().height);
	 }  

	 if (world != null)
	  {
	    // viewPoint est dclar par le constructeur
	    // et chang interactivement pour dplacer l'objet
		makeViewWorld(viewPoint); // translation

		clippedWorld = viewworld.clipworld(frustum);
		clippedWorld.toScreen(width , height ); // aucun effet
		clippedWorld.depthOrder(viewPoint);

		if (virtualscreen != null)
		{
			//virtualscreen.setLight(light);
            // cre image clippe sur cran virtuel
                
       //     System.out.println("Canvas3d - draw " + width + "x" + height);
		    clippedWorld.drawVirtual(virtualscreen);
		    g.setColor(Color.green);
		    g.fillRect(0 , 0, width, height);
		    // chang aussi pour centr l'cran virtuel affich
			g.drawImage(virtualscreen.getImage(), xOffset, yOffset, this); // place l'image
//			g.drawImage(virtualscreen.getImage(), width, height, this); // place l'image
		}
	   }
   }

}


