/*
    Advanced Explorer (c) 1999-2006 by D.G. Sureau

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    webmaster@scriptol.org
    http://www.scriptol.org
*/
package common;
/**
 * Component3d is a standard 3d component.

   rotate, translate, depthOrder  transforment l'image
   toScreen applique les transformation
   clipworld clip l'image en fonction de frustrum de 5 plans
   drawVirtual affiche l'image sur un cran virtuel
   
   Dtermine un tableau de vertex qui sera rempli par la
   texture dans virtual 3d indpendamment de la taille
   de celle-ci. Ce tableau donne la taille
 */

import java.awt.*;

public class Component3d 
{
	Component parent;
	Polygon3d poly[] = new Polygon3d[100];
	int polyCount = 0;

	public Component3d() {			//{{INIT_CONTROLS
		//}}
}	

    /*
       Constructeur 
       qui clone un autre Component3d
       en paramtre
    */   
	public Component3d(Component3d v) 
	{
	  for (int i = 0; i < v.poly.length; i++)
	  {
	 	if (v.poly[i] != null)
		  this.poly[i] = new Polygon3d(v.poly[i]);
      }
	  this.polyCount = v.polyCount;
	}	

    /*
       Ajoute un polygone
    */   
	public void add(Polygon3d p) 
	{
		poly[polyCount++] = new Polygon3d(p);
	}

    /*
       Fusionne un autre Component3d
       donc ajoute ses polygones  la liste
    */   
	public void add(Component3d v) 
	{
		for (int i = 0; i < v.polyCount; i++)
			poly[polyCount++] = v.poly[i];
	}

    /* Rotation des polygones */  
    
	public void rotate(int a, int b, int c) 
	{
		rotate(new Orient(a, b, c));
	}


	public void rotate(Orient o1) 
	 {
	  int i, j;
	  for (i = 0; i < polyCount; i++) 
	  {
		poly[i].rotate(o1);
	  }
     }


    /* Translation. Selon un vertex  */   
	public void translate(Vertex disp1) 
	 {
      int i, j;
      for (i = 0; i < polyCount; i++) 
	   {
		 poly[i].translate(disp1);
	   }
     }


    /**
       Gre l'ordre d'affichage
       utilis aprs rendering de l'image sur l'cran virtuel 
    */

	public void depthOrder(Viewpoint vp) 
	{
		Polygon3d temp = new Polygon3d();
		boolean isCorrectOrder;

		int i, j, z, w, h, v;
		for (i = polyCount - 2; i >= 0; i--) 
		{
			for (j = 0; j <= i; j++) 
			{
				isCorrectOrder = false;

				if (this.poly[j].minz >= this.poly[j + 1].maxz) 
				{
					isCorrectOrder = true;
				}
				else if (this.poly[j].maxz < this.poly[j + 1].minz) 
				{
				}
				else 
				{
					double planeDist = poly[j + 1].normal.dotproduct(poly[j + 1].p[0]);
					for (int k = 0; (k < poly[j].vertcount) && !isCorrectOrder; k++) {
						if (poly[j + 1].normal.dotproduct(poly[j].p[k])
							< planeDist)
//{
//System.out.print(" not swapping " + poly[j + 1].normal.dotproduct(poly[j].p[k]) + 
//" with " + planeDist);
					isCorrectOrder = true;
//}
				}
			}

			if (!isCorrectOrder) 
			{
					temp = this.poly[j];
					this.poly[j] = this.poly[j + 1];
					this.poly[j + 1] = temp;
			}
		  }
	  }
	}

 /**
     Ajuste les polygones selon la taille de l'cran    
     aucun effet remarqu  la suppression
 */
	public void toScreen(int w, int h) 
	{
		int i, j, v;
		double minz, maxz, z;
		int midwidth, midheight;
    
		midwidth = w / 2;
		midheight = h / 2;
		for (i = 0; i < polyCount; i++) 
		{
			minz = 10000000;
			maxz = 0;
			for (j = 0; j < this.poly[i].vertcount; j++) 
			{
				z = this.poly[i].p[j].z;
				if (z < minz) 
				{
					minz = z;
				}
				if (z > maxz) 
				{
					maxz = z;
				}
				
				this.poly[i].spx[j] = (int)(midwidth
					+ (midwidth * (poly[i].p[j].x / z)));
				this.poly[i].spy[j] = (int)(midheight
					- (midwidth * (poly[i].p[j].y / z)));
			}
			this.poly[i].minz = minz;
			this.poly[i].maxz = maxz;
		}
	}



    /**
       Clip l'image
    */    
	public Component3d clipworld(Plane[] frustum) 
	{
		int i, j;
		boolean infrustum, oneoutside;

		Component3d clippedworld = new Component3d();
		i = 0;
		while (i < this.polyCount) 
		{
			oneoutside = false;
			infrustum = true;
			if (this.poly[i].p[0].dotproduct(this.poly[i].normal) > 0) 
			{
				infrustum = false;
			}
			else 
			{
				j = 0;
				while ((j < this.poly[i].vertcount) && (!oneoutside)) {
					if (this.poly[i].p[j].z <= 2) 
					{
						oneoutside = true;
					}
					else if (this.poly[i].p[j].z
						<= Math.abs(this.poly[i].p[j].x)) 
					{
						oneoutside = true;
					}
					else if (this.poly[i].p[j].z
						<= Math.abs(this.poly[i].p[j].y)) 
					{
						oneoutside = true;
					}
					j++;
				}
				if (oneoutside) 
				{
					infrustum = this.poly[i].clipback(frustum[0]);
					if (infrustum) 
					{
						infrustum = this.poly[i].clipback(frustum[1]);
						if (infrustum) 
						{
							infrustum = this.poly[i].clipback(frustum[2]);
							if (infrustum) 
							{
								infrustum
									= this.poly[i].clipback(frustum[3]);
								if (infrustum) 
								{
									infrustum
										= this.poly[i].clipback(
											frustum[4]);
								}
							}
						}
					}
				}
			}
			if (infrustum) 
			{
				clippedworld.add(this.poly[i]);
			}
			i++;
		}
		return clippedworld;
	}

    /**
       On repaint le composant 3d, sur un cran virtuel
       (donc tous les polygones qui le composent)
    */   
	
	public void drawVirtual(Virtual3d g3d) 
	{
	   // System.out.println("component3d - paint " + polyCount + " polygons");

		for (int i = 0; i < polyCount; i++) 
		{
			poly[i].paint(g3d);
		}
	}
	
	//{{DECLARE_CONTROLS
	//}}
}
