/*
    Advanced Explorer (c) 1999-2006 by D.G. Sureau

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    webmaster@scriptol.org
    http://www.scriptol.org
*/
package comgr;

/**
   Displays a series of images using a
   soft fade index colour model filter to perform the transitions.
 */

import java.lang.String;
import java.net.MalformedURLException;
import java.net.URL;
import java.awt.*;
import java.awt.event.*;
import java.applet.Applet;
import java.applet.AppletContext;
import java.io.*;
import java.net.*;
import java.awt.image.*;


public class Fade extends Applet implements Runnable
{

	// Our very own thread
	transient private Thread thrFade;

	// Image information
	private CacheImage imgCache[] = new CacheImage[3];// Our cache of image producers
	private boolean imagesReady = false;		        	// Are the images ready?
	private FadeFilter fader = new FadeFilter();		  // The filter
	private Image imgDisplay;		// The image that's drawn on screen

	/** Start the applet running */
	public void start()
  {
		if (thrFade == null)
    {
			isRunning = true;
			thrFade = new Thread(this);
			thrFade.start();
		}
	}

	/**
	 * Stop the applet running. Notice it does this by setting a flag -
	 * a much nicer way of stopping than by using a rather nasty "stop()"
	 * call.
	 */
	public void stop() {isRunning = false;	}

	// The flag we will use to stop the applet
	private boolean isRunning = true;

	/**
	 * The main loop of the applet. This method will keep running until
	 * the isRunning boolean is set to false.
	 */
	public void run()
  {
		/*
		 * The initial light level is set to 501. Anything above 255
		 * will be ignored, but by setting the light level at a high
		 * value we ensure that the image will remain constant for a
		 * few seconds before fading.
		 */
		int light = 401;

		// This is the amount the image will be fade by at each stage
		int stepSize = 10;

		// Are we fading up (+1), down (-1) or constant (0)?
		int lightDir = 0;

		// The number of the image we are currently looking at
		int imageNo = 0;

		while (isRunning)
    {
			// Only work with the images once they are loaded
			if (imagesReady)
      {
  			if ((light += lightDir) < stepSize)
        {	// If we have faded to black...
					lightDir = stepSize;			// ...start to fade up...
					if (++imageNo >= imgCache.length)	// ...and switch to next image
						imageNo = 0;
				}
				else
        if (light > 400)				// If we are at full brightness...
					lightDir = -stepSize;			// ...start to fade down

				fader.setLightLevel(light);			// Set new light level

				imgDisplay = createImage(			// Filter next frame
					new FilteredImageSource( imgCache[imageNo],fader));
			}

			repaint();	// Ask for a screen repaint

			// Sleep very briefly (use sleep(...) rather than field() for
			// a constant update rate
			try {	Thread.sleep(5);	}
			catch (InterruptedException e)
      {
				System.out.println("InterruptedException: " + e);
				isRunning = false;
			}
		}
	}


	/**
	 * Initialise the applet
	 */
	public void init()
  {
		for (int i = 0; i < imgCache.length; i++)
    {
			// Create a new CacheImage
			imgCache[i] = new CacheImage(
				getImage(getDocumentBase(), getParameter("image" + (i + 1))),	this);
		}
		imagesReady = true;	// Images are now loaded
	}

	/**
	 * Update the screen. Explicitly call paint this way to
	 * prevent unnecessary background colour painting
	 */
	public void update(Graphics g) { paint(g); }

	/**
	 * Paint the imgDisplay image to the screen (if it
	 * is ready).
	 *
	 * @param g		The graphics object to draw to
	 */
	public void paint(Graphics g)
  {
		if (imgDisplay != null)
			g.drawImage(imgDisplay, 0, 0, this);
	}
}

