/*
    Advanced Explorer (c) 1999-2006 by D.G. Sureau

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    webmaster@scriptol.org
    http://www.scriptol.org
*/
package comgr;

import java.io.*;
import java.awt.image.*;
import java.awt.*;

/**
    Standard Windows' ICO file management
    by D.G. Sureau
    1999
    
    Il faut convertir les short et int en byte
    a cause de la faon dont ils sont lus dans les fichiers
    
    Les fonctions externes sont:
    - capture: transforme une image en icon
               il faut instancier avec les paramtres
    - getImage: transforme l'icon en image
    - save: cre l'icon en fichier standard
    - load: cre l'icon  partir d'un fichier ICO
            pour l'afficher
  */    

public class Icon
{
 short idReserved = 0; // Reserved; must be zero.
 byte  idType = 1;     // Specifies the resource type. This member is set to 1.
 byte  idType1 = 0;   // high order
 byte  idCount = 1;    // Specifies the number of entries in the directory.
 byte  idCount1 = 0;    // high order

 byte  width = 32;       // width of the icon, 16, 32, and 64.
 byte  height = 32;      // height
 byte  bColorCount = 16;  // number of colors in the icon. 2, 8, and 16.
 byte  bColorCount1 = 0;  
 short wPlanes = 0;       // number of color planes in the icon bitmap.
 short wBitCount = 0;     // number of bits in the icon bitmap.

 byte  dwBytesInRes = (byte) 254;  // size of the resource, in bytes.
 byte  dwBytesInRes1 = 2;
 byte  dwBytesInRes2 = 0;
 byte  dwBytesInRes3 = 0; 

 byte  dwImageOffset = 22; // offset, in bytes, from the beginning of the file to the image.
 byte  dwImageOffset1 = 0;
 byte  dwImageOffset2 = 0;
 byte  dwImageOffset3 = 0;

// BITMAPINFOHEADER

 byte  biSize = 40;        // Size of this info header
 byte  biSize1 = 0;
 byte  biSize2 = 0;
 byte  biSize3 = 0; 
 
 byte  biWidth = 32;
 byte  biWidth1 = 0;
 byte  biWidth2 = 0;
 byte  biWidth3 = 0; 
 
 byte  biHeight  = 32;
 byte  biHeight1 = 0;
 byte  biHeight2 = 0;
 byte  biHeight3 = 0; 

 
 byte  biPlanes  = 1;
 byte  biPlanes1 = 0;
 
 byte  biBitCount = 4;
 byte  biBitCount1 = 0;
 
 byte  biCompression = 0;
 byte  biCompression1 = 0;
 byte  biCompression2 = 0;
 byte  biCompression3 = 0; 
 
 byte  biSizeImage = (byte) 128;
 byte  biSizeImage1 = 2;
 byte  biSizeImage2 = 0;
 byte  biSizeImage3 = 0; 
 
 int   biXPelsPerMeter = 0;
 int   biYPelsPerMeter = 0;
 int   biClrUsed = 0;
 int   biClrImportant = 0;


 /*
    RGB inside icon palette, from 1 to 4
    BYTE    rgbBlue;
    BYTE    rgbGreen;
    BYTE    rgbRed;
    BYTE    rgbReserved;
    C'est l'inverse de l'entier java pour une couleur
    
    Valeur avant reclassement:
     0 0 0       128 0 0    0 128 0    128 128 0
     0 0 128     128 0 128  0 128 128  128 128 128 
     192 192 192 255 0  0   255 0 255   0 255 255
     0 0 255     0 255 0    255 255 0   255 255 255
     
*/

 byte icColors[][] = new byte[16][4];

 byte icXOR[] = new byte[512];
 byte icAND[] = new byte[128];


// Variables de travail

 transient int pixels[];    // Entiers de reprsentation RGB de couleur
 transient int transparent = 7;
 transient int maxColors = 16;


 public Icon() { 		//{{INIT_CONTROLS
		//}}
}


 public Icon(int colors, int w, int h)
 {
  setMaxColors(colors);  
  
  icColors = new byte[colors][4];

  width  = (byte) w;
  height = (byte) h;

  biWidth  = (byte) w;
  biHeight = (byte) h;
 
 }

void setMaxColors(int colors)
{
  bColorCount  = (byte) (colors & 255);
  bColorCount1 = (byte) ((colors >> 8) & 255);
  maxColors = colors;
}  

/** A partir d'une icone charge */

void setMaxColors()
{
  maxColors = unsigned(bColorCount1) * 256 + unsigned(bColorCount);
  System.out.println("count " + bColorCount + " " + maxColors);
}  


int translate(int c)
{
 if( (c==1) || (c==4))  c = 5 - c;
 if( (c==3) || (c==6))  c = 9 - c;
 if( (c==7) || (c==8))  c = 15 - c;
 if( (c==9) || (c==12)) c = 21 - c;
 if(( c==11)|| (c==14)) c = 25 - c;

 return(c);
}



/**
   Convertit une couleur de la palette de l'icone en couleur Java
   
   icon: bleu, vert, rouge, alpha 
   java: alpha, rouge, vert, bleu
*/

int icoToRGB(int i)
{
  int r = unsigned(icColors[i][2]); 
  int g = unsigned(icColors[i][1]); 
  int b = unsigned(icColors[i][0]);  
  return 0xFF << 24 | r << 16 | g << 8 | b;
}

int unsigned(byte b)
{
 if(b < 0) return (int) b + 256;     
 else return (int) b;
}    

/** Retourne les couleurs java */
Color[] getColors()
{
 Color[] couleurs = new Color[maxColors];
 
 for(int i = 0; i < maxColors; i++)
 {
  int b = unsigned(icColors[i][0]);          
  int g = unsigned(icColors[i][1]); 
  int r = unsigned(icColors[i][2]); 

  couleurs[translate(i)] = new Color(r, g, b);
 }
 return couleurs;    
}  

/**
   Transforme l'image en tableau d'entiers RGB
   et cre la palette de l'icone
   
   Noter que les dimension sont < 128 d'ou
   la conversion directe byte/entier
   
   Noter que les pixels sont des entiers RGB car
   mme si on utilise le modle index, il remplit
   le tableau avec getRGB!
*/    

void capture(Image image, PaletteShaded palette)
{
  //pixels = new int[width][height];
  pixels = new int[width * height];

  PixelGrabber pg = new PixelGrabber(image, 0, 0, width, height, pixels, 0, width);
  pg.setColorModel(palette.getColorModel());
  
  try { pg.grabPixels(); }  catch(InterruptedException i) { }

  if ((pg.getStatus() & ImageObserver.ABORT) != 0) 
  {
	System.err.println("Icon capture aborted or errored");
	return;
  }
	
  //pixels = (int[]) pg.getPixels();
  //width  = (byte) pg.getWidth();
  //height = (byte) pg.getHeight();
  //System.out.println("Icon " + pg.getWidth() + " x " + pg.getHeight() + " len " + pixels.length);

  // Cration de la palette  partir d'une palette shaded
  // L'ordre doit tre translat
  for(int i = 0; i < maxColors; i++)
  {
   int j = translate(i);
   Color color = palette.getColor(j);  
  
   icColors[i][0] = (byte) color.getBlue();
   icColors[i][1] = (byte) color.getGreen();
   icColors[i][2] = (byte) color.getRed();
   icColors[i][3] = (byte) 255;  
  }
  
  // Convertit les couleurs directes en index
  // L'ordre de l'index est celui du format ICO
  for(int i = pixels.length - 1; i >= 0; i--)
  {
   int k = pixels[i];
   int l = transparent;
    
   for(int j = 0; j < maxColors; j++)
   {
     if(k == icoToRGB(j)) { l = j; continue; }
   }
   pixels[i] = l;  // reoit l'index ou transparent
  }  
  
  switch(maxColors)
  {
   case 256: make256(); break;
   default:  make16();  break;
  } 
   
}    


/**
    Capture l'image  partir d'un tableau de pixels
    
    Il faut que les pixels soient en mode IndexColorModel
    On ne translate pas car les index sont construits
     partir de la palette de l'icone
*/

void make256()
{

}

void make16()
{
 int m;
 int adr     = 0;
 int maskadr = 0;
 int lmask   = 0;
 int maskval = 0;
 final int COLORMASK = 15;

// int t = 0;

  for(int j = height - 1; j >= 0; j--)
   for(int k = 0; k < width; k+=2)
   {
    int i = j * width + k;
    
    // On construit un octet de deux pixels
    int c =  pixels[i];
//    if(t++ < 16) System.out.println(t +") " + c);
    
    if(c == transparent) m = 1; else  m = 0;
    int val =  (c & COLORMASK) << 4;
    maskval = (maskval << 1) | m;

    c =  pixels[i + 1];
     if(c == transparent) m = 1; else  m = 0;
    val = val | (c & COLORMASK);
    maskval = (maskval << 1) | m;

    byte b; 
    if(val < 128) b = (byte) val; else b = (byte) (val - 256); 
    icXOR[adr++] = (byte) b;

    lmask += 2;
    if(lmask == 8)
     {
      lmask = 0;
      icAND[maskadr++] = (byte) maskval;
      maskval = 0;
     }
   }
}


/**
   Transforme l'icone en image
   Toujours le modle direct
*/   

Image getImage()
{
  // Cre un tableau de pixels en mode direct 
  int len = icXOR.length;
  int i = 0;
  int j = 0; 
  int k = 0;
  int maskbit = 128;
  int maskptr = 0;
  int mask;
  
  //System.out.println("icon get AND len " + len);    
  
  setMaxColors();
  //System.out.println("icon max col " + maxColors);  
  //System.out.println("icon width " + width);  
  //System.out.println("icon height " + height);    
  
  switch(maxColors)
  {
   case 256:
    pixels = new int[len]; 

    for(j = height - 1; j >= 0; j--)
     for(i = 0; i < width; i++)
       pixels[k++] = icoToRGB((int) icXOR[j * width + i]); 

    break;

   default:
     pixels = new int[len * 2]; 
     //System.out.println("pixels size " + pixels.length + " " + len);
    
     
     try
     {
      int w = width / 2;  
      j = height - 1; 
      while(j >= 0)
      {
       for(i = 0; i < w; i++)
       {
        int pix = (int) icXOR[j * w + i];
        int high = (pix >> 4) & 15;
        int low =  pix & 15;
        //mask = icAND[maskptr] & maskbit;
        pixels[k++] = icoToRGB(high);
        pixels[k++] = icoToRGB(low);
        //maskbit >> 1;
        //if(maskbit == 8) { maskbit = 0; mask
       }
       j --;
      } 
     } catch(ArrayIndexOutOfBoundsException a)
     { System.out.println("Icon: error in getImage");  }
    break;
  
  }  
    
  MemoryImageSource mis =
   new MemoryImageSource(width, height, pixels, 0,  width);

  return Toolkit.getDefaultToolkit().createImage(mis);
    
} 



 //----------------- Sauvegarde


 void load(String path)
 {
  
  try 
  {
   DataInputStream p = new DataInputStream(new FileInputStream(path));

     idReserved = p.readShort();
     idType = p.readByte();
     idType1 = p.readByte();
     idCount = p.readByte();
     idCount1 = p.readByte();
     width = p.readByte();
     height = p.readByte();
     bColorCount = p.readByte();
     bColorCount1 = p.readByte();
     
     wPlanes = p.readShort();       // number of color planes in the icon bitmap.
     wBitCount = p.readShort();     // number of bits in the icon bitmap.
     
     dwBytesInRes = p.readByte();
     dwBytesInRes1 = p.readByte();
     dwBytesInRes2 = p.readByte();
     dwBytesInRes3 = p.readByte(); 

     dwImageOffset = p.readByte();
     dwImageOffset1 = p.readByte();
     dwImageOffset2 = p.readByte();
     dwImageOffset3 = p.readByte();

     // BITMAPINFOHEADER

     biSize = p.readByte();
     biSize1 = p.readByte();
     biSize2 = p.readByte();
     biSize3 = p.readByte(); 
 
     biWidth = p.readByte();
     biWidth1 = p.readByte();
     biWidth2 = p.readByte();
     biWidth3 = p.readByte(); 
 
     biHeight = p.readByte();
     biHeight1 = p.readByte();
     biHeight2 = p.readByte();
     biHeight3 = p.readByte(); 
 
     biPlanes = p.readByte();  
     biPlanes1 = p.readByte();
 
     biBitCount = p.readByte();
     biBitCount1 = p.readByte();
 
     biCompression = p.readByte();
     biCompression1 = p.readByte();
     biCompression2 = p.readByte();
     biCompression3 = p.readByte(); 
 
     biSizeImage = p.readByte();
     biSizeImage1 = p.readByte();
     biSizeImage2 = p.readByte();
     biSizeImage3 = p.readByte(); 
 
     biXPelsPerMeter = p.readInt();
     biYPelsPerMeter = p.readInt();
     biClrUsed = p.readInt();
     biClrImportant = p.readInt();

     for(int i = 0; i < 16; i++)
      p.read(icColors[i], 0, 4);

     p.read(icXOR, 0, 512);
     p.read(icAND, 0, 128);

     p.close();
  //System.out.println("icon load passed");    
  }
  catch(IOException io) { }

 }

 boolean save(String path)
 {
   try
    {
	 DataOutputStream p = new DataOutputStream(new FileOutputStream(path));

     p.writeShort(idReserved);
     p.writeByte(idType);
     p.writeByte(idType1);
     p.writeByte(idCount);
     p.writeByte(idCount1);
     p.writeByte(width);
     p.writeByte(height);
     p.writeByte(bColorCount);
     p.writeByte(bColorCount1);
     
     p.writeShort(wPlanes);       // number of color planes in the icon bitmap.
     p.writeShort(wBitCount);     // number of bits in the icon bitmap.
     
     p.writeByte(dwBytesInRes);
     p.writeByte(dwBytesInRes1);
     p.writeByte(dwBytesInRes2);
     p.writeByte(dwBytesInRes3); 

     p.writeByte(dwImageOffset);
     p.writeByte(dwImageOffset1);
     p.writeByte(dwImageOffset2);
     p.writeByte(dwImageOffset3);

     // BITMAPINFOHEADER

     p.writeByte(biSize);
     p.writeByte(biSize1);
     p.writeByte(biSize2);
     p.writeByte(biSize3); 
 
     p.writeByte(biWidth);
     p.writeByte(biWidth1);
     p.writeByte(biWidth2);
     p.writeByte(biWidth3); 
 
     p.writeByte(biHeight);
     p.writeByte(biHeight1);
     p.writeByte(biHeight2);
     p.writeByte(biHeight3); 

 
     p.writeByte(biPlanes);  
     p.writeByte(biPlanes1);
 
     p.writeByte(biBitCount);
     p.writeByte(biBitCount1);
 
     p.writeByte(biCompression);
     p.writeByte(biCompression1);
     p.writeByte(biCompression2);
     p.writeByte(biCompression3); 
 
     p.writeByte(biSizeImage);
     p.writeByte(biSizeImage1);
     p.writeByte(biSizeImage2);
     p.writeByte(biSizeImage3); 
 
     p.writeInt(biXPelsPerMeter);
     p.writeInt(biYPelsPerMeter);
     p.writeInt(biClrUsed);
     p.writeInt(biClrImportant);


     for(int i = 0; i < 16; i++)
      p.write(icColors[i], 0, 4);
   
     p.write(icXOR, 0, 512);
     p.write(icAND, 0, 128);

     p.flush();
     p.close();
     return true;
  }
  catch(IOException i) { }

  return false;
 }


	//{{DECLARE_CONTROLS
	//}}
}
