/*
    Advanced Explorer (c) 1999-2006 by D.G. Sureau

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    webmaster@scriptol.org
    http://www.scriptol.org
*/
package comgr;

import java.awt.image.*;
import java.awt.*;

/*
   Palette is an "active" container for an IndexColorModel.
   Gre un tableau de composantes

   Ne pas oublier d'appeller "setColorModel" aprs changement
   de couleur (dont rotation)
   pour mettre  jour le ColorModel
*/


public class Palette
{
  protected ColorModel colorModel;

  protected byte a[] = new byte[256];
  protected byte r[] = new byte[256];
  protected byte g[] = new byte[256];
  protected byte b[] = new byte[256];

  int transparent = 7;

  Palette()
   {
    fill();
    setColorModel();
   		//{{INIT_CONTROLS
		//}}
}


  /**
      Construit un colormodel  partir de r,g,b,a
  */
  public void setColorModel()
   {
	colorModel = new IndexColorModel(8, r.length, r, g, b, a);
   }


  public ColorModel getColorModel() { return colorModel; }

  public void fill()
   {
	for (int i = 0; i < a.length; i++)
    {
     a[i] = (byte) 255;
     r[i] = (byte) i;
     g[i] = (byte) i;
     b[i] = (byte) i;
    }

    a[transparent] = (byte) 0;
   }

 /**
    Reconstruit le colorModel  partir des tableaux r,g,b
 */

 public void setTransparent(int transp)
 {
   transparent = transp;
   colorModel = new IndexColorModel(8, r.length, r, g, b, transp);
 }


  public int getRGB(int i)
  {
    return( (a[i] << 24) | (r[i] << 16) | (g[i] << 8) | b[i] );
  }
 /**
    Convertit une entre du tableau en couleur
 */

 public Color getColor(int index)
 {
  int red   = colorModel.getRed(index);
  int green = colorModel.getGreen(index);
  int blue  = colorModel.getBlue(index);
  return new Color(red, green, blue);
 }


 /**
    Convertit le tableau en couleurs relles
 */
 public Color[] getColors()
 {
   int l = r.length;
   Color colors[] = new Color[l];

   for(int i = 0; i < l; i++)
    colors[i] = getColor(i);

  return colors;
 }

 public void changeColor(int index, Color col)
 {
  int rgb = col.getRGB();

  r[index] = (byte) ((rgb >> 16) & 255);
  g[index] = (byte) ((rgb >> 8) & 255);
  b[index] = (byte) (rgb & 255);
 }


/**  Display the palette */

public void display(Graphics g, int x, int y, int sw, int sh)
{
   int i, j;

   int h = sh / 16;
   int w = sw / 16;


   for(j = 0; j < 16; j++)
    for(i = 0; i < 16; i++)
    {
     g.setColor(getColor(j * 16 + i));
 	   g.fillRect(x + i * w , y + j * h, w, h);
    }

   // Cadre
   g.setColor(Color.black);
   g.drawRect(x, y, sw, sh);

  return;
}




 public void rotate(int rotateNo)
  {
	int start;
	int i, oldi, l;
	byte temp;

	start = i = rotateNo;
	l = r.length;

	while(start != 0)
    {
	oldi = i;
	i = (i + rotateNo) % l;

	temp = r[oldi];
	r[oldi] = r[i];
	r[i] = temp;

	temp = g[oldi];
	g[oldi] = g[i];
	g[i] = temp;

	temp = b[oldi];
	b[oldi] = b[i];
	b[i] = temp;

	if (i == start)
      {
		start--;
		i = start;
	  }
	}
  }

 public void rotateRed(int rotateNo)
  {
	int start;
	int i, oldi, l;
	byte temp;

	start = i = rotateNo;
	l = r.length;

	while(start != 0)
    {
	 oldi = i;
	 i = (i + rotateNo) % l;

	 temp = r[oldi];
	 r[oldi] = r[i];
	 r[i] = temp;

	 if (i == start)
      {
		start--;
		i = start;
	  }
	}
  }

 public void rotateGreen(int rotateNo)
  {
	int start;
	int i, oldi, l;
	byte temp;

	start = i = rotateNo;
	l = r.length;

	while(start != 0)
    {
	 oldi = i;
	 i = (i + rotateNo) % l;

	 temp = g[oldi];
	 g[oldi] = g[i];
	 g[i] = temp;

	 if (i == start)
      {
		start--;
		i = start;
	  }
	}

  }

public void rotateBlue(int rotateNo)
 {
 int start;
 int i, oldi, l;
 byte temp;

 start = i = rotateNo;
 l = r.length;

 while(start != 0)
  {
	oldi = i;
	i = (i + rotateNo) % l;

	temp = b[oldi];
	b[oldi] = b[i];
	b[i] = temp;

	if (i == start)
     {
	 start--;
	 i = start;
	 }
  }
 }
	//{{DECLARE_CONTROLS
	//}}
}

