/*
    Advanced Explorer (c) 1999-2006 by D.G. Sureau

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    webmaster@scriptol.org
    http://www.scriptol.org
*/
package comgr;

import java.awt.*;
import java.awt.image.*;
import java.awt.event.*;

/**
   Affiche une palette
   Permet de slectionner une couleur
   de choisir une couleur pour

   C'est une classe statique mais avec des vues multiples
   de tailles variables
*/

public class PalettePanel extends Canvas  implements MouseListener, ActionListener, MouseMotionListener
{

  int SQUAREW = 1;  // Taille d'un pav de couleur
  int SQUAREH = 1;
  static int PAVEW = 16;   // Nombre de pavs de couleur
  static int PAVEH = 16;


  static Point paveloc = null;
  static Point oldloc = null;

  static boolean cursorDrawn = false;


  static Color selectedColor = Color.black;
  static int selectedInt = 0xFF000000;
  static int selectedIndex = 0;

  static Color penColor = Color.black;
  static int   penInt   = 0xFF000000;
  static int   penIndex = 0;

  static Color transparentColor = Color.white;
  static int transparentInt = 0xFFFFFFFF;
  static int transparentIndex = 15;

  static Color replaceColor = Color.black;

  static Color[] palColors = new Color[256];

  PopupMenu mousemenu = new PopupMenu();

  public PalettePanel()
   {
    super();

	this.add(mousemenu);
    addMouseListener(this);
    addMouseMotionListener(this);

    paveloc = new Point(0 ,0);
    oldloc = paveloc;
    cursorDrawn = false;
  };


 static public Rectangle align(int x, int y, int w, int h)
 {

   w = (w - 4) / PAVEW;     // mult largeur
   h = (h - 4) / PAVEH;      //

   w = w * PAVEW + 4 + 6;
   h = h * PAVEH + 4 + 6;

   return new Rectangle(x, y, w, h);
 }


  // Calcule taille pavs
  // Selon nombre hor et vert

  public void magnifyFactor()
  {
    // Taille fentre
    Dimension size = getSize();

    SQUAREW = (size.width) / PAVEW;
    SQUAREH = (size.height) / PAVEH;

  }

  public void changeColor(int r, int g, int b)
  {
   Color color = new Color(r, g, b);
   palColors[selectedIndex] = color;
   display(selectedIndex);
  }

  static public void setColors(Color[] couleurs)
  {
    int l = couleurs.length;
    int i = 0;

    while(i < l)
    {
     palColors[i] = couleurs[i];
     i++;
    }

    while(i < 256)
    {
     palColors[i++] = Color.black;
    }

  }


  // Conversion
  static public int colorToInt(Color col)
  {
   if(col == null) return 0;
   return ((0xFF << 24) | col.getRed() <<  16) | (col.getGreen() << 8) | col.getBlue();
  }

  public int getRGB(int index)
  {
   Color col = palColors[index];
   return ( (0xFF << 24) |
            (col.getRed() <<  16) |
            (col.getGreen() << 8) |
            col.getBlue());
  }

  static public Color intToColor(int i)
  {
    int alpha =  (i >> 24) & 255;
    int red = (i >> 16) & 255;
    int green = (i >> 8) & 255;
    int blue = i & 255;
    return new Color(red, green, blue);
  }


  /**
      Cherche un numro de couleur
      (notamment pour crer une image indexe)
  */
  public static int searchIndex(Color col)
  {
    return searchIndex(colorToInt(col));
  }

  public static int searchIndex(int intColor)
  {
   for(int index = 0; index < 256; index++)
   {
    if(intColor == colorToInt(palColors[index]))
      return index;
   }

   return -1;
  }

  /**
     Ces deux fonctions servent quand on change de panneau
     il faut changer la couleur courante selon l'index
     de slection, quand on affecte cette couleur courante
     et non  quand on place le curseur sur une couleur
     Ceci parceque le curseur est non-static
  */
  public void updateCursor()
   {

     unCursor();
     int y = penIndex / PAVEW;
     int x = penIndex - (y * PAVEW);
     paveloc.x = x;
     paveloc.y = y;
     cursor();

     //setPen(penIndex);
   }


  static public void makeIndex()
  {
    selectedIndex = paveloc.y * PAVEW + paveloc.x;
  }


  public void select(int no)
  {
   //selectedIndex = no;
   unCursor();
   paveloc.y = no / PAVEW;
   paveloc.x = no - (paveloc.y * PAVEW);

   pickColor(); // Dfinit selectedNNN avec setCurrent(no)
   cursor();
  }


  /** Dfinit les couleurs */

  static public void setCurrent()
  {
   selectedInt = colorToInt(selectedColor);
   selectedIndex = searchIndex(selectedInt);
  }

  static public void setCurrent(int no)
  {
    try {
     selectedColor = palColors[no];
     selectedInt = colorToInt(selectedColor);
     selectedIndex = no;;
    }
    catch(ArrayIndexOutOfBoundsException a) { a.printStackTrace(); }

  }


  public void setPen(Color color)
  {
    selectedColor = color;
    selectedInt = colorToInt(color);
    selectedIndex = searchIndex(selectedInt);
    setPen();

    updateCursor();

  }


  static public void setPen()
  {
   penColor = selectedColor;
   penInt   = colorToInt(penColor);
   penIndex = searchIndex(penInt);
  }

  static public void setTransparent()
  {
   setTransparent(selectedColor);
  }

  static public void setTransparent(int c)
  {
   transparentColor = palColors[c];
   transparentInt = colorToInt(transparentColor);
   transparentIndex = c;
  }

  static public void setTransparent(Color color)
  {
   transparentColor = color;
   transparentInt = colorToInt(transparentColor);
   transparentIndex = searchIndex(transparentInt);
  }


  static public void setReplace(Color color)
  {
   replaceColor = color;
  }


  static public void reverse()
  {
   for(int i = 0; i < palColors.length; i++)
   {
    Color c = palColors[i];

    int r = c.getRed();
    int g = c.getGreen();
    int b = c.getBlue();

    palColors[i] = new Color(b, g, r);
   }
  }

  static public void lighten()
  {
   for(int i = 0; i < palColors.length; i++)
   {
    Color c = palColors[i];

    int r = c.getRed();
    int g = c.getGreen();
    int b = c.getBlue();

    palColors[i] = new Color(Math.min(r * 2, 255), Math.min(g * 2, 255), Math.min(b * 2, 255));
   }
  }

  static public void darken()
  {
   for(int i = 0; i < palColors.length; i++)
   {
    Color c = palColors[i];

    int r = c.getRed();
    int g = c.getGreen();
    int b = c.getBlue();

    palColors[i] = new Color(r / 2, g / 2, b / 2);
   }
  }



  // Tableau d'entiers pour les 256 couleurs
  public void display(int no)
  {
   Graphics g = getGraphics();
   if(g != null)
   {
    magnifyFactor();

    int hs = PAVEW * SQUAREH;
    int ws = PAVEH * SQUAREW;
    int y = no / PAVEW;
    int x = no - (y * PAVEW);

    g.setColor(palColors[no]);
    g.fillRect(x * SQUAREW + 1, y * SQUAREH + 1, SQUAREW - 1, SQUAREH - 1);
   }
  }

  public void display()
  {
   int x, y;

   Graphics g = getGraphics();

   magnifyFactor();

   int hs = PAVEW * SQUAREH;
   int ws = PAVEH * SQUAREW;

   //System.out.println("Fill factor " + SQUAREW + " " + SQUAREH);
   if(g == null)
   {
     System.out.println("No graphic context, enable to display palette");
     return;
   }


   int idx = 0;
   for(y = 1; y < hs; y += SQUAREH)
   for(x = 1; x < ws; x += SQUAREW)
   {
     if(idx < 256)
     {
      try
      {
       g.setColor(palColors[idx++]);
       g.fillRect(x, y, SQUAREW - 1, SQUAREH - 1);
      }
      catch(ArrayIndexOutOfBoundsException a)
       { System.out.println("PalettePanel - Out of bounds index=" + (idx-1));}
      catch(NullPointerException n) { }
     }
   }

  }


  /** Retourne une couleur choisie dans la palette */
  static public Color getColor(int no) { return palColors[no];  }


  /** Change la couleur courante  */
  public ColorModel getColorModel()
   {
    ColorModel colorModel;

    byte a[] = new byte[256];
    byte r[] = new byte[256];
    byte g[] = new byte[256];
    byte b[] = new byte[256];

    for(int i = 0; i < 256; i++)
    {
     a[i] = (byte) 0xFF;
     r[i] = (byte) palColors[i].getRed();
     g[i] = (byte) palColors[i].getGreen();
     b[i] = (byte) palColors[i].getBlue();
    }

  	colorModel = new IndexColorModel(8, r.length, r, g, b);

  	return colorModel;
   }

  /**  Met  jour la variable loc */
  public void moveCursor(int x, int y)
   {
     paveloc.x = x / SQUAREW;
     paveloc.y = y / SQUAREH;
   }


  /** Pointe sur une couleur */
  public void drawCursor(Color color)
   {
     int i = oldloc.x;
     int j = oldloc.y;
     int x = i * SQUAREW;
     int y = j * SQUAREH;

     Graphics g = getGraphics();

     g.setColor(color);
     g.drawRect(x , y , SQUAREW, SQUAREH);
   }


   public void cursor()
   {
     oldloc = paveloc;
     drawCursor(Color.white);
     cursorDrawn = true;
   }

   public void unCursor()
   {
     if(cursorDrawn) drawCursor(Color.black);
     cursorDrawn = false;
   }


   /** Retourne une couleur */

   public Color pickColor()
   {
     int i = paveloc.x;
     int j = paveloc.y;

     if((i >= PAVEW) || (j >= PAVEH))
     {
      System.out.println("PickColor error: x " + i + " y " + j + " (Max "+ PAVEW + "x" + PAVEH+ ")" );
     }
     else
     setCurrent(j * PAVEW + i);

     return(selectedColor);
   }


  /** Dans Component, paint ne fait rien */

   public void paint(Graphics g)
    {
      unCursor();    // si dessin, enlve en oldCursor;
      display();
      cursor();
    }


  //------------------------------------ Load Save

  public static String message = "OK";

  static public boolean load(String path, String name)
  {
   int size = Data.getSize(path);
   boolean result = true;

   switch(size)
   {
    case 1024: {
               int buffer[] = Data.iLoad(path);
               for(int i = 0; i < 256; i++)
                palColors[i] = intToColor(buffer[i]);
               }
               break;

    case 768:  {
               byte buffer[] = Data.bLoad(path);
               for(int i = 0; i < 256; i++)
                {
                 int j = i * 3;
                 int r = buffer[j];
                 int g = buffer[j + 1];
                 int b = buffer[j + 2];
                 palColors[i] = new Color(r, g, b);
                }
               }
               break;
    default:   message = "Unknow palette format " + name;
               result = false;
               break;
   }

   return result;
 }


  static public void save(String path, String name)
  {
    int buffer[] = new int[256];

    for(int i = 0; i < 256; i++)
    {
     buffer[i] = colorToInt(palColors[i]);
    }

    Data.iSave(path, buffer);
  }



//----------------------------------------- Mouse



  public void mousePressed(MouseEvent e)
  {
  }


  public void mouseReleased(MouseEvent e)
  {
    e.consume();
    /*
    // Popup trigger event =  right button
    if (e.isPopupTrigger()) sendAction("right");
                       else sendAction("left");
    */
    unCursor();   // efface
    moveCursor(e.getX(), e.getY());  // change paveloc et paveloc
    pickColor();
    cursor();   // place ailleurs
    sendAction("set");
  }


   // The following methods must be included here, due to the fact that
   // we implement MouseListener
    public void mouseDragged(MouseEvent e) { }
    public void mouseClicked(MouseEvent e) { }
    public void mouseEntered(MouseEvent e) {}
    public void mouseExited(MouseEvent e) {}
    public void mouseMoved(MouseEvent e) { }


    // implement actions for each menu item
   public void actionPerformed(ActionEvent e)
    {
       String ac = e.getActionCommand();
    }


   transient ActionListener actionListener;

   /**
     * Adds the specified action listener to receive action events
     * from this button.
     */
   public void addActionListener(ActionListener l)
    {
	 actionListener = AWTEventMulticaster.add(actionListener, l);
    }

   void sendAction(String command)
   {
    if (actionListener != null)
	  actionListener.actionPerformed(new ActionEvent(this, ActionEvent.ACTION_PERFORMED, command));
   }


}


