/*
    Advanced Explorer (c) 1999-2006 by D.G. Sureau

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    webmaster@scriptol.org
    http://www.scriptol.org
*/
package comgr;
/**
    A 3d polygon.
    Compos d'un ensemble de Vertex
    
    p contient les vertex d'orientation et distance
 */

import java.awt.*;

public class Polygon3d 
{
	int spx[] = new int[16];
	int spy[] = new int[16];
	Vertex p[] = new Vertex[16];
	Vertex normal = new Vertex();
	Vertex middle = new Vertex();
	double lightprop;
	int vertcount;
	Color col;
	double minz;
	double maxz;
	Texture texture;

	public Polygon3d() { 		//{{INIT_CONTROLS
		//}}
}	

    /*
       Cre un polygone clone d'un modle
    */   
	public Polygon3d(Polygon3d poly) 
	{
	    // copie le modle en paramtre
		for (int i = 0; i < spx.length; i++) 
		{
			this.spx[i] = poly.spx[i];
			this.spy[i] = poly.spy[i];
			if (poly.p[i] != null)
				this.p[i] = new Vertex(poly.p[i]);
		}
		
		// prend les autres lments
		this.middle = new Vertex(poly.middle);
		this.normal = new Vertex(poly.normal);
		this.lightprop = poly.lightprop;
		this.vertcount = poly.vertcount;
		this.col = new Color(poly.col.getRed(), poly.col.getGreen(), poly.col.getBlue());
		this.minz = poly.minz;
		this.maxz = poly.maxz;
		this.texture = poly.texture;
	}	

//	public void setlightprop(Light l1) {
//		this.lightprop = l1.lightpos.dotproduct(normal) / l1.lightlen;
//
//		if (this.lightprop < 0) {
//			this.lightprop = 0;
//		}
//	}

	public boolean clipback(Plane clip) 
	{
	int i, newi;
	boolean oldin, newin;
	double r;
	Vertex temp = new Vertex();
	Polygon3d newv = new Polygon3d(this);
	double olddist, newdist;
	boolean clipback = false; // assume it will not be visible until shown otherwise

	i = 0;
	newi = 0;
	olddist = this.p[i].dotproduct(clip.normal);
	oldin = (olddist >= clip.dist);


   while (i < this.vertcount) 
   {
 	if (oldin) 
	 {
		newv.p[newi++] = new Vertex(this.p[i % this.vertcount]);
		clipback = true;
	 }
    
	i = i + 1;

	newdist = this.p[i % this.vertcount].dotproduct(clip.normal);
        
	newin = (newdist >= clip.dist);

	if (newin != oldin) 
	{
	 r = (clip.dist - olddist) / (newdist - olddist);
	 temp.x = this.p[i - 1].x + (r * (this.p[i % this.vertcount].x
						- this.p[i - 1].x));
	 temp.y = this.p[i - 1].y 
					+ (r * (this.p[i % this.vertcount].y 
						- this.p[i - 1].y));
	 temp.z = this.p[i - 1].z
					+ (r * (this.p[i % this.vertcount].z
						- this.p[i - 1].z));
	 temp.u = this.p[i - 1].u
					+ (int)(r * (this.p[i % this.vertcount].u
						- this.p[i - 1].u));
	 temp.v = this.p[i - 1].v
					+ (int)(r * (this.p[i % this.vertcount].v
						- this.p[i - 1].v));
	 newv.p[newi++] = new Vertex(temp);
	 clipback = true;
	}

	oldin = newin;
	olddist = newdist;
  }

	newv.vertcount = newi;

	for (i = 0; i < spx.length; i++) 
	{
		this.spx[i] = newv.spx[i];
		this.spy[i] = newv.spy[i];
		if (i < newv.vertcount) 
		{
				this.p[i] = new Vertex(newv.p[i]);
		}
	}
	this.normal = newv.normal;
	this.lightprop = newv.lightprop;
	this.vertcount = newv.vertcount;
	this.col = new Color(newv.col.getRed(), newv.col.getGreen(), newv.col.getBlue());

	double mx = 0.0, my = 0.0;

	for (i = 0; i < newi; i++) 
	 {
		if (newv.p[i].z < newv.minz)
				newv.minz = newv.p[i].z;
		if (newv.p[i].z > newv.maxz)
				newv.maxz = newv.p[i].z;
		mx += newv.p[i].x;
		my += newv.p[i].y;
	 }
	newv.middle = new Vertex(
		mx / (double)newv.vertcount,
		my / (double)newv.vertcount,
		((double)minz + (double)maxz) / 2.0
		);
	this.minz = newv.minz;
	this.maxz = newv.maxz;
	this.middle = newv.middle;

	return clipback;
  }

	public void rotate(int a, int b, int c) 
	{
		rotate(new Orient(a, b, c));
	}

	public void rotate(Orient o1) 
	{
		for (int j = 0; j < vertcount; j++) 
		{
			p[j].rotate(o1);
		}
		normal.rotate(o1);
	}

	public void translate(Vertex disp1) 
	{
		for (int j = 0; j < vertcount; j++) 
		{
			p[j].x = p[j].x + disp1.x;
			p[j].y = p[j].y + disp1.y;
			p[j].z = p[j].z + disp1.z;
		}
	}



	public void paint(Virtual3d g3d) 
	{
		g3d.fillPoly(p, vertcount, normal, texture);
	}
	
	
	
	public String toString() 
	{
		String s = "";

		for (int i = 0; i < vertcount; i++)
			s = s + p[i];

		return s;
	}

	
	//{{DECLARE_CONTROLS
	//}}
}
