/*
    Advanced Explorer (c) 1999-2006 by D.G. Sureau

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    webmaster@scriptol.org
    http://www.scriptol.org
*/
package comgr;

import java.awt.*;
import java.awt.image.*;
import java.io.*;

/**
    Define a "Raw" image

    Le transparent peut tre lu dans le header
    ou dans la palette indexe.
*/

public class RawEdit extends Raw
{

  /** Constructor with default value and empty buffer */
  public RawEdit()
  {
    super();

}

  /** Construct from an array of pixels */

  public RawEdit(int [] code, int width, int height)
  {
    super(code, width, height);
  }

  public RawEdit(Image image)
  {
   this();
   capture(image);
  }

  /**
      Direction byte false = from top to bottom
                byte true =  from bottom to top

      Default it top to bottom
  */

  public void setInversed(boolean d) { bottomToTop = (d == true) ? BTRUE: BFALSE; }
  public void setIndexed(boolean m) { colorModel = (m == true) ? INDEXED : DIRECT; }
  public void setCompression(int compressMode)
  {
    if(compressMode != RLE) compressMode = UNCOMPRESSED;
    compression = signed(compressMode);
  }

  public void setPalette(int n) { palette = signed(n); }
  public void setTransparent(int t) {  transparent = signed(t); }


  /**
     Compress
  */
  static public int[] RLECompress(int code[])
  {
    int buffer[] = new int[code.length];

    if(buffer.length > code.length) return null;
    return buffer;
  }

  public int RLE(int rawImage[])
  {
    int buffer[] = RLECompress(rawImage);

    if(buffer == null)
    {
     code = rawImage;
     setCompression(UNCOMPRESSED);
     return 0;
    }

    code = buffer;
    setCompression(RLE);
    return code.length;

  }


 /**
    Retrieve the image as Raw
    Convert this to raw
 */

  Raw getRaw()
  {
   return new Raw(code, width, height);
  }

  /**
     Retrieve the image as an array of pixels
  */

  public int[] getPixels()
  {
    switch(compression)
    {
     case RLE: return UnRLE();
     default:  return code;
    }
  }




 /**
     Capture l'image pour la sauvegarder
 */


 void capture(Image image)
 {
   capture(image, new PalettePanel());
 }

 void capture(Image image, PalettePanel panel)
 {

  width  = (short) image.getWidth(null);
  height = (short) image.getHeight(null);

  code = new int[width * height];

  PixelGrabber pg = new PixelGrabber(image, 0, 0, width, height, code, 0, width);
  pg.setColorModel(panel.getColorModel());

  try { pg.grabPixels(); }  catch(InterruptedException i) { }

  if ((pg.getStatus() & ImageObserver.ABORT) != 0)
  {
	System.err.println("Icon capture aborted or errored");
	return;
  }

  // Convertit les couleurs directes en index
  // Si la couleur n'est pas trouve dans la palette
  // On place le code du transparent
  int counter = 0;

  if(colorModel == INDEXED)
  for(int i = 0; i < code.length; i++)
  {
   int k = code[i];
   int l = unsigned(transparent);
   int r;
   int g;
   int b;

   int or = (k >> 16) & 255;
   int og = (k >> 8) & 255;
   int ob = (k & 255);

   int oprox = 255 * 3;

   for(int j = 0; j < 256; j++)
   {
     int rgb = panel.getRGB(j);
     if(k == rgb) { l = j; counter++; continue; }

     r = (rgb >> 16) & 255;
     g = (rgb >> 8) & 255;
     b = (rgb & 255);

     int nprox = Math.abs(r - or) + Math.abs(g - og) + Math.abs(b - ob);
     if(nprox < oprox)
      {
       oprox = nprox;
       l = j;
      }

   }
   code[i] = l;  // reoit l'index ou transparent

  }
  //System.out.println(counter + " found");
}




 boolean save(String path)
 {
   try
    {
	 DataOutputStream p = new DataOutputStream(new FileOutputStream(path));
     byte buffer[] = getBytes();
     int size = buffer.length;

     for(int i = 0; i < size; i++) p.writeByte(buffer[i]);

     p.flush();
     p.close();
     return true;
  }
  catch(IOException i) { }

  return false;
 }

 /**
    Convertit en chaine
 */
 public String toString()
 {
   String s;

   s = "";
   s = s + "Width: " + width + "\n";
   s = s + "Height: " + height + "\n";
   String direction;
   if(bottomToTop == BTRUE) direction = "Bottom to top";
   else direction = "Top to bottom";
   s = s + "Direction: " + direction  + "\n";

   String c;
   switch(compression)
   {
    case UNCOMPRESSED: c = "None"; break;
    case RLE: c = "Run-Length Encoding"; break;
    default: c = "unknow"; break;
   }

   s = s + "Compression: " + c + "\n";

   String codehead;
   if(colorModel == INDEXED)
   {
    c = "Indexed";
    codehead = "(decimal)";
   }
   else
   {
    c = "Direct Alpha-RGB";
    codehead = "(hexadecimal)";
   }
   s = s + "Color model: " + c + "\n";
   s = s + "Palette #" + palette + "\n";
   s = s + "Version: " + version + "\n";

   s = s + "Code " + codehead + ":" + "\n";
   int j = 0;
   for(int i = 0; i < code.length; i++)
   {
    if(colorModel == INDEXED)
      s = s + code[i] + " ";
    else
      s = s + Integer.toHexString(code[i]) + " ";

    if(++j == 10) { j = 0; s = s + "\n"; }
   }

   return s;
 }

	//{{DECLARE_CONTROLS
	//}}
}
