/*
    Advanced Explorer (c) 1999-2006 by D.G. Sureau

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    webmaster@scriptol.org
    http://www.scriptol.org
*/
package comgr;

import java.io.*;
import java.awt.*;


public class SprFile
{

 String name = null;
 int SPRMAX;                  /* Number of sprites in table */

 /**
     Organisation:
     un tableau de bytes contenant:
     - une liste des positions des sprites
     - le code de chacun l'un aprs l'autre
     - pour chacun, 6 octets d'en-tte puis le code du dessin
 */

 int SPVECTOR[];
 byte SPCODE[];

 public SprFile() { };

 public SprFile(String n) { name = n; }



 /**
    Donne la position du code d'un sprite dans le tableau
 */
 public int GETSPADR(int no)
 {
   if(no >= SPVECTOR.length) no = 1;

   try { return SPVECTOR[no]; }
   catch(NullPointerException n)
   {
    System.out.println(no + " out of bound, SPVECTOR ("+ SPVECTOR.length +")");
   }
   return 0;
 }

/* Function to get sprite type code */

 int getCode(int n)
  {
   int adr = GETSPADR(n);
   return (int) SPCODE[adr + 4];
  }


 int getMax() { return SPVECTOR[0]; }

/* Function to add or replace extension of filenames */

 String changExt(String name, String ext)
 {
   int i = name.lastIndexOf(".");

   if(i > 0)
    return name.substring(0, i) + ext;
   return name + ext;
 }

/* Error message for loading function */

 static String MESSAGE[] =
 {
 "",
 "File no found",
 "Enable to save",
 };



 static void dispError(String name, int mess)
 {
    if(mess == 0) System.out.println(name);
    else
    {
    System.out.println( MESSAGE[mess] + " " + name);
    System.exit(0);
    }
 }



 static int intConvert(int origin)
 {
  int un = origin >> 24;
  int deux = (origin >> 16) & 255;
  int trois = (origin >> 8) & 255;
  int quatre = origin & 255;

  return (quatre << 24) + (trois << 16) + (deux << 8) + un;
 }

 /**
    Charge un fichier de sprites dans SPVECTOR/SPCODE
 */   
 int load(String path)
  {
    int toload = 0;
    int loaded = 0;

    try
    {
	 DataInputStream fiS = new DataInputStream(new FileInputStream(path));

     int available = fiS.available();
     System.out.println("available= " + available);

     try {
      toload = intConvert(fiS.readInt());
      System.out.println("size to load= " + toload);
      SPVECTOR = new int [toload + 2];
      SPVECTOR[0] = toload;
     } catch(IOException e) { }

     int i = 0;
     while(i <= toload)  // toujours une adresse de plus
     {
      try
      {
       SPVECTOR[i + 1] = intConvert(fiS.readInt());
       System.out.println(SPVECTOR[i + 1]);
       loaded++;
      }
      catch(IOException ef) { dispError("Error in vector", 0); }
      i++;
    }

     available = fiS.available();
     SPCODE = new byte[available];

     int notAtEnd = 1;
     while(notAtEnd > 0)
      {
    	 notAtEnd = fiS.read(SPCODE);
      }

     fiS.close();
   } catch (IOException e) { dispError(path, 1); }

    System.out.println("loaded " + loaded);
    System.out.println("SPVECTOR len (6x4=24)" + SPVECTOR.length);
    System.out.println("SPCODE len " + SPCODE.length);
    SPRMAX = loaded;
    return loaded;
   }


 int load()
  {
    return  load(name);
  }



 void debug(String s) { System.out.println(s); }

/**
  Dimensions d'un sprite
 */

 public Dimension spSize(int ns)
 {
  int p;
  int mode;
  int w, h;

  p = GETSPADR(ns);

  //debug("adr = " + p);
  mode = getCode(ns);

  //debug("code= " + mode);
  h = (int) SPCODE[p]     + (int) SPCODE[p + 1] * 256;
  w = (int) SPCODE[p + 2] + (int) SPCODE[p + 3] * 256;

  //debug(h + "x" + w);

  switch(mode)
  {
   case 0:
   case 6:
   case 3:
   case 2:
  	        w = w * 8;
	          break;
   case 1:
   case 4:
   case 7:
   case 5:

	          break;
   default: break;
  }

  return new Dimension(w, h);
 }

}
