/*
    Advanced Explorer (c) 1999-2006 by D.G. Sureau

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    webmaster@scriptol.org
    http://www.scriptol.org
*/
package comgr;

import java.awt.*;
import java.lang.Math.*;
import java.awt.image.*;


/**
    Dessine sur un cran virtuel
    On peut donner un tableau de pixels en paramtres
    On dessine de bas en haut

    Retourne une image.
    Cette image n'a pas de contexte graphique.
*/

public class VirtualScreen
{

 int screen[];
 MemoryImageSource mis;
 int width = 0;
 int height = 0;
 int size = 0;
 int COLOR = 0;

 int SCREEN_WIDTH = 640;
 int SCREEN_HEIGHT = 400;

 static PaletteShaded palette;
 static Toolkit toolkit = null;

 public VirtualScreen()
 {
   toolkit = Toolkit.getDefaultToolkit();
   Dimension d = toolkit.getScreenSize();
   SCREEN_WIDTH = d.width;
   SCREEN_HEIGHT = d.height;
   palette = new PaletteShaded();

 		//{{INIT_CONTROLS
		//}}
}

 public VirtualScreen(int w, int h)
 {
  this();
  setSize(w, h);
 }

 public VirtualScreen(int [] iArray, int w, int h)
 {
  this();

  setScreen(iArray, w, h);
 }


 public boolean error(String s)
 {
  System.out.println(s);
  return false;
 }

 /** Dfinit la taille */

 public void setSize(int w, int h)
 {
  screen = new int[w * h];
  width  = w;
  height = h;
  size   = w * h;
 }


/**
   Change/set the array
*/

 public void setScreen(int[] newscreen, int w, int h)
 {
  screen = newscreen;
  width = w;
  height = h;
  size  = w * h;
 }

 /** Donne la taille de l'cran */
 public int getSize() { return width * height; }

 Dimension getDimensions() { return new Dimension(width, height); }


 public void setTexels(int[] t, int ow, int oh)
 {
  int i = 0, j = 0;
  int idx = 0;
  try
  {
   for(j = 0; j < oh; j++)
   for(i = 0; i < ow; i++)
   {
    screen[j * width + i] = t[idx++];
   }
  }
  catch(ArrayIndexOutOfBoundsException a)
   { System.out.println("Virtual setTextels " + i + "x" + j); }


 }
 /**
    Copy the screen on another, dest may be larger or smaller!
 */
 public void copy(VirtualScreen dst)
 {
  int dw = Math.min(dst.width, width);
  int dh = Math.min(dst.height, height);
  int i = 0, j = 0;

  try
  {
   for(j = 0; j < dh; j++)
   for(i = 0; i < dw; i++)
   {
    dst.screen[j * dst.width + i] = screen[j * width + i];
   }
  }
  catch(ArrayIndexOutOfBoundsException a)
   { System.out.println("Virtual copy " + i + "x" + j); }

 }


 public void copyArea(VirtualDirect dst, int x, int y, int w, int h, int dx, int dy)
 {
  int i = 0, j = 0;
  int noff, nx, ny;

  try
  {
   for(j = 0; j < h; j++)
   {
    ny = y + dy + j;
    if(ny < height)
     for(i = 0; i < w; i++)
     {
      nx = x + dx + i;
      noff = ny * width + nx;
      if (nx < width)
       if (noff < size)
        dst.screen[noff] = screen[(y + j) * width + x + i];
     }
   }
  }
  catch(ArrayIndexOutOfBoundsException a)
   { System.out.println("Virtual copy " + i + "x" + j); }

 }


/**
    Efface le fond
*/

void clear(int color)
{
 int i = getSize() - 1;

 while(i >= 0); screen[i--] = color;
}

/**
   Set the default/current color
*/
void setColor(int c) {  COLOR = c; };


/** Set pixel at (x, y) with color c */

public void pixel(int x, int y)
 {
  if(x < 0) return;
  if(y < 0) return;
  if(x >= width) return;
  if(y >= height) return;

  try
  {
   screen[((height - 1) - y) * width + x] = COLOR;
  }
  catch(ArrayIndexOutOfBoundsException e)
  {
   //int index = ((height - 1) - y) * width + x;
   error("Pixel x:" + x + "/" + width + " y:" + y + "/" + height);
  }

 }

public void pixel(int x, int y, int color) { COLOR = color; pixel(x, y); }


 /**
    Lit un pixel
 */
 int getPixel(int x, int y)
 {
  if(x < 0) return 0;
  if(y < 0) return 0;
  if(x >= width) return 0;
  if(y >= height) return 0;

  try
  {
   return screen[((height - 1) - y) * width + x];
  }
  catch(ArrayIndexOutOfBoundsException a)
  {
   //error("y=" + y);
   error("getPixel error - x:" + x + "/" + width + " y:" + y + "/" + height);
  }

  return 0;
 }


/**
   Draws a line from x,y to x2,y2 in color c
*/

void tbLine(int x, int y, int x2, int y2)
{
 line(x, height - y, x2, height - y2);
}

void line(int x, int y, int x2, int y2, int c)
{
  COLOR = c;
  line(x, y, x2, y2);
}

void line(int x, int y, int x2, int y2)
{
 boolean steep = false;

 int dx = Math.abs(x2 - x);
 int sx = ((x2 - x) > 0) ? 1 : -1;
 int dy = Math.abs(y2 - y);
 int sy = ((y2 - y) > 0) ? 1 : -1;

 if(dy > dx)
  {
     steep = true;
     x  ^= y;    y ^= x;    x ^= y; // swap x and y
     dx ^= dy;  dy ^= dx;  dx ^= dy;// swap dx and dy
     sx ^= sy;  sy ^= sx;  sx ^= sy;// swap sx and sy
  }

 int e = 2 * dy - dx;

 for(int i = 0;i < dx;i++)
   {
      if(steep) pixel(y, x);
          else  pixel(x, y);

      while(e >= 0)
       {
         y += sy;
         e -= 2 * dx;
       }
      x += sx;
      e += 2 * dy;
    }
  pixel(x2, y2);
}

/*
   Draws a circle at (xc, yc) with radius r in color c

   The scaling factor of (SCREEN_WIDTH / SCREEN_HEIGHT)
   is used when updating d.
   This makes round circles.  If you want ellipses, you can
   modify that ratio.
*/

void circle(int xc, int yc, int r, int c)
{
  COLOR = c;
  circle(xc, yc, r);
}

void circle(int xc, int yc, int r)
 {
  int x = 0;
  int y = r;
  int d = 2 * (1 - r);

  while(y > 0)
   {
    pixel(xc + x, yc + y);
    pixel(xc + x, yc - y);
    pixel(xc - x, yc + y);
    pixel(xc - x, yc - y);

    if(d + y > 0)
     {
      y -= 1;
      d -= (2 * y * SCREEN_WIDTH / SCREEN_HEIGHT) - 1;
     }

    if(x > d)
     {
      x += 1;
      d += (2 * x) + 1;
     }
   }

 }


void ellipse(int xc, int yc, int r, int r2, int c)
{
  COLOR = c;
  ellipse(xc, yc, r, r2);
}

void ellipse(int xc, int yc, int r2, int r)
 {
  int x = 0;
  int y = r;
  int d = 2 * (1 - r);

  while(y > 0)
   {
    pixel(xc + x, yc + y);
    pixel(xc + x, yc - y);
    pixel(xc - x, yc + y);
    pixel(xc - x, yc - y);

    try
    {
     if(d + y > 0)
      {
      y -= 1;
      d -= (2 * y * r2) - 1;
      }

     if(x > d)
      {
      x += 1;
      d += (2 * x) + 1;
      }
    }
    catch(Exception e) { }
   }

 }



 /** Tracs */

 void hLine(int x, int y, int xr, int c) { COLOR = c; hLine(x, y, xr); }
 void hLine(int x, int y, int r)
 {
  if(x > r) { int t = x; x = r; r = t; }
   while(x <= r) pixel(x++, y);
 }

 void vLine(int x, int y, int y2, int c) { COLOR = c; vLine(x, y, y2); }

 void vLine(int x, int y, int y2)
 {
  if(y2 < y) { int temp = y; y = y2; y2 = temp; }
  while(y <= y2)  pixel(x, y++);
 }


 /**
    Rectangle vide et plein
 */

 void rectangle(int x, int y, int x2, int y2, int col)
 {
  COLOR = col;
  rectangle(x, y, x2, y2);
 }

 void rectangle(int x, int y, int x2, int y2)
 {
  hLine(x,  y,  x2);
  hLine(x,  y2, x2);
  vLine(x,  y,  y2);
  vLine(x2, y,  y2);
 }

 void fillRect(int x, int b, int r, int t, int col)
 {
  COLOR = col;  fillRect(x, b, r, t);
 }


 void fillRect(int x, int b, int r, int t)
 {
  if(t < b) { int temp = b; b = t; t = temp; }

  for(int y = b; y <= t; y++)  hLine(x, y, r);
 }

 /*
    Abstract function to implement with VirtualDirect
    Return inverted color
 */
 int invert(int col) { return col; }
 void invert(int x, int y)
 {
   pixel(x, y, invert(getPixel(x, y)));
 }

 void outline(int x, int y, int w, int h)
 {
  int i = x;
  int ww = w;

  while(ww-- > 0)  // horizontal
   {
    invert(i, y); // left
    invert(i, y + h - 1); // right
    i+= 2;
   }


  while(h-- > 0)
  {
    invert(x, y);
    invert(x + w - 1, y);
    y+= 2;
  }

 }

/**
    Fill
*/

int OLDFILL;

void hlinecol(int l, int h, int r)
{
 while(l <= r) pixel(l++, h);
}

// Note: la valeur 0 est une couleur impossible
// noir vaut 0xFF000000

private int getScan(int x, int y)
{
  if(x < 0) return 0;
  if(x >= width) return 0;
  if(y < 0) return 0;
  if(y >= height) return 0;
  return getPixel(x, y);

}

int scanleft(int x, int y)
{
 int c;

 do
 {
   c = getScan(--x, y);
 }
 while(( c == OLDFILL ) && ( x >= 0));

 return(++x);
}

int scanright(int x, int y)
{
 int c;

 do
 {
   c = getScan(++x, y);
 }
 while((c == OLDFILL ) && (x < width));

 return(--x);
}

int fillcounter;

void linefill(int x, int y, int d)
{
 int xlf, xrg;

 if(--fillcounter < 1) return;
 if( (y < 0) || ( y >= height)) return;

 xlf = scanleft( x, y);
 xrg = scanright(x, y);
 hlinecol(xlf, y, xrg);

 for( x = xlf; x <= xrg; x++)
  if(getScan(x, y - d) == OLDFILL) linefill(x, y - d, -d);

 for( x = xlf; x <= xrg; x++)
  if(getScan(x, y + d) == OLDFILL) linefill(x, y + d, d);

}

void fill(int x, int y)
{
 //error("vscreen fill");
 OLDFILL = getPixel(x, y);

 if(OLDFILL == COLOR) return;
 fillcounter = 220;

 //error("fill " + OLDFILL + " new " + COLOR);
 linefill(x, y, -1);
 //error("fill passed");
}



/**
    Spray
*/

int rnd(int size)
{
 return (int) (Math.random() * size);
}

void spray(int x, int y)
{
 for(int i = 1; i < 10; i++)
 {
  pixel(x + rnd(i) - 1,y + rnd(i) - 1);
  pixel(x - rnd(i) + 1,y - rnd(i) + 1);
  pixel(x + rnd(i) - 1,y - rnd(i) + 1);
  pixel(x - rnd(i) + 1,y + rnd(i) - 1);
 }
}


 /**
     Shade

     oldcolor est un entier RGB
     row un numro de ligne shaded
     colornum un numro de couleur sur une ligne
 */

void shade(int oldcolor, int sens, PalettePanel palette)
{
 int j, i;
 int stt;
 int colornum;

 int rownum = palette.paveloc.y;;
 int basenum = rownum * 16;


 if(sens > 0) stt = 0;
       else   stt = 15;

 for(j = 0; j < height; j++)
 {


  colornum  = stt + sens;
  if(sens > 0) colornum = Math.min(colornum,15);
      	  else colornum = Math.max(colornum,0);

  for(i = 0; i < width; i++)
  {
   if(getPixel(i, j) == oldcolor)
    {
     COLOR = palette.getRGB(colornum + basenum);
     pixel(i, j);
     colornum +=  sens;
     if(sens > 0) colornum = Math.min(colornum, 15);
	         else colornum = Math.max(colornum, 0);
    }
   else colornum = stt;
  }
 }
 return;
}



 /** Mirror */

 void mirrorH()
 {
  int lineptr;
  int rig;
  int middle = width / 2;

  for(int j = 0; j < height; j++)
  {
   lineptr = j * width;
   rig = lineptr + width - 1;

   for(int i = middle - 1; i >= 0; i--)
   {
     screen[rig - i] = screen[lineptr + i];
   }
  }
 }


 void mirrorV()
 {

  int dstptr;
  int srcptr;

  int middle = height / 2;

  for(int j = 0; j < middle; j++)
  {
   srcptr = j * width;
   dstptr = (height - 1 - j) * width;
   for(int i = 0; i < width; i++)
   {
    screen[dstptr + i] = screen[srcptr + i];
   }
  }

 }


 void flipV()
 {

  int dstptr;
  int srcptr;

  int middle = height / 2;
  int temp;

  for(int j = 0; j < middle; j++)
  {
   srcptr = j * width;
   dstptr = (height - 1 - j) * width;
   for(int i = 0; i < width; i++)
   {
    temp = screen[dstptr + i];
    screen[dstptr + i] = screen[srcptr + i];
    screen[srcptr + i] = temp;
   }
  }

 }


 void flipH()
 {
  int lineptr;
  int rig;
  int middle = width / 2;

  for(int j = 0; j < height; j++)
  {
   lineptr = j * width;
   rig = lineptr + width - 1;

   for(int i = middle - 1; i >= 0; i--)
   {
     int temp = screen[lineptr + i];
     screen[lineptr + i] = screen[rig - i];
     screen[rig - i] = temp;
   }
  }

 }



/** Edition */

void deleteH(int x)
{
  int i, j;
  int k = width - 1;
  int black = 0xFF000000;

  for(j = 0; j < height; j++)
  {
   i = x;
   while(i < k)
    {
     pixel(i, j, getPixel(i + 1, j));
     i++;
    }

   pixel(i, j, black);
  }
}


void insertH(int x)
{
 int i,j;

 int k = width - 1;

 for(j = 0; j < height; j++)
  {
   for(i = k; i > x; i--)
    {
     pixel(i,j, getPixel(i - 1, j));
    }
  }
}


void deleteV(int y)
{
  int i,j;
  int black = 0xFF000000;

  for(j = y; j < (height - 1); j++)
   for(i = 0; i < width; i++)
     pixel(i, j, getPixel(i, j + 1));

  for(i = 0; i < width; i++)
     pixel(i, j, black);
}


void insertV(int y)
{
  int i,j;

  for(j = height - 1;j > y; j--)
   for(i = 0; i < width; i++)
     pixel(i, j, getPixel(i, j - 1));
}



/** Remplacer une couleur */
void colorToColor(int prev, int next)
{
 for(int i = 0; i < getSize(); i++)
  if(screen[i] == prev) screen[i] = next;
}


/**
   Scrolling
  */

void scrollLeft()
 {
   int i, j;
   int s;

   int xsr = width - 1;

   for(j = 0; j < height; j++)
   {
    int ptr = j * width;
    s = screen[ptr];

    for(i = 0; i < xsr; i++)
    {
      screen[ptr + i] = screen[ptr + i + 1];
    }
    screen[ptr + xsr]= s;
  }

 }

 void scrollDown()
 {
   int i, j;
   int s;
   int yst = height - 1;

   for( i = 0; i < width; i++)
   {
    s = screen[yst * width + i];
    for( j = yst; j > 0; j--)
     screen[j * width + i] = screen[(j-1) * width + i];
    screen[i] = s;
  }

 }

 void scrollRight()
 {
  int i, j;
  int s;
  int xsr = width - 1;

  for(j = 0; j < height; j++)
   {
     int ptr = j * width;

     s = screen[ptr + xsr];
     for(i = xsr; i > 0; i--)
       screen[ptr + i] = screen[ptr + i - 1];

     screen[ptr]= s;
   }

 }

 void scrollUp()
 {
   int i, j;
   int s;

   int yst = height - 1;

   for( i = 0; i < width; i++)
   {
    s = screen[i];
    for( j = 0; j < yst; j++)
     screen[j * width + i] = screen[(j+1) * width + i];

    screen[yst * width + i] = s;
  }

 }



 /** Cre et retourne l'image */

 Image getImage()
 {
  mis = new MemoryImageSource(
    width, height, palette.getColorModel(), screen, 0, width);

  return toolkit.createImage(mis);

 }

 /**
     Ajouter un client
     C'est un objet avec l'interface ImageConsumer
 */
 public void addConsumer(ImageConsumer consumer)
 {
  mis.addConsumer(consumer);
 }

	//{{DECLARE_CONTROLS
	//}}
}
