/*
    Advanced Explorer (c) 1999-2006 by D.G. Sureau

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    webmaster@scriptol.org
    http://www.scriptol.org
*/
package comgr;

import java.io.*;
import java.awt.image.*;
import java.awt.*;

/**
    Standard Xbm file management
    by D.G. Sureau - May 2000
  */

public class Xbm
{

 int width = 32;       // width of the icon, 16, 32, and 64.
 int height = 32;      // height
 int transparentIndex = 7;
 int transparentInt;
 int rgbPixels[];    // Entiers de reprsentation RGB de couleur
 int packedPixels[];    // Version compacte

 public Xbm() { 		//{{INIT_CONTROLS
		//}}
}


 public Xbm(int t)
 {
  this();
  setTransparent(t);
 }   



 public void setTransparent(int t)
 {
  transparentIndex = t;
 }
 
/**
   Transforme l'image en tableau d'entiers RGB
   Dfinit la valeur argb de transparentInt 
*/

 void capture(Image image, PalettePanel panel)
 {
  width  = image.getWidth(null);
  height = image.getHeight(null);

  transparentInt = panel.getRGB(transparentIndex);
  
  rgbPixels = new int[width * height];

  PixelGrabber pg = new PixelGrabber(image, 0, 0, width, height, rgbPixels, 0, width);
  pg.setColorModel(panel.getColorModel());

  try { pg.grabPixels(); }  catch(InterruptedException i) { }

  if ((pg.getStatus() & ImageObserver.ABORT) != 0)
  {
	System.err.println("XBM capture aborted or errored");
	return;
  }


  compress();
}


void compress()
{
 int m;
 int maskadr = 0;
 int lmask   = 0;
 int maskval = 0;

 packedPixels = new int[rgbPixels.length / 8];
 //System.out.println("" + transparentInt);   
 
 for(int j = 0; j < height; j++)
  for(int k = 0; k < width; k++)
   {
    int idx = j * width + k;

    // On construit un octet de deux pixels
    try 
    { 

      if(rgbPixels[idx] == transparentInt)  m = 0;
      else  m = 128;
    
      maskval = (maskval >> 1) | m;
      lmask++;
      if(lmask == 8)
      {
       packedPixels[maskadr++] = maskval;
       //System.out.println("mask " + maskval + " idx " + maskadr + " " + width + "x" + height );
       maskval = 0;
       lmask = 0;
      } 
     }
     catch(ArrayIndexOutOfBoundsException a) 
      {
        a.printStackTrace(); 
        System.out.println("index " + idx + " " + width + "x" + height);
      }
   } // boucles
}



 //----------------- Sauvegarde



 String hexa(int val)
 {
  String arr = "0123456789abcdef";  
    
  int high = val / 16;
  int low  = val % 16;
  char[] c = new char[5];
  c[0] = '0';
  c[1] = 'x';
  if(high > 15) c[2] = '?'; 
           else c[2] = arr.charAt(high);
  if(low > 15)  c[3] = '?';
           else c[3] = arr.charAt(low);
  c[4] = ',';
  return new String(c);
 }   
 

 boolean save(String dir, String name)
 {
    int pwidth = width / 8;
    String eol = System.getProperty("line.separator");
    
   	StringBuffer sb = new StringBuffer("// XBM 2 colors image for web browsers" + eol);
    sb.append("// Pixel format is 8 pixels per byte" + eol);
    sb.append("#define width " + width + eol);
    sb.append("#define height " + height + eol);    
    sb.append("static char " + Path.getNode(name) + "[] = { " + eol); 
    
    int idx = 0;
    for(int y = 0; y < height; y++)
    {
     int mw = 0;   
     for(int x = 0; x < pwidth; x++)
     {
       int val = packedPixels[idx++]; 
       sb.append(hexa(val));
       // cut long line to 80 if more than 100 a line
       if ((++mw > 16) && (pwidth > 20))  sb.append(eol);
     } 
     sb.append(eol);
    }  
    
    sb.append("};" + eol);

    Disk.save(dir, name, sb.toString());
    return true;
 }


	//{{DECLARE_CONTROLS
	//}}
} // class
