/*
    Advanced Explorer (c) 1999-2006 by D.G. Sureau

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    webmaster@scriptol.org
    http://www.scriptol.org
*/
package common;

import java.awt.*;
import java.awt.event.*;
import java.io.File;
import common.*;

public class ArchivePanel extends Panel implements ActionListener, ItemListener
{
  // Archiver
  final short ARC_NONE = 0;
  final short ARC_SINGLE = 1;
  final short ARC_TAGGED = 2;
  final short ARC_FOLDER = 3;
  short ArchiveMode;

  Frame frame;

  TextField fieldPath;
  TextField fieldName;
  DButton buttonMake;
  Label labeltitle;
  Label labelArc;
  Label labelArcName;
  Label labelArcPath;

  // Setup
  DCheckbox checkRecurse;
  DCheckbox checkInclude;
//  DCheckbox checkStore;



 String list[] = null;
 String srcdir = null;
 String source = null;

 public ArchivePanel(Frame f)
 {

  super();
  setVisible(false);
  setLayout(new GridLayout());

  frame = f;

  int fieldSize = 24;
  if(!OS.isWindows()) fieldSize = 30;

  DBPanel panelInner = new DBPanel(DBPanel.BEVEL_LOWERED);
  panelInner.setBounds(0, 0, 450,320);
  add(panelInner);

  Font font = new Font("Dialog", Font.BOLD, 16);

  labeltitle = new Label();
  labeltitle.setBounds(16,32,320,24);
  labeltitle.setFont(font);
  panelInner.add(labeltitle);

  labelArcPath = new Label();
  labelArcPath.setBounds(16,60,420,24);
  labelArcPath.setFont(font);
  panelInner.add(labelArcPath);

  labelArc = new Label("Path of the archive:");
  labelArc.setBounds(16,103,378,20);
  panelInner.add(labelArc);

  fieldPath = new TextField();
  fieldPath.setBounds(16,127,360,fieldSize);
  panelInner.add(fieldPath);

  labelArcName = new Label("Name of the archive:");
  labelArcName.setBounds(16, 160, 158, 20);
  panelInner.add(labelArcName);

  fieldName = new TextField();
  fieldName.setBounds(16, 184, 256, fieldSize);
  panelInner.add(fieldName);

  checkRecurse = new DCheckbox("Recurse sub-directories", Preferences.ArcRecurse);
  checkRecurse.setBounds(32, 248, 200, 24);
  checkRecurse.addItemListener(this);
  panelInner.add(checkRecurse);

  checkInclude = new DCheckbox("Include full paths", Preferences.ArcInclude);
  checkInclude.setBounds(32, 272, 200, 24);
  checkInclude.addItemListener(this);
  panelInner.add(checkInclude);

  buttonMake = new DButton("Make", true);
  buttonMake.setBounds(200, 320,120,40);
  buttonMake.setTip("Make the archive with name and path above");
  buttonMake.addActionListener(this);
  buttonMake.setEnabled(false);
  panelInner.add(buttonMake);

}




  /** Quel genre d'archive */
//  void setTitle(String title) { labelArcMode.setText(title); }
  /** Quels fichiers archivs */
  void setAction(String title)
  {
   labeltitle.setText(title);
  }

  void setSource(String title, String fpath)
  {
    setAction(title);
    labelArcPath.setText(fpath);
    source = fpath;
  }
  /** Chemin de l'archive */
  void setPath(String dpath) { fieldPath.setText(dpath); }
  /** Nom de l'archive */
  void setArchive(String name) { fieldName.setText(name); }
  /** Bouton enabled quand les champs sont remplis */
  void setList(String l[]) { list = l; }
  void setButtonEnabled(boolean f) { buttonMake.setEnabled(f); }


  public String getPath()    { return fieldPath.getText(); }
  public String getArchive() { return fieldName.getText(); }
  public String getSource()  { return source; }
  public String[] getList()  { return list; }

  String arcname = null;

  public void init(String source, String fics[],  String dpath)
  {
    String title = null;
    String fpath = null;
    srcdir = source;

    int number = fics.length;

    ArchiveMode = ARC_NONE;

    if(arcname == null) arcname = "";

    if(number == 1)
     {
       String fic = fics[0];
       fpath = Path.merge(source, fic);
       File f = new File(source, fic);
       if(f.isDirectory())  ArchiveMode = ARC_FOLDER;
       else                 ArchiveMode = ARC_SINGLE;
     }
    else if(number > 1) ArchiveMode = ARC_TAGGED;

    boolean buttonMode = false;

    switch(ArchiveMode)
     {
       default:
       case ARC_NONE:
                     title = "ARCHIVER";
                     fpath = "Select files or directories to archive";
                     dpath = "";
                     break;
      case ARC_SINGLE: title = "Archiving a file";
                     buttonMode = true;
                     break;
      case ARC_FOLDER: title = "Archiving a directory";
                       buttonMode = true;
                       break;
      case ARC_TAGGED:
                       title = "Archiving a list of files";
                       fpath = source + File.separator + "[TAGGED LIST]";
                       buttonMode = true;
                       break;
     }

      setButtonEnabled(buttonMode);
	  setSource(title, fpath);
	  setPath(dpath);
	  setArchive(arcname);
	  setList(fics);

}


    // Action quand on presse le bouton d'archivage

   String arcpath = "";  // utilis en externe

   void archive()
   {
    //debug(""); debug("enter archiving");

    arcpath = getPath();    // Chemin archive = Dst Path au dpart
    String arcname = getArchive();    // Nom archive
    String fichiers = getSource();   // Nom fichier ou TAGGED LIST
    String archive;    // Chemin complet avec nom archive

    //debug("arc  " + archive);
    Archiver.setMessage("Error");

    if(ArchiveMode == ARC_NONE)
    {
      dispTip("Nothing to do! Select files/dir in first panel, please...");
      return;
    }

    if(arcname == null || arcname.length() == 0)
    {
      dispTip("Give a name to the archive, please!");
      return;
    }

    if(Path.hasExtension(arcname))
    {
     if(Directory.isGZip(arcname))
     {
      if (ArchiveMode != ARC_SINGLE)
       {
        dispTip("GZip require a single file"); return;
       }
     }
     if(!Directory.isArchive(arcname))
      {
       dispTip("Give a valid archive extension or none (default is zip)");
       return;
      }
    }
    else
    {
     arcname = arcname + ".zip";
    }

    setArchive(arcname);
    archive  = Path.merge(arcpath, arcname);

    // Test si archive existe dja
    boolean replacing = (new File(arcpath, arcname)).exists();

    if(replacing)
     {
       DialogCYN cd = new DialogCYN(frame, "Erasing the current " + arcname.toUpperCase() + " archive, continue?");
       cd.setVisible(true);
       if(cd.answer != cd.YES) return;
     }


    boolean result = false;

    Cursor cursor = getCursor();
    setCursor(new Cursor(Cursor.WAIT_CURSOR));


    if(ArchiveMode == ARC_SINGLE)
    {
     Message("Archiving " + fichiers + " into "+ archive);

     result = Archiver.archiveAnyFile(fichiers, archive);

     if(result) Message("Archive completed");
           else Message(Archiver.getMessage());

     setCursor(cursor);
     return;
    }

    String fics[] = getList();
    String srcpath = null;

    //debug("fics len " + fics.length);

    if(ArchiveMode == ARC_FOLDER)
    {

     String fic = fics[0];
     Message("Archiving " + fic + "'s content into "+ archive);

     // Je veux inclure le nom du chemin
     if (Preferences.ArcInclude)
     {
       srcpath = srcdir;
     }
     else
     {
       srcpath = Path.merge(srcdir, fic);
       fics = (new File(srcpath)).list();
     }

    }


    if(ArchiveMode == ARC_TAGGED)
    {
     Message("Archiving " + fichiers + " into "+ archive);
     srcpath = srcdir;
    }

    // Si la liste contient des lments, on archive
    if(fics.length > 0)
    {
     int fr = Archiver.archive(srcpath, fics, archive);
     Message("Archived " + fr + " files");

     if(Archiver.error)
       (new DialogMessage(frame, "Error in archiving")).setVisible(true);
    }
    else dispTip("Nothing to do");

   setCursor(cursor);
   return;

  }


 //------------------------------ EVENT HANDLER

    String message = "";

   final int MESSAGE = 0;
   final int TIP = 1;
   final int MADE = 3;

   void Message(String s) { message = s; sourceActionEvent(MESSAGE); };
   void dispTip(String s) { message = s; sourceActionEvent(TIP); };


   public void actionPerformed(ActionEvent event)
	{
	  Object object = event.getSource();
  	  if (object == buttonMake)
  	  {
  	    archive();
  	    sourceActionEvent(MADE);
  	  }
    }


   /** External listener */
   ActionListener actionListener = null;

   /** Fire an action event to the listeners. */
   protected void sourceActionEvent(int action)
	{
	  if (actionListener == null) return;
 	  String command = new Integer(action).toString();
	  actionListener.actionPerformed(new ActionEvent(this, ActionEvent.ACTION_PERFORMED, command));
	}

   /**
      Allow to add an external listener,
      to send it action events
   */
   public void addActionListener(ActionListener l)
	{
     actionListener = AWTEventMulticaster.add(actionListener, l);
	}

   /** Gestion des checkBoxes */
   public void itemStateChanged(ItemEvent event)
    {
  	 Object object = event.getSource();

     if (object == checkRecurse)
	  Preferences.ArcRecurse = checkRecurse.getState();
     else if (object == checkInclude)
	  Preferences.ArcInclude = checkInclude.getState();
//     else if (object == checkStore)
//	  Preferences.StoreOnly = checkStore.getState();
   }

}
