/*
    Advanced Explorer (c) 1999-2006 by D.G. Sureau

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    webmaster@scriptol.org
    http://www.scriptol.org
*/
package common;

import java.util.*;
import java.util.zip.*;
import java.io.*;
import java.awt.event.*;
import java.awt.*;
import java.awt.Point;
import common.*;

public final class Archiver
{

  static String archiveName;
  public static String messagetext = null;
  static boolean error = false;
  static boolean insidePath = false;
  static long lastModified = -1;

  public Archiver()  {  }


  // Problme avec noms de fichiers
  static public String zipToJava(String zipentry)
  {
   return zipentry.replace('/' , File.separatorChar);
  }

  static public String javaToZip(String zipentry)
  {
   return zipentry.replace(File.separatorChar, '/');
  }

/*
  public static boolean isZip(String name)
  {
   name = name.toLowerCase();
   if(name.endsWith(".zip")) return true;
   if(name.endsWith(".jar")) return true;
   if(name.endsWith(".sxw")) return true;
   return false;
  }
*/

 /*
     Extraction d'un lment
 */

 static final int ZNR = 1;
 static final int ETW = 2;
 static final int IOE = 3;
 static final int ETU = 4;  // enable to unarchive, erreur sur fichier
 static int messageNum = 0;
 static int written = 0;

 /*
     Extrait un simple fichier
 */

 static private boolean extract(ZipInputStream zis, String dest, String entryname, boolean restore)
 {
    //Disp.it("create file " + fname + " in " + destpath);
   Path.split(entryname);
   String dir   = Path.Dir;
   String fname = Path.Filename;
   String destpath;
   boolean ok = true;

   if(restore)
         destpath = Path.merge(dest, dir);
   else  destpath = dest;

   if((new File(destpath)).exists() == false)
       Directory.createDir(destpath);

    File df = new File(destpath, fname);
    written = 0;
    //Disp.it("made " + df.getName());

    if(Preferences.ArcReplace || !df.exists())
     {
      try
      {
       FileOutputStream fos = new FileOutputStream(df);
       byte[] buffer = new byte[2048];
       int rsize = 1;
       while(rsize > 0)
        {
         try
         {
           rsize = zis.read(buffer);
           if(rsize > 0)
           {
             fos.write(buffer, 0, rsize);
             written += rsize;
           }
         } catch(InternalError re)
           {
             rsize = 0; messageNum = ZNR; ok = false;
           }
        }

       fos.close();
       if(lastModified != -1) df.setLastModified(lastModified);
      }
      catch(IOException i) { messageNum = IOE; ok = false; }
     } // if

    return ok;
  } // extract



 /**
     Cette fonction rend un nom de fichier corrompu acceptable
 */
 public static String makeName(String name)
 {
    StringBuffer b = new StringBuffer(name);
    for (int i = 0; i < b.length(); i++)
     {
       char c = b.charAt(i);
       if (Character.isLetterOrDigit(c)) continue;
       if (c == '_') continue;
       if (c == '+') continue;
       if (c == '!') continue;
       if (c == '#') continue;
       if (c == '?') continue;
       if (c == '-') continue;
       if (c == '/') continue;
       if (c == '\\') continue;
       if (c == '*') continue;
       b.setCharAt(i, '_');
     }
    return  new String(b.toString()); // enlever code invalides
 }


  /**
     Extrait un fichier dans une archive
     archive:   archive affiche
     entryname: nom complet avec chemin
     dest:      rpertoire d'accueil
  */
  public static boolean extractFile(String archive, String entryname, String dest, boolean restore)
  {
   boolean error = false;
   messageNum = 0;

   try
   {
     //Disp.it("archive " + archive + " entry " + entryname + " dest " + dest);
     FileInputStream fis = new FileInputStream(archive);
     BufferedInputStream bis = new BufferedInputStream(fis);
     ZipInputStream zis = new ZipInputStream(bis);
     ZipEntry ze = null;

     while((ze = zis.getNextEntry()) != null)
     {
      String zentry =  ze.getName();
      lastModified  = ze.getTime();

      if(zentry.equals(entryname))
      {
       entryname = zipToJava(entryname);

       if(ze.isDirectory())  // Rpertoire simple  restituer
         error = !Directory.createDir(dest, entryname);
       else
       {
        int total = (int) ze.getSize();
        error = !extract(zis, dest, entryname, restore);
        if (error) messageNum = ETU;
        if (total != -1) if (written != total) error = true;
       }
     } // if equals
    } // while nextentry

     //zf.close();
    zis.close();
    bis.close();
    fis.close();
   }
   catch(IOException ioe) { messageNum = IOE;  }
   catch(InternalError re) { messageNum = ZNR; }
   //catch(Exception z)  { messagetext = z.toString(); }
   // Message d'erreur mme si fichiers ok

   switch(messageNum)
   {
    case ZNR: setMessage("Zip format not recognized"); error = true; break;
    case ETW: setMessage("Enable to write"); error = true; break;
    case IOE: setMessage("Enable to extract into " + dest); error = true; break;
    case ETU: setMessage("Enable to extract " + makeName(entryname)); error = true; break;
    default:  break;
   }

   return !error;
  } // extract


 /**
    Dsarchiver une archive zip entire sur laquelle on pointe
    dans DstPath
 */

 public java.awt.Point unArchive(String archive, String dest)
 {
   int counter = 0;
   int found = 0;
   error = false;
   messageNum = 0;

   try
   {
     FileInputStream fis = new FileInputStream(archive);
     BufferedInputStream bis = new BufferedInputStream(fis);
     ZipInputStream zis = new ZipInputStream(bis);
     ZipEntry ze = null;

     while((ze = zis.getNextEntry()) != null)
     {
       String entryname = zipToJava(ze.getName());
       //Disp.it("entry=" + entryname);
       if(ze.isDirectory())
       {
        if(Preferences.ArcRestore)
         {
           if(!Directory.createDir(dest, entryname))
           {
             error = true;
             //System.out.println("archiver dir not created");
           }
         }
       }
       else
       {
        found++;
        int total = (int) ze.getSize();
        lastModified = ze.getTime();
        boolean ok = extract(zis, dest, entryname, Preferences.ArcRestore);
        //System.out.println("return of extract =" + ok + " error=" + error);
        if(total != -1) if (total != written)  error = true;
        if (ok)  { counter++; dispTip(entryname + " extracted"); }
        else     { message("Error on " + makeName(entryname)); error = true;  }
       } // if else
     } // while

     zis.close();
     bis.close();
     fis.close();
    }
    catch(IOException i) { messageNum = IOE; }
    catch(InternalError re) { messageNum = ZNR; }


   // Message d'erreur mme si fichiers ok
   switch(messageNum)
   {
    case ZNR: setMessage("Zip format not recognized"); error = true; break;
    case ETW: setMessage("Enable to write"); error = true; break;
    case IOE: setMessage("Error while extracting into " + dest); error = true; break;
    default:  setMessage(counter + " unarchived"); break;
   }

   return new Point(found, counter);
 }   // unarchive


  // Mthode choisie
  static int getMethod()
  {
//    if(!Preferences.StoreOnly) return ZipEntry.STORED;
    return ZipEntry.DEFLATED;
  }


  /*
     Enregistre un fichier dans l'archive
  */

  static private boolean putFile(ZipOutputStream zos, ZipEntry ze, File src)
  {
   if(!src.exists())  { setMessage(src + " not found"); return false; }
   if(!src.canRead()) { setMessage(src + " is unreadable"); return false; }

   try
   {
    FileInputStream fis = new FileInputStream(src); // Stream du fichier  compresser
    zos.putNextEntry(ze);       // Ajoute une entre

    int rsize = 1;
    byte[] Buffer = new byte[2048]; // buffer

    while(rsize > 0)
     {
      rsize = fis.read(Buffer);
      if(rsize > 0)
	    zos.write(Buffer, 0, rsize);
     }

    zos.closeEntry();
    zos.flush();
    fis.close();
    return true;
   }
   catch(IOException ioe) { setMessage(ioe.toString()); }

   return false;
  }

 /**
    Archiver un fichier (dans un fichier archive ddi)
 */

 static public boolean archiveFile(String fname, String arcname)
 {
   File src = new File(fname);
   File dst = new File(arcname);
   ZipEntry ze = null;

   boolean result = true;
   String rp;

   if(Preferences.ArcInclude)  rp = Path.getRelativePath(fname);
   else                        rp = Path.getFile(fname);

   try {
     ze = new ZipEntry(rp);
     ze.setTime(src.lastModified());
   }   // Dfinition entre
   catch(NullPointerException npe)
   {
     setMessage(rp + " forms bad zip entry " + ze.getName());
     result = false;
   }
   if(!result) return false;

   try
   {
     FileOutputStream fos = new FileOutputStream(dst);// Output stream de l'archive
     ZipOutputStream zos = new ZipOutputStream(fos);// Output stream surcharg en Zip
     zos.setMethod(getMethod());

     result = putFile(zos, ze, src);

     zos.close();
     fos.close();

   } catch(IOException ioe) { setMessage(ioe.toString()); result = false; }

  return result;
 }



  /**
   Archivage
   Fonction rcursive pour une liste (dpart/contenu d'un rpertoire)
  */

  static String source = "";

  static private int recurse(String path, String[] fics, ZipOutputStream zos)
   {
     //Disp.it("Enter Recurse");
     //Disp.it("source " + source);
     //Disp.it("Path " + path);
     //Disp.it("first file " + fics[0]);

    int fl = fics.length;
    //Disp.it("size " + fl);
    int counter = 0;

    for(int i = 0; i < fl; i++)
     {
     try
     {
      //Disp.it("next name " + fics[i]); // list en paramtre
      String fname   = fics[i];
      String srcpath = Path.merge(source, path);   // full path
      //String srcfic  = Path.merge(srcpath, fname); // path inside current dir


      File src = new File(srcpath, fname);
      if(!src.exists())  { System.out.println(src + " not found"); break;}

      String dstfic;

      // On passe l'archive si elle est dans le rpertoire.
      if (insidePath)
      {
       String fullpath =  Path.merge(srcpath, fname);
       //System.out.println("dstfic " + fullpath + " archive " + archiveName);
       //System.out.println("same " + fullpath.equals(archiveName));
       if (fullpath.equals(archiveName)) break;
      }

      if(Preferences.ArcInclude)
         dstfic = Path.merge(path, fname); // rel. path
      else
         dstfic = fname; // filename only

      ZipEntry ze = null;

      boolean result = true;
      try {
       ze = new ZipEntry(dstfic);
       //ze.setMethod(getMethod()); // Mthode sur stream
       ze.setTime(src.lastModified());
      }   // Dfinition entre
      catch(NullPointerException npe)
      {
        setMessage(dstfic + " forms bad zip entry ");
        result = false;
        error = true;
      }
      if(!result) return 0;

      //Disp.it("next entry " + ze.getName());

      if(src.isDirectory())
      {
       if(Preferences.ArcRecurse)
         counter += recurse(Path.merge(path, fname), src.list(), zos);
      }
      else
      {
       //Disp.it("entry " + ze.getName());
       if (putFile(zos, ze, src)) counter++;
      } // Si c'est un fichier
     } catch(Exception e)
     {
       setMessage("Archive error - " + e.toString());
       //System.out.println(messagetext);
       error = true;
     }
    } // fichier suivant

   return counter;
} // recurse


 /**
     Cre une archive d'une liste de fichiers
 */

 static public int archive(String srcpath, String fics[], String archive)
 {
   //Disp.it("archive src=" + srcpath);
   //Disp.it("archive arc=" + archive);
   //Disp.it("archive array=" + fics[0]);
   String dstpath = Path.splitPath(archive);
   if (dstpath.indexOf(srcpath) != -1)
   {
    insidePath = true;
    archiveName = archive;
   }
   else
   {
    insidePath = false;
    archiveName = "";
   }

   //System.out.println("archive " + archive + " " + archiveName + " " + insidePath);
   //System.out.println("dpath " + dstpath + " src " + srcpath);

   File dst = new File(archive);
   int counter = 0;
   error = false;

   try
   {
     FileOutputStream fos = new FileOutputStream(dst); // Output stream de l'archive
     ZipOutputStream zos = new ZipOutputStream(fos);  // Output stream surcharg en Zip
     zos.setMethod(getMethod());

     source = srcpath;

     counter += recurse("", fics, zos);

     zos.close();
     fos.close();
   }
   catch(IOException ioe) { setMessage(ioe.toString()); error = true; }

   //Disp.it("Archiver - counter " + counter);
   return counter;
 }



//------------------------------------- G Z I P




 static public boolean archiveAnyFile(String fname, String arcname)
 {
//  if (Directory.isGZip(arcname)) return makeGZip(fname, arcname);

  if (Directory.isZip(arcname))
     return archiveFile(fname, arcname);
  return false;
 }

/*
 static public boolean makeGZip(String fname, String arcname)
 {
   //System.out.println("enter gzip " + fname + " " + arcname);
   File src = new File(fname);
   File dst = new File(arcname);

   boolean result = true;
   String rp = Path.getFile(fname);

   //System.out.println("in gzip to " + arcname);

   try
   {
     FileInputStream  fis = new FileInputStream(src);
     FileOutputStream fos = new FileOutputStream(dst);// Output stream de l'archive
     GZIPOutputStream gos = new GZIPOutputStream(fos, 2048);// Output stream surcharg en Zip

     int rsize = 1;
     byte[] buffer = new byte[2048]; // buffer
     while(rsize > 0)
     {
      rsize = fis.read(buffer);
      if(rsize > 0) gos.write(buffer, 0, rsize);
     }

     fis.close();
     fos.close();
     gos.close();

     messagetext = arcname + " completed";

   } catch(IOException ioe) { messagetext = ioe.toString(); result = false; }

  return result;
 }


 // routine
 static private int fromGZip(GZIPInputStream zis, String dest, String name)
 {
   File df = new File(dest, name);
   int written = 0;

   if(Preferences.ArcReplace || !df.exists())
     {
      try
      {
       FileOutputStream fos = new FileOutputStream(df);
       byte[] buffer = new byte[2048];
       int rsize = 1;
       while(rsize > 0)
        {
         try
         {
           rsize = zis.read(buffer);
           if(rsize > 0)
           {
             fos.write(buffer, 0, rsize);
             written += rsize;
           }
         }
         catch(InternalError re) { rsize = 0; messageNum = ZNR; }
        }

       fos.close();
      } catch(IOException i) { messageNum = IOE; }
     } // if

    return written;
  } // extract

  */
  /**
     Extrait un fichier dans un GZ
  */

  /*
  public static int extractGZip(String archive, String dest)
  {
   int num = 0;
   messageNum = 0;
   String name = Path.removeExt(archive);

   try
   {
     FileInputStream fis = new FileInputStream(archive);
     BufferedInputStream bis = new BufferedInputStream(fis);
     GZIPInputStream zis = new GZIPInputStream(bis);

     fromGZip(zis, dest, name);

     zis.close();
     bis.close();
     fis.close();
     num = 1;
   }
   catch(IOException ioe) { messageNum = IOE; }
   catch(InternalError re) { messageNum = ZNR; }

   switch(messageNum)
   {
    case ZNR: messagetext = "Format not recognized"; error = true; break;
    case ETW: messagetext = "Enable to write"; error = true; break;
    case IOE: messagetext = "Enable to extract into " + dest; error = true; break;
    default:  break;
   }

   return num;
  } // extract

*/


   /** External listener */
   ActionListener actionListener = null;

   /** Fire an action event to the listeners. */

   protected void dispTip(String mess)
	{
	  if (actionListener == null) return;
	  setMessage(mess);
	  //System.out.println("disptip " + messagetext);
	  actionListener.actionPerformed(new ActionEvent(this, ActionEvent.ACTION_PERFORMED, "t"));
	}


   protected void message(String mess)
	{
	  if (actionListener == null) return;
	  setMessage(mess);
	  //System.out.println("message " + messagetext);
	  actionListener.actionPerformed(new ActionEvent(this, ActionEvent.ACTION_PERFORMED, "m"));
	}

   public static String getMessage() { return messagetext; }
   public static void setMessage(String s) { messagetext = s; }
   /**
      Allow to add an external listener for actions
   */

   public void addActionListener(ActionListener l)
    {
     actionListener = AWTEventMulticaster.add(actionListener, l);
    }

}
