/*
    Advanced Explorer (c) 1999-2006 by D.G. Sureau

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    webmaster@scriptol.org
    http://www.scriptol.org
*/
package common;

import java.awt.*;
import java.applet.*;
import java.awt.event.*;
import java.net.URL;
import common.*;

/**
 * An abstract class used to implement special buttons.
 * It supports 3-D beveled borders, and the posting of continuous
 * notification events while the button is pressed.
 * <p>
 */
public abstract class DButtonBase extends Canvas
{
    String label;

    boolean blackBorder = false; // DGS, cadre noir
    boolean incrustedBorder = false;
    boolean comboExtension = false; // DGS prolongement

    Color extendDark = null;
    Color extendLight = null;

    Color borderLight = null;
    Color borderDark  = null;

    Color parentColor = null;

    Tip tip = null;

    /**
     * Constructs a default ButtonBase. The defaults are no notifyWhilePressed,
     */
    protected DButtonBase()
    {
        pressed			= false;
        released		= true;
        notifyWhilePressed	= false;
        running			= false;
        notified		= false;
        useOffset		= true;  // objet dcal quand on appuie sur le bouton
        showURLStatus		= true;
        isAdded			= false;
        notifyTimer		= null;
        notifyDelay		= 1000;

        bevel			= 1;

        pressedAdjustment	= 0;

        if(OS.isWindows()) setShowFocus(true);

	}




    /** borders & style
      Change les couleurs, donc mettre setBackground avant car il dfinit
      la couleur parent, la mme que le bouton par dfaut
    */
    public void setStyle(boolean border,
                         boolean incrusted,
                         int bevel)
    {
      setBorderEnabled(border);
      setIncrustedEnabled(incrusted);
      setBevelHeight(bevel);
    }

    public void setBorderEnabled(boolean b)
    {
     blackBorder = b;
     if(b)
     {
      borderDark = Color.black;
      borderLight = Color.black;
     }
    }

    public void setIncrustedEnabled(boolean i)
    {
      incrustedBorder = i;
//      if(i) setBorderColors(buttonColor);
    }

	/** Sets the current border color */
    public void setIncrustedBorder()
    {
      if(incrustedBorder)
      {
       borderDark  = ColorUtils.darken(parentColor, (double) 2 / 5);
       borderLight = ColorUtils.lighten(parentColor, (double) 2 / 5);
       // parentColor.brighter()
      }
      else
      {
       borderDark = Color.black;
       borderLight = Color.black;
      }
   }


    /**
       DGS
       These functions modify the appearance
       for the button included int a combo box complex object
    */
    public void setComboEnabled(boolean e)
    {
      comboExtension = e;
      if(e) setComboColors(buttonColor);
    }

    public void setComboColors(Color c)
    {
      extendDark =  ColorUtils.darken(c, (double) 2 / 5);
      extendLight = ColorUtils.lighten(c, (double) 1 / 2);
    }

/**
   Sets the current button color.
   The highlights of the button are derived from this color.
 */

  public void setBackground(Color color)
  {
    super.setBackground(color);

    if (!objectsEqual(buttonColor, color))
    {
	buttonColor = color;
	hilightColor		= ColorUtils.lighten(buttonColor,	(double) 1 / 2);
	pressedHilightColor	= Effect.darken(buttonColor, (double) 58 / 100);
	disabledHilightColor	= Effect.lighten(buttonColor, (double) 666 / 1000);
	shadowColor		= ColorUtils.darken(buttonColor,	(double) 2 / 5);// 1 / 4
	pressedShadowColor	= Effect.darken(buttonColor, (double) 1 / 10);
	disabledShadowColor	= Effect.darken(buttonColor, (double) 166 / 1000);
	disabledButtonColor	= Effect.lighten(buttonColor, (double) 1 / 3);
	pressedButtonColor	= Effect.darken(buttonColor, (double) 1 / 10);
     }
  }


    /** Sets the "height" (cross-section) of a beveled edge, in pixels */
    public void setBevelHeight(int b) { bevel = b; }

    /** Returns the current "height" of a beveled edge  */
    public int getBevelHeight() { return bevel; }

    /**
     * Sets whether the button will continually post notify events while pressed.
     */
    public void setNotifyWhilePressed(boolean flag)
    {
    	if(notifyWhilePressed != flag)
    	{
	        notifyWhilePressed = flag;
	        if (notifyWhilePressed)
	        {
	            notifyTimer = new Timer(notifyDelay, true);
	            notifyTimer.addActionListener(action);
	        }
	        else if (notifyTimer != null)
	        {
	            notifyTimer = null;
	        }
		}
    }

    /** Gets whether the button will continuously post events while pressed */
    public boolean isNotifyWhilePressed() {  return notifyWhilePressed; }


    /**
     * Sets the notification event delay in milliseconds.
     * @param delay the delay between notification events in milliseconds
     */
    public void setNotifyDelay(int delay)
    {
    	if(notifyDelay != delay)
    	{
	        notifyDelay = delay;
	        if(notifyTimer != null)
	        	notifyTimer.setDelay(notifyDelay);
		}
    }

    /** Returns the current delay in milliseconds between notification events */
    public int getNotifyDelay() {  return notifyDelay; }

    /**
     * Sets whether objects in the button will be offset down and to the right
     * bevel height amount or not.
     * This also impacts the way the button is drawn when it is pressed.
     */
    public void setUseOffset(boolean flag) { useOffset = flag; }

    /**
     * Returns whether this button use offset
     */
    public boolean isUseOffset() {  return useOffset;  }

    /**
       Sets whether this button will be shown as having
       the focus when the mouse enters.
     */
    public void setShowFocus(boolean flag) { showFocus = flag; }

    /**
     * Returns whether this button will be shown as having the focus when the mouse enters.
     */
    public boolean isShowFocus() { return showFocus; }

    /**
     * Sets whether the linkURL will be displayed in the status area when the mouse
     * is over the button.
     * This flag also controls erasing of the status area after the URL has been displayed.
     */
    public void setShowURLStatus(boolean flag) { showURLStatus = flag;  }

    /**
     * If true show the linkURL in the status area when the mouse is over the button.
     * If the linkURL is null, nothing is displayed, regardless of this flag.
     * This flag also controls erasing of the status area after the URL has been displayed.
     * @return true if the linkURL will be displayed in the status area when the mouse
     * is over the button.
     * @see #setShowURLStatus
     */
    public boolean isShowURLStatus()  {  return showURLStatus; }

    /**
     * Sets the URL of the document to link to when the button is clicked.
     * @param url the URL
     * @see #getLinkURL
     */
    public void setLinkURL(URL url)
    {
        linkURL = url;
        context = null;
    }

    /**
     * Returns the URL of the document to link to when the button is clicked.
     * @see #setLinkURL
     */
    public URL getLinkURL()	{ return linkURL; }

	/**
	 * Sets the frame specifier for showing a URL document in a browser or applet
	 * viewer. It is interpreted as follows:
	 * <UL>
	 * <DT>"_self"  show document in the current frame</DT>
	 * <DT>"_parent"    show document in the parent frame</DT>
	 * <DT>"_top"   show document in the topmost frame</DT>
	 * <DT>"_blank" show document in a new unnamed toplevel window</DT>
	 * <DT>all others   show document in a new toplevel window with the given name</DT>
	 * </UL>
     * @param newFrame the frame specifier
     * @see #getFrame
     */
    public void setFrame(String newFrame) { frame = newFrame; }

	/**
	 * Gets the frame specifier for showing a URL document in a browser or applet
	 * viewer. It is interpreted as follows:
	 * <UL>
	 * <DT>"_self"  show document in the current frame</DT>
	 * <DT>"_parent"    show document in the parent frame</DT>
	 * <DT>"_top"   show document in the topmost frame</DT>
	 * <DT>"_blank" show document in a new unnamed toplevel window</DT>
	 * <DT>all others   show document in a new toplevel window with the given name</DT>
	 * </UL>
	 * @return the frame specifier
     */
    public String getFrame() { return frame; }

	/**
	 * Ensures that this component is laid out properly, as needed.
	 * This is a standard Java AWT method which gets called by the AWT to
	 * make sure this component and its subcomponents have a valid layout.
	 * If this component was made invalid with a call to invalidate(), then
	 * it is laid out again.
	 * It is overridden here to locate the applet containing this component.
	 */
    public void validate()
    {
        super.validate();
    }

    /**
     * Enables this component so that it will respond to user input.
     * This is a standard Java AWT method which gets called to enable or disable
     * this component. Once enabled this component will respond to user input.
     *
     * @param flag true if the component is to be enabled; false if it is to be disabled.
     * @see java.awt.Component#isEnabled
     */
    public void setEnabled(boolean flag)
    {
     if(isEnabled() != flag)
      {
	   	if(flag)
	     {
    		super.setEnabled(true);
		    pressed = false;
		    pressedAdjustment = 0;
		 }
		else
		{
    		super.setEnabled(false);
		    if (notifyTimer != null) notifyTimer.stop();

		    pressed = false;
		    pressedAdjustment = 0;
		}

	    repaint();
	  }
    }


	/**
	 * Tells this component that it has been added to a container.
	 * This is a standard Java AWT method which gets called by the AWT when
	 * this component is added to a container. Typically, it is used to
	 * create this component's peer.
	 * It has been overridden here to hook-up event listeners.
	 */
	public synchronized void addNotify()
	{
		super.addNotify();

		//Hook up listeners
		if (focus == null)
		{
			focus = new BBFocus();
			addFocusListener(focus);
		}
		if (key == null)
		{
			key = new BBKey();
			addKeyListener(key);
		}
		if (mouse == null)
		{
			mouse = new BBMouse();
			addMouseListener(mouse);
		}
		isAdded = true;

        validate();

        // DGS
        parentColor = getParent().getBackground();
  	    if (objectsEqual(buttonColor, Color.black))
         setBackground(parentColor);

        setIncrustedBorder(); // selon parent color
	}

	/**
	 * Tells this component that it is being removed from a container.
	 * This is a standard Java AWT method which gets called by the AWT when
	 * this component is removed from a container. Typically, it is used to
	 * destroy the peers of this component and all its subcomponents.
	 * It has been overridden here to unhook event listeners.
	 */
	public synchronized void removeNotify()
	{
		//Unhook listeners
		if (focus != null)
		{
			removeFocusListener(focus);
			focus = null;
		}
		if (key != null)
		{
			removeKeyListener(key);
			key = null;
		}
		if (mouse != null)
		{
			removeMouseListener(mouse);
			mouse = null;
		}

		super.removeNotify();
		isAdded = false;
	}

    /** Handles redrawing of this component on the screen */
    public void update(Graphics g) { paint(g);}

	/**
	 * Paints this component using the given graphics context.
     * This is a standard Java AWT method which typically gets called
     * by the AWT to handle painting this component. It paints this component
     * using the given graphics context. The graphics context clipping region
     * is set to the bounding rectangle of this component and its [0,0]
     * coordinate is this component's top-left corner.
     *
     * @param g the graphics context used for painting
     * @see java.awt.Component#repaint
     * @see #update
	 */
    public void paint(Graphics g)
    {
	   updateButtonImage();
	   g.drawImage(buttonImage, 0, 0, this);
    }

    /**
     * Is the given bevel size valid for this button.
     */
    protected boolean isValidBevelSize(int i)
    {
        Dimension s = getSize();

        return !(i < 0 || i >= (s.width / 2) || i >= (s.height / 2));
    }


	/**
	 * Tell the browser to show the document referenced by the linkURL.
	 * If the frame specifier is not null or empty, then tell the browser
	 * to open the document with that frame.
	 */

    protected void linkToURL()
     {
		if (context != null && linkURL != null)
		{
		    if (frame == null || frame.length() == 0)
		        context.showDocument(linkURL);
		    else
		        context.showDocument(linkURL, frame);
		}
     }

    /** Sets the applet context used to view documents */
    protected void setAppletContext(AppletContext c) { context = c; }

	/**
	 * Maintains the buttonImage size and draws the
	 * button in the buttonImage offscreen image. Called by paint
	 */

    protected void updateButtonImage()
	{
        Dimension s	= getSize();
        int width	= s.width;
        int height	= s.height;
        // DGS entirement chang, maintenant utilis pour les bevel
        // et non plus seulement le rectangle de fond
        int x		= 0;
        int y		= 0;
        int w		= width - 1;  // en fait largeur - 1 pour acclrer
        int h		= height - 1;
        int i;
        Color highlight1, highlight2, fillColor;
		boolean raised = !(pressed && inButton);  // pas press ou bouton dehors
        int iOffset = 0;

        if(incrustedBorder)
        {
           iOffset = 1;
        }

		if(isButtonImageInvalid())
		{
			buttonImage = createImage(width, height);
			try
			{
	          MediaTracker tracker = new MediaTracker(this);
	          tracker.addImage(buttonImage, 0);
	          tracker.waitForID(0);
            }
            catch(InterruptedException e){}
        }


        biG = buttonImage.getGraphics();
        Color oldColor = biG.getColor();


	if(isEnabled())	//Enabled
	{
		//borderColor = borderColor;
	 if (raised)
	  {
		fillColor	= buttonColor;
		highlight1	= hilightColor;
		highlight2	= shadowColor;
	  }
	else	//Pressed
          {
	    fillColor	= pressedButtonColor;
	    highlight1	= pressedHilightColor;
	    highlight2	= pressedShadowColor;
	   }
	}
	else //Disabled
	{
	    //borderColor	= disabledBorderColor;
	   fillColor		= disabledButtonColor;
	   highlight1		= disabledHilightColor;
	   highlight2		= disabledShadowColor;
	}


        if(comboExtension)
        {
           int l = 0;
           int t = 0;
           int r = l + width - 1;
           int b = t + height - 1;

           biG.setColor(extendDark);
           biG.drawLine(l , t, r, t);   // top
           biG.drawLine(l , t + 1, r - 1, t + 1); // top

           // La premiere ligne reste en couleur du fond
           biG.setColor(extendLight);
           biG.drawLine(r, t + 1, r, b); // right
           biG.drawLine(l, b, r, b);  // bottom

           x = l;
           y = t + 2;
           w = w - 2;
           h = h - 4;
        }

		//Draw the black border
	if(blackBorder)
	{
 	  final int offset = 0;

 	  biG.setColor(borderDark);
          biG.drawLine(offset, 0, w - offset - offset - iOffset, 0);  // top
          biG.drawLine(0, offset, 0, h - offset - offset);  // left

          biG.setColor(/*buttonColor.brighter()*/ borderLight);
          biG.drawLine(offset, h, w - offset - offset, h);  // bottom
          biG.drawLine(w, h - offset - offset, w, offset + 1);  // right
          x++;
          y++;
          w -= 2;
          h -= 2;
        }


    	int mup = bevel;
		if(bevel >  1) mup--;   // au moins une ligne sauf si bevel = 0

    	//Fill the button content
		biG.setColor(fillColor);
		biG.fillRect(x, y, w - x + 1, h - y + 1);

		if(!raised && useOffset)  // si press et dcalage
		{
		//Draw the bevels
		biG.setColor(highlight1);
		for(i = 0; i < mup; i++)
		{
		     biG.drawLine(x + i, y + i, x + i, y + h); // left
		     biG.drawLine(x + i, y + i, x + w, y + i); // top
		}
           }

		if(raised || !useOffset)  // si normal ou si pas de dcalage
		{
		    //Draw the bevels
	   	   biG.setColor(highlight1); // clair
		   for(i = 0; i < mup; i++)
		    {
	             biG.drawLine(x + i, y + i, x + i, y + h - i); // left
	             biG.drawLine(x + i, y + i, x + w - i - 1, y + i); // top
		    }

		    biG.setColor(highlight2);
		    for(i = 0; i < bevel; ++i)
		    {
		     biG.drawLine(x + i, y + h - i, x + w - i, y + h - i); // bottom
		     biG.drawLine(x + w - i, y + i + 1, x + w - i, y + h - i); // right
		    }
		}

        // dessine pointill du focus
        // a revoir si besoin selon x,y et bevel...
        if (hasFocus && showFocus)
        {
          int j;
	  biG.setColor(java.awt.Color.darkGray);
	  x = x + mup + 1;
	  y = y + mup + 1;
	  w = w - mup - bevel;
	  h = h - mup - bevel + 1;

          for(i = x; i <= w; i += 3) biG.drawLine(i , y, i + 1, y);// top
          for(j = y; j < h; j += 3) biG.drawLine(x, j, x, j + 1);// left
          for(i = x; i <= w; i += 3) biG.drawLine(i,  h, i + 1, h);// bottom
          for(j = y; j < h; j += 3) biG.drawLine(w, j, w, j + 1);// right
        }

	//Set the clipping area to be the inside of the button.
        //biG.clipRect(bevel + 1, bevel + 1, width - bevel - bevel - 2, height - bevel - bevel - 2);

        //Restore the original color
  	  biG.setColor(oldColor);
	}

	/**
	 * Returns true if a button image has been set, but it is not the
	 * size of this component.
	 */
  protected boolean isButtonImageInvalid()
  {
	Dimension s = getSize();
	return (buttonImage == null || s.width	 != buttonImage.getWidth(this) || s.height != buttonImage.getHeight(this));
  }


	/**
	 * Returns the recommended dimensions to properly display this component.
     * This is a standard Java AWT method which gets called to determine
     * the recommended size of this component.
	 */
    public Dimension getPreferredSize()
    {
	return new Dimension(bevel + bevel + 2, bevel + bevel + 2);
    }

    /**
	 * Returns the minimum dimensions to properly display this component.
     * It simply returns the results of a call to preferedSize().
	 */
    public Dimension getMinimumSize() { return getPreferredSize();  }

    public boolean isFocusable() { return true; }


    /**
     * Sets the command name of the action event fired by this button.
     * @param command The name of the action event command fired by this button
     */
    public void setActionCommand(String command) { actionCommand = command; }

    /**
     * Returns the command name of the action event fired by this button.
     * @return the action command name
     */
    public String getActionCommand() {  return actionCommand;  }

    /**
     * Adds the specified action listener to receive action events
     * from this button.
     * @param l the action listener
     */
   public void addActionListener(ActionListener l)
   {
	actionListener = AWTEventMulticaster.add(actionListener, l);
   }

    /**
     * Removes the specified action listener so it no longer receives
     * action events from this button.
     * @param l the action listener
     */
	public void removeActionListener(ActionListener l)
	{
		actionListener = AWTEventMulticaster.remove(actionListener, l);
	}

	class BBFocus extends java.awt.event.FocusAdapter implements java.io.Serializable
	{
	    public void focusGained(FocusEvent e)
	    {
	        hasFocus = true;
	        repaint();
	    }

	    public void focusLost(FocusEvent e)
	    {
	        hasFocus = false;
	        repaint();
	    }
	}

	class BBKey extends java.awt.event.KeyAdapter implements java.io.Serializable
	{
	    public void keyPressed(KeyEvent evt)
	    {
	        boolean isSpaceBar = (evt.getKeyCode() & KeyEvent.VK_SPACE) == KeyEvent.VK_SPACE;

	        if(isSpaceBar && hasFocus && showFocus && !OS.isMacintosh())
	        {
	        inButton = true;
	        notified = false;
	        if (notifyTimer != null && notifyWhilePressed && !running)
	        {
	            running     = true;
	            notifyTimer.start();
	        }

	        pressed           = true;
	        released          = false;
	        if(useOffset) pressedAdjustment = bevel;
	        repaint();
	        }

	    }

	    public void keyReleased(KeyEvent evt)
	    {
	        boolean isSpaceBar = (evt.getKeyCode() & KeyEvent.VK_SPACE) == KeyEvent.VK_SPACE;

	        if(isSpaceBar && hasFocus && showFocus && !OS.isMacintosh())
	        {
	        inButton = false;
	        if (notifyTimer != null && running)
	        {
	            running = false;
	            notifyTimer.stop();
	        }

	        if (pressed)
	        {
	            pressed = false;
	            pressedAdjustment = 0;

	            if (!notifyWhilePressed || !notified)
	            {
	            	//Handle going to the linkURL
	            	linkToURL();
	                sourceActionEvent();
	            }
	        }

	        released = true;
        	repaint();
	        }

	    }
	}

	/**
	 * This is the BBMouse Event handling innerclass.
	 */
	class BBMouse extends java.awt.event.MouseAdapter implements java.io.Serializable
	{
		/**
	     * Handles the BBMouse Pressed events
		 * If the notifyWhilePressed flag is true the notification Timer is started
		 * @param e the MouseEvent
		 * @see #setNotifyWhilePressed
		 * @see #setNotifyDelay
		 * @see #mouseReleased
		 */
		public void mousePressed(MouseEvent e)
		{
            //System.out.println("pressed");
	        requestFocus();
	        notified = false;
	        if (notifyTimer != null && notifyWhilePressed && !running)
	        {
	            running     = true;
	            notifyTimer.start();
	        }

	        pressed           = true;
	        released          = false;
	        if(useOffset) pressedAdjustment = bevel;
	        repaint();
            //System.out.println("passed");
		}

	    /**
	     * Handles the BBMouse Released events
	     * If the notification timer is running it is stopped.
	     * If the mouse was pressed inside the button then fire an action event.
	     * @param e the MouseEvent
	     * @see #mousePressed
	     */
	    public void mouseReleased(MouseEvent e)
	    {
	        if (notifyTimer != null && running)
	        {
	            running = false;
	            notifyTimer.stop();
	        }

	        if (pressed)
	        {
	            pressed = false;
	            pressedAdjustment = 0;

	            if (!notifyWhilePressed || !notified)
	            {
	            	//Handle going to the linkURL
	            	linkToURL();
	                sourceActionEvent();
	            }
	        }

	        released = true;
	        if(inButton) repaint();
			if(tip != null) tip.hideIt();
	    }

		/**
		 * Handles BBMouse Entered events
		 * @param e the MouseEvent
		 */
		public void mouseEntered(MouseEvent e)
		{
	        inButton = true;
	        //Disp.it("entered");

	        if (!released)
	        {
		        pressed = true;
		        if(useOffset) pressedAdjustment = bevel;

			    if(notifyTimer != null && notifyWhilePressed && !running)
				{
		            running = true;
		            notifyTimer.start();
				}

				//Display the linkURL
				if (showURLStatus && context != null && linkURL != null)
				{
					context.showStatus(linkURL.toString());
				}

		        repaint();

			}

	        if(tip != null) tip.showIt();

		}

		/**
		 * Handles BBMouse Exited events
		 * @param e the MouseEvent
		 */
		public void mouseExited(MouseEvent e)
		{
	        inButton = false;

	        if (notifyTimer != null && running)
	        {
	            running = false;
	            notifyTimer.stop();
	        }

	        if (pressed)
	        {
	            pressed = false;
	            pressedAdjustment = 0;
	            repaint();
	        }

			if (showURLStatus && context != null && linkURL != null)
			{
				context.showStatus("");
			}

			if(tip != null) tip.hideIt();
		}
	}

	/**
	 * This is the Action Event handling innerclass.
	 */
	class Action implements java.awt.event.ActionListener, java.io.Serializable
	{
		//Implement ActionListener to catch ActionEvents sent by either the notifyTimer.
		/**
		 * Handles Action events
		 * @param e the ActionEvent
		 */
		public void actionPerformed(ActionEvent e)
		{
	        if (e.getSource() == notifyTimer && notifyWhilePressed /* && !java.beans.Beans.isDesignTime()*/)
	        {
	        	notified = true;
	            sourceActionEvent();
	            return;
	        }
		}
	}

    /**
     * Fire an action event to the listeners.
     */
	protected void sourceActionEvent()
	{
		if (actionListener != null)
			actionListener.actionPerformed(new ActionEvent(this, ActionEvent.ACTION_PERFORMED, actionCommand));
	}


 /**  Compare deux objets  */
  public static boolean objectsEqual(Object objectA, Object objectB)
   {
		if (objectA == null) return (objectB == null);
		return objectA.equals(objectB);
   }


 /** Set the tool tip text */
 public void setTip(String str)
 {
   tip = new Tip(this, str);
 }

    /**
     * True if the button is currently pressed.
     */
    transient protected boolean   pressed;
    /**
     * True if the button has been released.
     */
    transient protected boolean   released;
    /**
     * True if the mouse is over this button.
     */
    transient protected boolean   inButton;
    /**
     * If true the button will continuously post events while pressed.
     */
    protected boolean   notifyWhilePressed;
    /**
     * True if the notify timer is running.
     */
    transient protected boolean   running;
    /**
     * True if a notification has been posted in response to a mouse down.
     */
    transient protected boolean   notified;
    /**
     * If true show the focus when the mouse enters the button.
     */
    protected boolean   showFocus = false;
    /**
     * If true set pressedAdjustment accordingly, else, it is always 0.
     */
    protected boolean   useOffset;
    /**
     * If true show the linkURL in the status area when the mouse is over the button.
     * If the linkURL is null, nothing is displayed, regardless of this flag.
     * This flag also controls erasing of the status area after the URL has been displayed.
     */
    protected boolean   showURLStatus;
    /**
     * Keeps track of wheather or not the button is added to a container.
     * Check before attempting to getFontMetrics() to avoid getting a null pointer.
     */
    transient protected boolean   isAdded;
    /**
     * The "height" (cross-section) of a beveled edge, in pixels.
     */
    protected int       bevel;
    /**
     * The delay in milliseconds between notifications while the button is pressed.
     */
    protected int       notifyDelay;
    /**
     * A drawing location adjustment for the 3-D bevel while button is pressed.
     */
    protected int       pressedAdjustment;
	/**
	 * Frame specifier for showing a URL document in a browser or applet
	 * viewer. It is interpreted as follows:
	 * <UL>
	 * <DT>"_self"  show document in the current frame</DT>
	 * <DT>"_parent"    show document in the parent frame</DT>
	 * <DT>"_top"   show document in the topmost frame</DT>
	 * <DT>"_blank" show document in a new unnamed toplevel window</DT>
	 * <DT>all others   show document in a new toplevel window with the given name</DT>
	 * </UL>
	 */
    protected String	frame = null;
    /**
     * The color of the border around the button.
     */
    protected Color	borderColor;
    /**
     * The color of the content of the button.  The highlights are derived from this color.
     */
    protected Color	buttonColor = Color.black;
    /**
     * Timer used to time notification events while button pressed.
     */
    protected Timer	notifyTimer;
    /**
     *	The offscreen buffer to draw the button in.
     */
    transient protected Image	buttonImage = null;
    /**
     *	The Graphics of the offscreen buffer to draw the button in.
     */
    transient protected Graphics	biG = null;
    /**
     * The URL of the document to show when the button is clicked.
     */
    protected URL	linkURL = null;
    /**
     * Applet context that shows the document.
     */
    transient protected AppletContext context = null;

    String actionCommand;
	ActionListener actionListener = null;
	transient boolean hasFocus = false;

	private Color hilightColor			= null;
	private Color pressedHilightColor	= null;
	private Color disabledHilightColor	= null;
	private Color shadowColor			= null;
	private Color pressedShadowColor	= null;
	private Color disabledShadowColor	= null;
	private Color disabledBorderColor	= null;
	private Color disabledButtonColor	= null;
	private Color pressedButtonColor	= null;

    private BBKey key = null;
    private BBFocus focus = null;
    private Action	action		= new Action();
    private BBMouse		mouse		= null;

}
