/*
    Advanced Explorer (c) 1999-2006 by D.G. Sureau

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    webmaster@scriptol.org
    http://www.scriptol.org
*/
package common;

import java.awt.*;
import common.*;

/**
   DPanel
   est un panel d'alignement sans fond
*/   

public class DPanel extends Container
{
    public static final int BEVEL_LOWERED = 0;
    public static final int BEVEL_RAISED = 1;
    public static final int BEVEL_LINE = 2;
    public static final int BEVEL_NONE = 3;

    public static final int LOWERED = 0;
    public static final int RAISED = 1;
    public static final int LINE = 2;
    public static final int NONE = 3;


    static final String base = "DPanel";
    int nameCounter = 0;
    String  name;

    //Color background;
    Color borderColor = Color.black;
    int border;

    /**
     * Creates a new DPanel. The default layout for all panels is FlowLayout.
     */

    public DPanel() { this(new FlowLayout()); }

    /**
     * Creates a new DPanel with the specified layout manager.
     * @param layout the layout manager for this DPanel
     */
    public DPanel(LayoutManager layout) { this(layout, BEVEL_NONE); }
    
    public DPanel(int b) { this(new FlowLayout(), b); }    
    
    public DPanel(LayoutManager layout, int b)
    {
       this.name = base + nameCounter++;
       setLayout(layout);
       border = b;
    }

    public void setBorder(int b) { border = b; }
    public void setBevelStyle(int bs) { border = bs; }
    public void setBorderColor(Color bc) { borderColor = bc; }

    public void update(Graphics g)
    { 
     paint(g); 
    } 

    public void paint(Graphics g)
    {
     super.paint(g);
     if(border == BEVEL_NONE) return;
     
     Color background = getBackground();

 	 Dimension s = getSize();
  	 int x = 0;
  	 int y = 0;
  	 int w = s.width - 1;
  	 int h = s.height - 1;

     if(border == BEVEL_RAISED)
     {
	   g.setColor(background.brighter());
	   g.drawLine(x, y, x + w, y);
	   g.drawLine(x, y, x, y + h);
	   g.setColor(background.darker());
	   g.drawLine(x, y + h, x + w, y + h);
	   g.drawLine(x + w, y, x + w, y + h);
     }
     else if(border == BEVEL_LOWERED)
     {
	  g.setColor(background.brighter());
	  g.drawLine(x, y, x + w, y);
	  g.drawLine(x, y, x, y + h);
	  g.setColor(background.darker());
	  g.drawLine(x, y + h, x + w, y + h);
	  g.drawLine(x + w, y, x + w, y + h);
     }   
     else if(border == BEVEL_LINE)
     {
      g.setColor(borderColor);
      g.drawRect(x, y, w, h);
     }  
   } // paint   
}
