/*
    Advanced Explorer (c) 1999-2006 by D.G. Sureau

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    webmaster@scriptol.org
    http://www.scriptol.org
*/
package common;

import java.io.*;
import java.awt.*;
import java.util.StringTokenizer;

public class DProperties
{
 VectorString keys;
 VectorString values;

 public DProperties(int size)
 {
   keys = new VectorString(size);
   values = new VectorString(size);

   //{{INIT_CONTROLS
		//}}
}

 public DProperties() { this(64); }

 /** Liste des lments */
 public void list(PrintStream ps)
 {
   for(int i = 0; i < keys.size(); i++)
   {
    ps.println(keys.elementAt(i) + "=" + values.elementAt(i));
   }
 }

 /** Recherche la valeur associe  une cl */

 public String getProperty(String key, String def)
  {
    int index = keys.indexOf(key);

    if(index == -1) return def;
    return values.elementAt(index);
  }

  // String
  public String get(String key, String dv)
  {
    if(dv.equals("null")) dv = null;
    return getProperty(key, dv);
  }


  // integer
  public int get(String key, int defaut)
   {
    try 
    { 
     String tmp = get(key, String.valueOf(defaut)); 
     return Integer.valueOf(tmp).intValue(); 
    }
    catch (NumberFormatException nfe) { } 
    return 0;
   } 

  // Boolean
  public boolean get(String key, boolean defaut)
   {
    try 
    { 
      String tmp = get(key, String.valueOf(defaut));
      return Boolean.valueOf(tmp).booleanValue(); 
    }
    catch (NumberFormatException nfe) {} 
    return false;
   } 

  // VectorString
  public VectorString get(String key, String first, String second)
  {
    VectorString vs = new VectorString(3);
    
    int index = keys.indexOf(key);
    if(index == -1)  // cl non trouve, prendre dfauts
    {
     if (first != null) vs.addElement(first);
     if (second != null) vs.addElement(second);
     return vs;
    }

    StringTokenizer st = new StringTokenizer(values.elementAt(index),
      " ,:\t\n\r", false);

    int size = st.countTokens();
    while (st.hasMoreTokens()) 
    {
      vs.addElement(st.nextToken());
    }
    return vs;
  }
  
 /** Convertit la valeur entire RGB en chane */
 public static String colorToString(Color c)
  {
     return Integer.toString(c.getRGB()); 
  }   

 /** Convertit un entier sous forme de chane en couleur */
  public static Color stringToColor(String s)
  {
     int index = ColorList.getIndex(s);     
     if(index == -1)
     {
      try 
      {  
       int rgb = Integer.valueOf(s).intValue(); 
       int r = ((rgb >> 16) & 255);  
       int g = ((rgb >> 8) & 255);          
       int b = (rgb & 255);           
       return new Color(r, g, b);
      }
      catch(Exception n) { n.printStackTrace(); }
      
      return Color.lightGray;
     }
    else
     return ColorList.getColor(index);
  }


  /**
     Diffrentes versions de lecture de proprit
  */   
  public String get(String key, Color c)
  {
     return get(key, colorToString(c));
  }  

  public String get(String key, SystemColor c)
  {
     return get(key, colorToString((Color) c));
  }  


  /**
      Style 
      Le defaut est l'tat courant
  */
  public String get(String key, Style d)
  {
   return get(key, d.getString());
  }  


  /**
    Ajoute une cl avec sa valeur concatnes 
    Fonction de chargement de fichier   
  */
  public void putItem(String str)
   {
     int j = str.indexOf("=");
     if(j > 0)
     {
      String lft = str.substring(0, j);
      String rgt = str.substring(j + 1, str.length());

      if(lft != null)
      {
       keys.addElement(lft);
       if(rgt == null) rgt = "";
       values.addElement(rgt);
      }
     }
  } // putItem

  /** Charge un fichier de proprits */

 public int load(String dir, String filename)
 {
   String path = Path.merge(dir, filename);
   int counter = 0;

   try
   {
    FileReader fr = new FileReader(path);
    BufferedReader br = new BufferedReader(fr);

    String s = "";  // pour entrer dans la boucle
    while(s != null)
     {
      try
      {
       s = br.readLine();  // lit ligne suivante
       if(s != null)
       {
         if(!s.startsWith("#"))  // passe commentaires
         {
          putItem(s);
          counter++;
         }
       }
      } catch(IOException rioe) { s = null; }
     }

     try { br.close(); } catch(IOException ioe) {}

    } catch (FileNotFoundException fnfe) { System.out.println(path + " not found"); ; }

    return counter;

 }


  /** Reconstruction */


  /** Pour un commentaire */
  public void put(String s)
  {
   keys.addElement(s);
   values.addElement((String) null);   // Pour l'indexation
  }
  public void put(String k, String val)
  { 
    keys.addElement(k); 
    values.addElement(val); 
  }
  public void put(String k, int val) { put(k, new Integer(val).toString()); } 
  public void put(String k, boolean b) { put(k, new Boolean(b).toString()); } 
  public void put(String k, Style obj) { put(k, obj.getString()); }   
  public void put(String k, Color obj) { put(k, colorToString(obj)); }   


  // Sauver
  
  public int save(String dir, String fname)
  {
   int count = 0;
   String path = Path.merge(dir, fname);
    
   try
   {
     FileWriter fw = new FileWriter(path);
     BufferedWriter bw = new BufferedWriter(fw);
    
     int size = keys.size();
     for(int i = 0; i < size; i++)
      {
       String s = keys.elementAt(i);
       String v = values.elementAt(i);
       if(v != null) s = s + "=" + v;   // Pour les commentaires 
       bw.write(s, 0, s.length());
       bw.newLine();
       count++;
      }
    
     bw.close();
   } catch(IOException ioe) { System.out.println("Error writing " + path); }
   
   return count;
 }   
 

  /** Effacer le fichier */
  public static boolean delete(String dir, String fName)
  {
   return (new File(dir, fName)).delete();   
  }

	//{{DECLARE_CONTROLS
	//}}
}
