/*
    Advanced Explorer (c) 1999-2006 by D.G. Sureau

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    webmaster@scriptol.org
    http://www.scriptol.org
*/
package common;

import java.awt.*;
import java.io.*;
import java.awt.event.*;


import common.VectorString;

public class DirManager extends Panel implements ActionListener, ComponentListener, KeyListener, MouseListener
{
  public Directory dir = new Directory(true);
  public ComboDir combo;
  public DirBox dirList;
  public DMessageBar statusBar;
  public History history = null;
  public Archiver archiver = null;
  public Temporary tempo = null;

  public String ZipName;
  public String m_dirName;
  public String m_fileName;
  public String m_pathName;

  public Label labelDirSize;

  public DToolBar toolbar;
  public DButton  buttonOpen;
  public DButton  buttonHome;
  public DButton  buttonParent;
  public DButton  buttonMake;
  public DButton  buttonDelete;
  public DButton  buttonCancel;

  public MenuItem miOpen;
  public MenuItem miRun;
  public MenuItem miEdit;
  public MenuItem miRename;
  public MenuItem miDelete;
  public MenuItem miInfo;
  public MenuItem miAddPrg;
  public MenuItem miAddDoc;
  public MenuItem miCopy;
  public MenuItem miUnarc;

  public Frame frame;
  public DPanel panelDirList;
  public Component vParent;    // parent visible pour popup
  public PopupMenu popup = null;

  public String MemName = "IZ.Memory";
  public String MemDir = System.getProperty("user.dir");

  public boolean associated = false;
  public boolean modified = false;

  public static final int SOURCE = 0;
  public static final int TARGET = 1;
  public static final int UNKNOW = 2;
  public int usage = UNKNOW;


  public DirManager(Frame f)
   {
    //System.out.println("enter dir man");
    frame = f;

    setLayout(new BorderLayout(0,0));
    setVisible(false);

    toolbar = new DToolBar();
    add(toolbar, BorderLayout.NORTH);

    panelDirList = new DPanel();
    panelDirList.setLayout(new BorderLayout(0,0));
    add(panelDirList);

    combo = new ComboDir();
    combo.setBounds(2,3,368,22);
    combo.setArrowStyle(true, false, 1);
    combo.setMaxItems(16);
    combo.setEditable(true);
    combo.setShowFocus(true);
    //combo.setFocusTraversalKeysEnabled(false);
    //combo.enableInputMethods(true);
    combo.addKeyListener(this);
    combo.addActionListener(this);
    combo.editBox.addKeyListener(this);
    panelDirList.add(combo, BorderLayout.NORTH);

    dirList = new DirBox(true);
    dirList.addActionListener(this);
    dirList.addMouseListener(this);
    dirList.addKeyListener(this);
    dirList.setFocusable(true);
    //dirList.setFrame(frame);
    panelDirList.add(dirList, BorderLayout.CENTER);

    labelDirSize = new java.awt.Label("");
    labelDirSize.setBounds(14,335,258,16);
    labelDirSize.setFont(new Font("SansSerif", Font.PLAIN, 14));
    add(labelDirSize, BorderLayout.SOUTH);

  // Panel source

    DButton.ADJUSTSIZE = false;
    DButton.setDefaultSize(36, 24);   // Force adjustsize

    buttonOpen = new DButton(Icons.DirOpen);
    buttonOpen.setTip("Open or process according to the extension");
    buttonOpen.addActionListener(this);
    toolbar.add(buttonOpen);

    buttonHome = new DButton(Icons.DirHome);
    buttonHome.setTip("Go to home directory");
    buttonHome.addActionListener(this);
    toolbar.add(buttonHome);

    buttonParent = new DButton(Icons.DirParent);
    buttonParent.setTip("Go to parent directory");
    buttonParent.addActionListener(this);
    toolbar.add(buttonParent);

    toolbar.setHeight(24);
    toolbar.addSeparator(11, 22);

    buttonMake = new DButton(Icons.DirMake);
    buttonMake.setTip("Create a new sub-directory");
    buttonMake.addActionListener(this);
    toolbar.add(buttonMake);

    buttonDelete = new DButton(Icons.DirDel);
    buttonDelete.setTip("Cross out / Delete selection");
    buttonDelete.addActionListener(this);
    toolbar.add(buttonDelete);

    buttonCancel = new DButton(Icons.Cancel);
    buttonCancel.setTip("Cancel selection / Refresh the display");
    buttonCancel.addActionListener(this);
    toolbar.add(buttonCancel);

    setPath(MemDir);

    addComponentListener(this);
    dir.jarMask = true;  // On affiche les jar

    addMouseListener(this);


 }


 public DirManager(Frame f, DMessageBar sb)
 {
   this(f);
   setObjets(sb);
 }



  public void setPopupParent(Component p, Font font)
  {

    popup = new PopupMenu();
    vParent = p;
    vParent.add(popup);

    Font fontBold = new Font(font.getName(), Font.BOLD, font.getSize());

    miOpen = new MenuItem("Open");
    miOpen.addActionListener(this);
    miOpen.setFont(fontBold);
    popup.add(miOpen);

    miRun = new MenuItem("Run");
    miRun.addActionListener(this);
    miRun.setFont(font);
    popup.add(miRun);

    miEdit = new MenuItem("Edit");
    miEdit.addActionListener(this);
    miEdit.setFont(font);
    popup.add(miEdit);

    popup.addSeparator();

    miInfo = new MenuItem("More info", new MenuShortcut(KeyEvent.VK_I));
    miInfo.addActionListener(this);
    //miInfo.addKeyListener(this);
    miInfo.setFont(font);
    popup.add(miInfo);

    miRename = new MenuItem("Rename", new MenuShortcut(KeyEvent.VK_N));
    miRename.addActionListener(this);
    miRename.setFont(font);
    popup.add(miRename);

    miDelete = new MenuItem("Delete");
    miDelete.addActionListener(this);
    miDelete.setFont(font);
    popup.add(miDelete);

    miAddPrg = new MenuItem("Add to Tools");
    miAddPrg.addActionListener(this);
    miAddPrg.setFont(font);
    popup.add(miAddPrg);

    miAddDoc = new MenuItem("Add to References");
    miAddDoc.addActionListener(this);
    miAddDoc.setFont(font);
    popup.add(miAddDoc);


    popup.addSeparator();

    miCopy = new MenuItem("Copy", new MenuShortcut(KeyEvent.VK_C));
    miCopy.addActionListener(this);
    miCopy.setFont(font);
    popup.add(miCopy);

    miUnarc  = new MenuItem("Extract", new MenuShortcut(KeyEvent.VK_U));
    miUnarc.addActionListener(this);
    miUnarc.setFont(font);
    popup.add(miUnarc);

  }


 public void setUsage(int u) { usage = u; } // source/target
 public int  getUsage() { return usage; }
 public void setObjets(DMessageBar sb) { statusBar = sb; }
 public void setHistory(History h) { history = h; }
 public void setFilename(String n) { MemName = n; }
 public void setArchiver(Archiver a) { archiver = a; }
 public void setTemporary(Temporary t) { tempo = t; }

 public void addNotify()
 {
   super.addNotify();
   combo.setBackground(getParent().getBackground());
 }


public void addButton(DButton b)
 {
  toolbar.add(b);
 }

/** Rafraichir si le chemin de sauvegarde est le chemin courant */
public void setModified(boolean mode)
{
  modified = mode;
  if (modified) dir.updated = false; // l'inverse
}


public void requestFocus() { dirList.requestFocus(); }
public void comboResize(int s) { combo.Resize(s); }

//------------------------------------ Preferences

  public void setStyle(Style s)
  {
    dirList.setStyle(s);
    combo.setStyle(s);

    Dimension newd = new Dimension();
    Dimension oldd = combo.getSize();
    newd.height = s.font.getSize();
    newd.width = oldd.width;

    combo.setSize(newd);
  }


//-------------------------------------- message

   public void message(String message)
   {
     dispTip(message);
     if( history != null) history.tell(message);
   }

   public void dispTip(String message)
   {
     if(statusBar != null)
      statusBar.setText(" " + message);
   }

   /**
      Test si un fichier coch,
      affiche message sinon
   */
   public boolean testSelected(int no)
   {
    if (dirList.isSelected()) return true;
    noSelected(no);
    return false;
   }


   public void noSelected(int no)
   {
     String[] win = { " in source", " in target", "" };
     dispTip("Select a file" + win[no] +", please...");
     Tool.beep();
   }

   public void noSelected()
   {
     noSelected(UNKNOW);  // pas de fentre donne
   }

   boolean error(String s)
     { System.out.println(s); return false; }


//-------------------------- fonction sur le chemin

   public String getDrive() { return dir.getDrive(); }
   public String getPath() { return dir.getPath(); }
   public String getDir()  { return dir.getDir(); }
   public int firstFile() { return dir.MAXINDIRS; }

   /*
      Change le chemin du rpertoire affich
      gre les filtres
      voir:  makePath: affecte les variables
   */
   public void setPath(String str)
   {
     if(str == null) str = MemDir;
     dir.setPath(str);
     updatePath(str);
   }


  public void updatePath(String str)
   {
    String suffix;
    try
     {
      if(dir.filter.length() > 0) suffix = " filter " + dir.filter;
      else suffix = "";
      dispTip("New path " + dir.getPath() + suffix);
     }
    catch(NullPointerException n) { dir.filter = ""; suffix = ""; }
  }

 public void makePath(String dir, String fic)
  {
    m_dirName = dir;
    m_fileName = fic;
    m_pathName = Path.merge(m_dirName, m_fileName);
  }


 // ------------------ fonctions sur la liste de fichiers

 public boolean selected()   { return isSelected(); }
 public boolean isSelected() { return dirList.isSelected(); }
 public boolean isTagged()   { return dirList.isTagged(); }

 public void setSelected(String s, boolean mode)
 {
   if(dirList.countItems() > 0) dirList.setSelectedItem(s, mode);
 }
 public void showIt(int no) { dirList.showIt(no); }
 public void setSelected(int i)
 {
   if(dirList.countItems() > 0)  dirList.setSelectedIndex(i, true);
 }
 public void selectCtrl(String fic) { dirList.selectMultiple(dirList.getIndex(fic), false, true); }
 public int getSelectedIndex() { return dirList.getSelectedIndex(); }
 public String [] getFiles() { return dirList.getFiles(getDir()); }
 public String [] getSelectedItems() { return dirList.getSelectedItems(); }
 public int [] getSelectedIndexes()  { return dirList.getSelectedIndexes(); }
 public String getSelected() { return dirList.getSelectedItem(); }
 public String[] getTaggedItems() { return dirList.getTaggedItems(); }
 public String getTagged() { return dirList.getTaggedItem(); }
 public void deselectAll() { dirList.deselectAll(); }
 public void selectAll() { dirList.selectAllFiles(firstFile()); }
 public void selectAny() { dirList.selectAll(); }
 public int countSelected() { return dirList.countSelected; }
 public void setArcSort(boolean b) { dir.setArcSort(b); }


 public String[] getListItems()
 {
    return dirList.getListItems();
 }

 public int countItems()
 {
   return dirList.countItems();
 }

 public void setPainting(boolean mode) { dirList.setPainting(mode); }
 public void update(boolean b) { dirList.update(b); }

 public void selectMultiple(int s)
 {
  dirList.selectMultiple(s, false, true);
 }

  // Teste si selection est rpertoire, et non multiple
   public boolean isDir()
   {
     int num[] = dirList.getSelectedIndexes();
     if (num.length > 1) return false;

     String dst = Path.merge(dir.getDir(), dirList.getSelectedItem());
     return new File(dst).isDirectory();
   }




 /**
      AtStart= true     redisp at start
             = false    redisp at same location
 */

 public void refresh(boolean atstart)
  {
   int no;
   if (atstart) no = 0;
   else no = dirList.getTopRow();

   dirDisp(no);
   showIt(no);
  }

 public void dirDisp(boolean atStart)
  {
   int no;
   if (atStart) no = 0;
   else no = dirList.getTopRow();
   //System.out.println("call dirdisp no=" + no);
   //System.out.println("dir=" + dir.getDir());
   dirDisp(no);
  }

 public void dirDisp(int no)
  {
    String labdir;
    String labfile;
    String labmask;
    String labsize;

    //error("filter " + dir.filter);
    //error("dir " + dir.getDir());
    //error("path " + dir.getPath());
    //System.out.println("enter dirdisp/no " + no);
    //System.out.println("DirManager dirDisp()");
    dir.dispDir(dirList, dir.filter, no);    // Noter disp dir

    //System.out.println("passed call dispdir");

    if(dir.MAXINDIRS > 1) labdir = " directories   ";
    else labdir = " directory   ";
    if(dir.MAXINFILES > 1) labfile = " files";
    else labfile = " file";
    if(dir.Masked > 0 ) labmask = "   " + dir.Masked + " hidden";
    else labmask = "";

    long size = dirList.TotalSize;
    if(size > 10485760) labsize = size / 1048576 + " megas used";
    else labsize = size + " bytes used";
    labelDirSize.setText(" " + dir.MAXINDIRS + labdir + dir.Displayed + labfile + labmask + "   " + labsize);

    combo.setText(dir.getPath());
    modified = false;
  }


//----------------------------- Affichage / dplacement

 public void parent()
  {
   String current = dir.goParent();
   updatePath(dir.getPath());



   dirDisp(true);

   if(current.length() > 0)
   {
     dirList.showIt(current);
   }
   else  dirList.showIt(0);


  }

 public void home()
  {
    dir.goHome();
    updatePath(dir.getPath());
    dirDisp(true);
    setSelected(0);
  }


 // ----------------------------------- Gestion de fichiers

 /** Creation de sous-rpertoire
    direct: raffichage ici sinon envoyer "md"
 */

 public String dirmade = "";
 public void make() { make("Creating a sub-directory"); }

 public void make(String title)
  {
    String name;

    if (isInside()) return; // dans une archive
    dispTip(title);

    Input md = new Input(frame);
    md.setTitle(title);
    md.setLabel("Type a name for the sub-directory to create:");
    md.setInput("");
    md.setVisible(true);

    name = md.getInput();

    if(md.ok())
     {
        File f = new File(dir.getDir(), name);

        if(f.mkdir())
        {
         message(name + " directory created");
         modified = true;
         dirmade = name;
         if (usage != TARGET) dirDisp(true);
         sendAction("md");
        }
        else dispTip("Not made");
      }
     else dispTip("Cancelled");
   }


 /** Refresh */
 public void cancel()
 {
    dirDisp(true);
 }



 public boolean isInside()
 {
    if(dir.zipMode)
     {
       dispTip("This can't be performed inside archive");
       return true;
     }
    return false;
 }




 public void moreInfo()
   {
     if (isInside()) return;

     String dirname = dir.getDir();
     String l[] = dirList.getSelectedItems();

     if(l.length == 1)
     {
      String name = dirList.getSelectedItem();
      if (new File(getDir(), name).isDirectory()) ; // rien
      else
      {
        message(DirTools.info(Path.merge(dirname, name)));
        return;
      }
     }

     int mode = DirTools.SELECT;

     dirList.setCursor(new Cursor(Cursor.WAIT_CURSOR));
     dispTip("calculating...");


     if (l.length == 0)
     {
        l = dirList.getListItems();
	mode = DirTools.FULL;
     }

     DirTools.occupation(dirname, l, mode);
     message(DirTools.message);
     dirList.setCursor(Cursor.getDefaultCursor());
   }




  /** Ouvre fichier ou inside archive */
 public boolean open()
 {
   if(!selected()) return false;
   makePath(dir.getDir(), dirList.getSelectedItem());
   // On est dja dans un zip
   if(dir.zipMode == true) return openInside(false);
   return openCommon(true);
 }

 /** Ouvre un fichier  l'intrieur d'une archive */

  public boolean openInside(boolean program)
 {
   message("Viewing " +  m_fileName + " inside " + dir.arcname);

   // Rp archive, archive, fichier  extraire
   if (tempo == null)
   {
     System.out.println("Temporary dir not created");
     return false;
   }
   //tempo.setNames(m_dirName, dir.arcname, m_fileName)
   //Disp.it("attemps to create " + m_dirName + " " + dir.arcname + " " + m_fileName);
   if(tempo.open(m_dirName, dir.arcname, m_fileName)) // si fichier extrait
    {
     //Disp.it( tempo.createdPath + " x " + tempo.createdFile );

     if (program)
       executable(tempo.createdPath, tempo.createdFile);
     else
       openPath(tempo.createdPath, tempo.createdFile, false, false); // lire sauf exe

    }
   else
     message(tempo.message);

   return tempo.result;
  }

 /** Ouvre fichier ou rpertoire */

  public boolean openCommon(boolean addit)
 {
   File which = new File(m_dirName, m_fileName);

   // C'est un rpertoire
   if(which.isDirectory())
    {
      dispTip("Directory");
      setPath(m_pathName);
      dirDisp(true);
      setSelected(0);
      dirList.requestFocusInWindow();
      return true;
    }
   return openAny(m_pathName, m_fileName, true, true);
  }


  /** Ouvre zip ou autre */
  public boolean openAny(String path, String name, boolean addit, boolean withexe)
  {
   // Fichier ZIP
   //System.out.println("enter any");
   if(Directory.isArchive(path)) { return openZip(); }
   //System.out.println("call open path");
   openPath(path, name, addit, withexe);
   return true;
 }


  /* ouvre tout fichier comme zip  sauf dir */
  public void viewZip()
   {
    if (!testSelected(0)) return;

    File f = new File(dir.getDir(), dirList.getSelectedItem());
    if (f.isDirectory())
    {
     noSelected();
     return;
    }
    openZip();
   }

  /** Ouvre archive */

  public boolean openZip()
   {
    message("Opening archive "+ m_fileName);

    makePath(dir.getDir(), dirList.getSelectedItem());
    ZipName = m_fileName;
    dir.setArchive(true);
    setPath(m_pathName);

    setCursor(new Cursor(Cursor.WAIT_CURSOR));
    dirDisp(true);
    setCursor(Cursor.getDefaultCursor());

    if(dir.zipMode)  // russi
     {
       message("Inside archive " + m_fileName);
       showIt(0);
       return true;
     }

    ZipName = "";
    setPath(m_dirName);
    message("Format not supported");
    ( new DialogMessage(frame, "Zip format not supported")).setVisible(true);
    return false;

  }


  public static String graflist = ".gif.jpg.ico.raw.pal.aaz.bmp.pcx.pic.tga.png.xbm.tiff.jpeg.jpe.";
  public static String others = ".class.hlp.";
  public static String soundlist = ".wav.au.aif.";


  /**
    Cherche l'extension dans la liste donne
  */

  public static boolean inside(String str, String search)
  {
   if (search == null) return false;
   if (search.equals("")) return false;

   String ext = search.toLowerCase();
   if (!ext.startsWith(".")) ext = "." + ext;
   if (!ext.endsWith("."))   ext = ext + ".";
   //System.out.println(str + " " + ext);
   int i = str.indexOf(ext);
   //System.out.println("index=" + i);
   return (i != -1);
  }


 /* Ouvre fichier, selon extension ou type */
  public boolean addit = false;

  public void openPath(String path, String name, boolean addit, boolean withexe)
 {
    //System.out.println("enter openpath " + path + " " + name);
    this.addit = addit;

    path = path.toLowerCase();

    m_pathName = path;
    m_fileName = name;

    //System.out.println("open path");
    if(Preferences.AssFirst)
     {
       System.out.println("first");
       associated = false;
       sendAction("assoc");
       if (associated) return;
     }

    // Built-in
    if(DirTools.isExecutable(name))
     {
        //System.out.println("exec");
        if (withexe)
         executable(m_pathName, name);
        // sendAction("run");
        return;
     }

    String extension = Path.getExtension(name);
    //System.out.println("ext=" + extension);

    if(inside(graflist, extension))
    {
      sendAction("graph");
      return;
    }

    // Fichiers son
    if (inside(soundlist, extension))
    {
      //System.out.println("snd");
      sendAction("sound");
      return;
    }


    if(!Preferences.AssFirst)
    {
      //System.out.println("last");
      associated = false;
      sendAction("assoc");
      if (associated) return;
    }

    //System.out.println("send text");
    sendAction("text");
    return;
 }



  public boolean openRun()
   {
    if(!selected()) return false;

    makePath(dir.getDir(), dirList.getSelectedItem());

    File which = new File(m_dirName, m_fileName);
    if(which.isDirectory()) return false;

   // On est dja dans un zip
    if(dir.zipMode == true) return openInside(true);

    return executable(m_pathName, m_fileName);
   }

  public boolean executable(String path, String fName)
  {

    if(DirTools.isMSDOS(path))
     {
       dispTip(fName + " is a MS-DOS program. Run it with Explorer");
       /*
       String prefix = "";
       if (OS.isWindowsNT()) prefix = "cmd /c ";
       else prefix = "command.com /c ";
       path = prefix + path;
       */
       return false;
     }

    if(DirTools.isCOMMAND())
     {
       dispTip(fName + " is a command, open the DOS window and type it");
       return false;
     }

    String winver = "";
    if(DirTools.isWIN31())
     {
        winver = "Windows 3.1 program ";
        //System.out.println(winver);
     }

    message("Launching " + winver + fName);
    launch(path, fName);
    return true;
   }



/*
   void sound(String path, String name)
   {
     run("mplayer", path, name);
   }
*/
  Process process;

  public void launch(String fullpath, String name)
  {
    //System.out.println("launch " + fullpath + " " + name);
    //Runtime rc = Runtime.getRuntime();
    boolean ok = false;

    String message = "Enable to run " + fullpath;

    try { process = Runtime.getRuntime().exec(fullpath);
    try { process.waitFor(); } catch (InterruptedException i) { }
     ok = true;
    }
    catch (IOException ioe) { ioe.printStackTrace(); };

    if(!ok) message(message);
  }


   /** Lance un programme */

  public boolean run(String prog, String path, String name)
   {
    Runtime rc = Runtime.getRuntime();
    boolean result = false;
    String message = "Enable to run " + prog + " with " + path;

    try
    {
     Process process = Runtime.getRuntime().exec(prog + " " + path);
     try { process.waitFor(); } catch (InterruptedException i) { }
     message = "Launched " + prog + " with " + path;
     result = true;
   }
   catch (IOException ioe) { };

   message(message);
   return result;
 }


  String pluriel(int number)
  {
   if (number > 1) return "s";
   return "";
  }

 public void crossOut()
   {
     boolean changed = false;
     int m = dirList.countItems();
     int crossed = 0;

     for(int i = 0; i < m; i++)
      {
       if(dirList.isSelected(i))
        {
         dirList.setDeselected(i);
         dirList.crossOut(i);
         changed = true;
         crossed++;
        }
      } // fin for

     if(changed)
      {
       dirList.blockPaint(false);
       message(crossed + " file" + pluriel(crossed) + " crossed out for deletion");
      }
     else noSelected();

 }


  /** Efface un fichier avec prompt */
 public int deleteConditional(String curdir, String fic)
   {
     Deleter.message = fic + " not deleted";

     int ret = Deleter.conditional(frame, curdir, fic);
     if(ret < 1) { message( Deleter.message ); return 0; }
     if(ret == 1) { message(fic + " deleted"); }
     return ret;
   }

 public void deleteSecond()
 {
      int counter = 0;
      int counterHere = 0;
      String curdir = dir.getDir();

      dispTip("Deleting crossed out files");

      int m = dirList.countItems();
      int index = -1;
      int toprow = dirList.getTopRow();

      for(int i = 0; i < m; i++)
      {
       if(dirList.isCrossOut(i))   // si soulign
        {
          dirList.crossOut(i, false); // en cas d'erreur
          counterHere++; // pour savoir si on raffiche
          counter += deleteConditional(curdir, dirList.getText(i));
          if(index == -1) index = i;
          if(i < toprow) toprow = i;
        }
      }

     if(counter > 0)
     {
      m = Math.max(0, m - counter);
      index = Math.min(index, m - 1);
      index = Math.max(0, index);

      if (counter > 1)
         message(counter + " deleted");

      dirDisp(dirList.getTopRow());
      setSelected(index);
      //System.out.println("after " + index);
      sendAction("modified");
     }
     //else dirList.repaint();
   }

    void delete()
    {
      if (isInside()) return;
      if(dirList.isCrossOut())  deleteSecond();
      else crossOut();
    }


   boolean rename()
   {
      if (isInside()) return false;

      boolean renFlag = false;

      int index = dirList.getSelectedIndex();
      String name = dirList.getSelectedItem();
      String direct = dir.getDir();

       //debug("name=" + name + " index=" + index);
      if(index == -1) {Tool.beep(); return false; }
      if(name == null || name.length() == 0) { Tool.beep(); return false; }

      Input ren = new Input(frame);
      ren.setTitle("Rename a file or a directory");
      ren.setLabel("Type the new name:");
      ren.setInput(name);
      //debug("setinput=" + ren.getInput());
      ren.setVisible(true);

      //debug("rename choice=" + ren.ok() + " input=" + ren.getInput());
      if(ren.ok())
       {
        File oldname = new File(direct, name );
        File newname = new File(direct, ren.getInput());
        try
        {
         if(oldname.renameTo(newname) == true)
         {
           dirList.setText(index, newname.getName());
           renFlag = true;
         }
         else renFlag = false;
        }
        catch(SecurityException e) { renFlag = false; }

       if(renFlag)
        {
         message(name + " renamed " + newname.getName());
         sendAction("modified");
        }
       else
         message("Enable to rename "+ name + " to " + newname.getName());
      }

      return true;
   }


public boolean unArchive(String destination)
  {
   makePath(getDir(), getSelected());

   Archiver.setMessage("Error");

   if(dir.zipMode == true)
    {
      String list[] = dirList.getSelectedItems();
      int l = list.length;
      if(l > 0)
      {
       String things;
       if(l > 1) things = "tagged list";
       else      things = m_fileName;
       message("Extracting " + things + " from " + dir.arcname);
       String arc = Path.merge(m_dirName, dir.arcname);
       for(int i = 0; i < l; i++)
       {
        String fic = list[i];
        if(!Archiver.extractFile(arc, fic, destination, Preferences.ArcRestore))
         message(Archiver.getMessage());
       }
       return true;
      }
      else dispTip("Select entries to unarchive, please!");
      return false;
    }


    if(Path.isDirectory(m_pathName))
     {
       noSelected();
       return false;
     }

    dispTip("Unarchiving "+ m_pathName);

    int num;

    //debug("format gzip " + Directory.isGZip(m_pathName));

//    if (Directory.isGZip(m_pathName))
//     num = Archiver.extractGZip(m_pathName, dstdir.getDir());
//    else
     Point p = archiver.unArchive(m_pathName, destination);

     //System.out.println("return archiver " + p.x + " " + p.y);
    //debug("unarc num="+ num);
     if (p.y == 0)
     {
      message(Archiver.getMessage());
      Tool.beep();
     }
     else
     {
      String prefix = "";
      if (p.y < p.x) prefix = "Error - ";
      message(prefix + m_pathName + " unarchived in " + p.y + "/" + p.x  + " file(s)");
     }

    if(Archiver.error)
     {
      (new DialogMessage(frame, "Error, not unarchived!")).setVisible(true);
     }

    return true;
 }


//------------------------------------ Path Bookmark


  /*
     ajoute directement dir aux favoris
  */

public boolean testSelected()
  {
    makePath(dir.getDir(), dirList.getSelectedItem());
    if (m_pathName == null) return false;
    if (new File(m_pathName).isDirectory()) return false;
    return true;
  }

public void addPrg()
  {
    if (!testSelected()) return;
    sendAction("addprg");
  }

public  void addDoc()
  {
    if (!testSelected()) return;
    sendAction("adddoc");
  }

  /*
  void favorites()
  {
    if (isInside()) return;

    File f = new File(dir.getDir(), dirList.getSelectedItem());
    String path;
    if (f.isDirectory())
      path = Path.merge(dir.getDir(), dirList.getSelectedItem());
    else
      path = dir.getDir();
    pathAdd(path);
    dispTip(path + " added to list of paths");
  }


    void pathAdd(String path)
    {
      combo.addUnicItem(path);
	  combo.separeDrives();
    }
   */

    /** Ajoute */
public void setPathCombo()
 {
    String entry = Path.setCanonical(combo.getText());
    setPath(entry);
    dirDisp(true);
 }


 /** Supprime */
public void pathDelete()
 {
       message("Removing " + dir.getPath() + " from list");
       combo.deleteInList();
       combo.setBaseSize();
       setPath(combo.getText());
       dirDisp(true);
 }


/*
void diskSpace()
{
  System.out.println("enter");

  String str;
  if (OS.isWindowsNT()) str = "cmd /c dir";
  else
  if (OS.isWindows()) str = "command /c dir";
  else return;

  System.out.println(str);

  try
  {
   Process p = Runtime.getRuntime().exec(str);
   DataInputStream procIn =
     new DataInputStream(p.getInputStream());

   while ( true )
   {
    String line = procIn.readLine();
    if (line == null ) break;
    message(line);
   }
  }
  catch(Exception e) {}
}
*/


public void showMenu()
  {
    if (vParent == null) return;
	popup.show(dirList, dirList.popx, dirList.popy);
	return;
  }

//------------------------------------------ Inner classes

      // File Manager
public void actionPerformed(java.awt.event.ActionEvent event)
 {
    Object object = event.getSource();

    if (object == buttonParent)          parent();
    else if (object == buttonHome)   home();
    else if (object == buttonOpen)   open();
    else if (object == buttonMake)   make();
    else if (object == buttonDelete) delete();
    else if (object == buttonCancel) cancel();
    else if (object == combo)
    {
       String command = event.getActionCommand();
       //Disp.it("com=" + command);
       if (command != null)
       {
         if(command.equals("del")) { pathDelete(); return; }
       }

       //Disp.it(combo.getSelectedItem());
       setPath(combo.getSelectedItem());
       dirDisp(true);
       dirList.showIt(0);
       return;
    }
        else if(object == dirList)
         {
           String command = event.getActionCommand();
           if(command.equals("Enter"))  { open();   return; }
           if(command.equals("Popup"))  { showMenu(); return; }
           if(command.equals("Delete")) { delete(); return; }
           if(command.equals("Esc"))    { cancel(); return; }
           if(command.equals("Up"))     { parent(); return; }
         }
        else if(object == miOpen)   open();
        else if(object == miRun)    openRun();
        else if(object == miEdit)   sendAction("edit");
        else if(object == miRename) rename();
        else if(object == miDelete) delete();
        else if(object == miInfo)   moreInfo();
        else if(object == miCopy)
        {
          if(dir.zipMode) sendAction("unarc");
          else          sendAction("copy");
        }
        else if(object == miUnarc)  sendAction("unarc");
        else if(object == miAddPrg) addPrg();
        else if(object == miAddDoc) addDoc();
        return;
}


  transient ActionListener actionListener;

  /**
    * Adds the specified action listener to receive action events
    * from this button.
    */

  public void addActionListener(ActionListener l)
   {
	actionListener = AWTEventMulticaster.add(actionListener, l);
   }

  public void sendAction(String command)
  {
    if (actionListener != null)
	  actionListener.actionPerformed(new ActionEvent(this, ActionEvent.ACTION_PERFORMED, command));
  }

  KeyListener keyListener;

  public void addKeyListener(KeyListener k)
  {
     keyListener = AWTEventMulticaster.add(keyListener, k);
  }


  // TOUCHES du CLAVIER
  // AEKey  KeyListener

  public void keyTyped(java.awt.event.KeyEvent event) { }
  public void keyReleased(java.awt.event.KeyEvent event) { }
  public void keyPressed(java.awt.event.KeyEvent event)
   {
    Object object = event.getSource();
    int keypressed = event.getKeyCode();

    if (event.isAltDown())
    {
     switch (keypressed )
     {
       case KeyEvent.VK_E:
       default:
            if( keyListener != null)
               keyListener.keyPressed(event);
            break;
     }
   }


   if(object == combo.editBox)
   {
     if(keypressed == KeyEvent.VK_ENTER)
      {
        setPathCombo();
        event.consume();
        return;
       }

      if(event.isControlDown())
       {
   	 if(keypressed == KeyEvent.VK_A)
   	 {
           combo.addUnicItem(combo.editBox.getText());
 	   combo.separeDrives();
   	   event.consume();
   	   return;
   	  }

   	  if((keypressed == KeyEvent.VK_Y)
   		  || (keypressed == KeyEvent.VK_D))
          {
   	    pathDelete();
   	    event.consume();
   	    return;
   	  }
       }
    }
    else if(object == dirList)
    {
      if(event.isControlDown())
	{
         switch(keypressed)
          {
          // comme les touches sont associes dans les menus
          // le fait d'appuyer excute les fonctions du menu!
          // la fonction consume vite cet effet...
     	   case KeyEvent.VK_N:    rename(); break;
           case KeyEvent.VK_K:
     	   case KeyEvent.VK_M:    make(); break;
     	   case KeyEvent.VK_I:    moreInfo(); break;
     	   //case KeyEvent.VK_F: diskSpace(); break;
           default: return;
          }
        event.consume();
       }
   } // else
 }




 public void mouseClicked(java.awt.event.MouseEvent event)
  {
	  Object object = event.getSource();

          int click = event.getClickCount();
          if (click == 2)
          {
		 if (object == dirList)	 open();
  	  }
   }

 public void mousePressed(MouseEvent e) { }
 public void mouseReleased(MouseEvent e) { }
 public void mouseEntered(MouseEvent e)  {}
 public void mouseExited(MouseEvent e)  {}



/**---------------------------------------------------
   bookmarks
   ---------------------------------------------------*/

  public void save()
  {
    String[] MemorySrc = combo.getListItems();


    int ls = MemorySrc.length;
    int i;
    int size = 16;

    if(ls > size) ls = size;

    VectorString vmem = new VectorString();

    vmem.addElement("SEB=" + getDir());

    for(i = 0; i < ls; i++)
     vmem.addElement("BKM=" + MemorySrc[i]);


    int result = Disk.saveLines(MemDir, MemName, vmem);

  }


  // Load memory

 public boolean load(String dir, String name)
  {
    MemDir = dir;
    return load(name);
  }

  public boolean load(String name)
  {
    MemName = name;
    return load();
  }

  public boolean load()
  {
    VectorString src   = new VectorString();
    VectorString vlist = new VectorString();
    int i;
    int loaded;

    int maxsize = 16; // Nombre restant

    // Charger la liste complte sur fichier
    loaded = Disk.loadUnicLines(MemDir, MemName, vlist);

    int size = vlist.size();  // Taille de cette liste
    for(i = 0; i < size; i++)
    {
     String test = vlist.elementAt(i);

     int j = test.indexOf("=");
     if(j > 0)
     {
      String lft = test.substring(0, j);
      String rgt = test.substring(j + 1, test.length());

      if(lft != null && rgt != null)
      {
      if(rgt.length() < 1) System.out.println(lft + " Error: Empty field");
      else if(lft.equals("BKM")) src.addElement(rgt);
      else if(lft.equals("SEB")) setPath(rgt);
      }
     }
    } // for


    combo.clear();
    combo.setMaxItems(16);


    if(size < 1) return false;  // Test si AE.Memory pas charge

    // Mets les units en tte et les ordonne
    int lbs = src.separeDrives();
    combo.setBaseSize(lbs);


    int l = src.size();
    for(i = 0; i < l; i++)
       combo.addItem(src.elementAt(i));

   return true;
 } // load


 /** Setup tardif, correctif */
 /*
 public void addDrives(VectorString diskslist)
 {
    int bsize = diskslist.size();
    //System.out.println("size=" + bsize);
    for(int i = 0; i < bsize; i++)
    {
     combo.addUnicItem(diskslist.elementAt(i));
    }

    combo.setBaseSize(bsize);
    combo.separeDrives();
  }
  */

 public void componentShown(ComponentEvent e)
 {
    String dirname = dir.getDir();

    if (dirname.equalsIgnoreCase(m_dirName) == false)
    {
      modified = true;
    }
    if (modified) refresh(false);
 }

 public void componentHidden(ComponentEvent e) { }
 public void componentMoved(ComponentEvent e) { }
 public void componentResized(ComponentEvent e) { }


}
