/*
    Advanced Explorer (c) 1999-2006 by D.G. Sureau

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    webmaster@scriptol.org
    http://www.scriptol.org
*/
package common;

/**
   Cette classe trie un tableau de chaine.
   Elle ne trie pas directement les chaines, mais
   produit un index ordonn
   bas sur l'ordre des chaines de caracteres
*/   


public class Sort
{

 int idx[] = null;
 String base[] = null;


 /**
    Constructeur
    Vous utilisez le tableau de chane de la classe Sort
    Le tableau est cr par le constructeur.
    Usage:
     Sort = new Sort(size);
     ... remplissage...
     sort.tri();
 */
 
 public Sort(int size)
 { 
   base = new String[size]; 
   makeIndex(); 
 		
		//{{INIT_CONTROLS
		//}}
};
 
 /**
    Constructeur
    Vous utilisez votre propre tableau de chane
 */   
 public Sort(String[] sa)
 {
  base = sa;
  makeIndex();
 }

 /**
    Constructeur
    Vous utilisez votre propre tableau de chane
    et votre index dja rempli
 */   
 public Sort(String[] sa, int[] idx)
 {
  base = sa;
  this.idx = idx;
 }

 

/*
  supercmp return an integer value such as:
   < 0     if str1 is less than str2
   = 0     if str1 is the same as str2
   > 0     if str2 is greater than str2
*/
/*
static char table[] = {
0,1,2,3,4,5,6,7,8,9,0,11,12,0,14,15,16,17,18,19,20,21,22,23,24,25,26,
'','','','','',' ','!','\"','#','$','%','&','\'','(',')','*','+',',','-','.','/',
'0','1','2','3','4','5','6','7','8','9',
':',';','<','=','>','?','@',
'a','b','c','d','e','f','g','h','i','j','k','l','m',
'n','o','p','q','r','s','t','u','v','w','x','y','z',
'[','\\',']','^','_','`',
'a','b','c','d','e','f','g','h','i','j','k','l','m',
'n','o','p','q','r','s','t','u','v','w','x','y','z',
'{','|','}','~','','c','u','e','a','a','a','a','c','e','e','e','i','i','i',
'a','a','e','?','?','o','o','o','u','u','y','o','u','?','?','','','?','a',
'i','o','u','n','n','a','o','','','','','','','','','','','','',
'','','','','','','','','','','','',
'','','','','','','','','','','','','','','','','','','',
'','','','','','','','','','','','','','','','','','','',
'','','','','','','','','','','','','','','','','',
'','','','','','','','','',
};
*/

/**
  * Compare deux chaines
  * Retourne 0   si gales
  *          < 0 si a avant b
  *          > 0 si a aprs b
  */

static int superCompare(String str1, String str2)
{
 char a;
 char b;
 int l1 = str1.length();
 int l2 = str2.length();
 int n = Math.min(l1, l2); // teste jusqu' la fin d'une chane

 for(int i = 0; i < n; i++)
 {
//  a = table[str1.charAt(i)];
//  b = table[str2.charAt(i)];
  a = Character.toLowerCase(str1.charAt(i));
  b = Character.toLowerCase(str2.charAt(i));

  // Si a > b, chaine 1 aprs chane 2
  if(a != b) return ((int) a - (int) b);
 }

 // Si chane 1 plus longue que chane 2, aprs elle
  return (l1 - l2);
}


/** Ignore case and accents */   

static boolean superEquals(String str1, String str2)
{
 int i = str1.length();
 if(i != str2.length()) return false;

 char a;
 char b;

 // Si mme longueur...
 while(i > 0)
  {
   i--; 
   a = Character.toLowerCase(str1.charAt(i));
   b = Character.toLowerCase(str2.charAt(i));
   if(a != b)  return false;
  }

  return true;
}



 /** Fill the index array */

 protected void makeIndex()
 {
  int l =  base.length;
  idx = new int[l];
  for(int i = 0; i < base.length; i++) idx[i] = i;
 }

 /** Construit un nouveau tableau de chane ordonn */  
 
 String[] rebuild()
 {
  int l = base.length;  
  String temp[] = new String[l];
  
  for(int i = 0; i < l; i++)
   temp[i] = base[ idx[i] ];
  
  return temp;  
 }   

 /**  Compare deux chanes pointes */  
 int compare(int a, int b)
 {
  return superCompare(base[idx[a]], base[idx[b]]);
 }

 /**  Compare deux chanes  */  
 int compare(String a, String b)
 {
  return superCompare(a, b);
 }


 /** Echange les numros d'indexation */   
 
 void swap(int a, int b)
 {
  int t = idx[a];
  idx[a] = idx[b];
  idx[b] = t;
 }

 /**
    Tri selon shell
    non test
 */   
 void shell(int nelem)
 {
  int gap = 1;
  int i, j;

  while(gap < nelem)
  {
    gap = gap * 3;
    gap ++;
  }

  while((gap /= 3) > 0)
   {
   for (i = gap; i < nelem; i ++)
    {
     j = i - gap;
     if (compare(i, i + gap) <= 0 )  break;
     swap(i, i + gap);
     if (j < gap) break;
     j = j - gap;
   }
  }
 } // sort



/*
   Insertion Sort
   from start to limit not included
*/

 public void insertion() { insertion(0, base.length); }
 public void insertion(int start, int limit)
 {
  int i, j;

  j = start;
  i = j + 1;
  while(i < limit)
   {
    while((j >= start) && compare(j, j + 1) > 0)
     {
      swap(j, j + 1);
      j--;
     }

    j = i;
    i++;
   } // jusqu' lim
} // insertion



  // Trie Contenu

  public void bubble() { bubble(0, base.length); }
  public void bubble(int st, int limit)
  {
    int j;

	while (st < limit)
	{

 	  limit--;

	  boolean swapped = false;

	  for (j = st; j < limit; j++)
	   {
		if((compare(j, j + 1) > 0))
		 {
	       swap(j, j + 1);
		   swapped = true;
		 }
	   } // fin for

	  if(swapped == false) return;

      for (j = limit; --j >= st;)
       {
		if((compare(j, j + 1) > 0))
		 {
		   swap(j, j + 1);
		   swapped = true;
		}
 	   } // fin for

	  if(swapped == false) return;

     st++;

	} // fin while

	return;
  }

  /**
       Get a string
  */     
  String get(int no)
  {
   return base[idx[no]];
  } 

  static final int Threshold = 16;

  /**
       Quick sort
  */     
  public void quick() { quick(0, base.length); }
  public void quick(int st, int limit)
    {
        
     while(limit - st > Threshold)
      {
      String pivot;
      String a = get(st);
      String b = get(st + (limit - st ) / 2);
      String c = get(limit - 1);

      if(compare( a, b ) < 0)
       {
        if(compare(b, c) < 0)       pivot = b;
        else if(compare(a, c ) < 0) pivot = c;
        else                        pivot = a;
       }
      else if(compare(a, c ) < 0) pivot = a;
      else if(compare(b, c ) < 0) pivot = c;
      else                        pivot = b;

      int cut   = st;
      int lastl = limit;

      while(true)
        {
        while(compare(get(cut), pivot) < 0) ++cut;

        --lastl;

        while (compare(pivot, get(lastl)) < 0) --lastl;

        if(cut >= lastl ) break;

        swap(cut, lastl);
        cut++;
        }

      if((cut - st) >= (limit - cut))
       {
        quick(cut, limit);
        limit = cut;
       }
      else
       {
        quick(st, cut);
        st = cut;
       }
      }
    } // quick


	
	//{{DECLARE_CONTROLS
	//}}
};

/*
void ssort (void  *base,
            size_t nel,
            size_t width,
            int (*comp)(const void *, const void *))
{
      size_t wnel, gap, wgap, i, j, k;
      char *a, *b, tmp;

      wnel = width * nel;
      for (gap = 0; ++gap < nel;) gap *= 3;
      while ( gap /= 3 )
      {
            wgap = width * gap;
            for (i = wgap; i < wnel; i += width)
            {
                  for (j = i - wgap; ;j -= wgap)
                  {
                        a = j + (char *)base;
                        b = a + wgap;
                        if ( (*comp)(a, b) <= 0 )
                              break;
                        k = width;
                        do
                        {
                              tmp = *a;
                              *a++ = *b;
                              *b++ = tmp;
                        } while ( --k );
                        if (j < wgap)
                              break;
                  }
            }
      }
}
*/





/*

   void QuickSort(int a[], int lo0, int hi0) throws Exception
   {
      int lo = lo0;
      int hi = hi0;
      int mid;

      if ( hi0 > lo0)
      {

         mid = Idx[ ( lo0 + hi0 ) / 2 ];

         // loop through the array until indices cross
         while( lo <= hi )
         {
	     while( ( lo < hi0 ) && pauseTrue(lo0, hi0) && ( a[lo] < mid ))
		 ++lo;

	     while( ( hi > lo0 ) && pauseTrue(lo0, hi0) && ( a[hi] > mid ))
		 --hi;

            // if the indexes have not crossed, swap
            if( lo <= hi )
            {
               swap(a, lo, hi);
               ++lo;
               --hi;
            }
         }

         if( lo0 < hi )           QuickSort( a, lo0, hi );

         if( lo < hi0 )           QuickSort( a, lo, hi0 );

      }
   }
*/



/*
    void QSort(int lo0, int hi0) throws Exception
    {
      int lo = lo0;
      int hi = hi0;
      int mid;

      if ( hi0 < lo0 )
      {
         // Arbitrarily establishing partition element as the midpoint of the array.

         mid = ( lo0 + hi0 ) / 2;

         // loop through the array until indices cross
         while( lo <= hi )
         {
         // find first element >= to the partition element starting from the left Index.
	     while( ( lo < hi0 ) && ( QSCompare( lo, mid) < 0 ))  ++lo;

         // find element <= to the partition element starting from the right Index.
	     while( ( hi > lo0 ) && ( QSCompare( hi, mid ) > 0 )) --hi;

         // if the indexes have not crossed, swap
           if( lo <= hi )
            {
               SwapList(lo, hi);
               ++lo;
               --hi;
            }
         }

         // If the right index has not reached the left side of array
         // must now sort the left partition.

         if( lo0 < hi )  QSort(lo0, hi );

         // If the left index has not reached the right side of array
         // must now sort the right partition.

         if( lo < hi0 )  QSort(lo, hi0 );

      }
   }
*/

/*
  void SortRange(int lo0, int hi0) throws Exception
  {
    QSort(hi0 - lo0 + 1, hi0);
  }
*/
