/*
    Advanced Explorer (c) 1999-2006 by D.G. Sureau

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    webmaster@scriptol.org
    http://www.scriptol.org
*/
package common;

import java.awt.*;
import java.awt.Color;
import common.*;

/**
  Defines a Spinner.
  Listeners can register to listen for ActionChanged Events on the Spinner.
 */

public class Spinner extends Component
{

    protected int value;
    protected int minValue = Integer.MIN_VALUE;
    protected int maxValue = Integer.MAX_VALUE;
    protected java.awt.event.ActionListener actionListener;

    protected java.awt.Polygon upArrow = new java.awt.Polygon();
    protected java.awt.Polygon dnArrow = new java.awt.Polygon();
    protected java.awt.Polygon upPressedArrow = new java.awt.Polygon();
    protected java.awt.Polygon dnPressedArrow = new java.awt.Polygon();

    protected boolean upPressed, dnPressed;
    protected boolean rollsOver = false;
    protected long oldMouseClickTime;
    private String actionCommand = "which?";

    private static long DELAY_PERIOD = 150;
    public boolean Changed = false;

    /** Constructs a Spinner. */

    public Spinner()
    {
      super();
      enableEvents( java.awt.AWTEvent.MOUSE_EVENT_MASK );
    }

    public Spinner(int v) {   this(); value = v;  }
    public Spinner(int v, int mi, int mx)
     {
        this();
        value = v;
        minValue = mi;
        maxValue = mx;
     }


   /**
     * Waits till after super.addNotify to ensure there will be a
     * graphics context for call to figureMySize, then determines the
     * component's size.

       Cette mthode est appele par la fonction "add" du container
       (et non par l'utilisateur).
     * @see java.awt.Component.addNotify
     * @see java.awt.Component.setSize
    */

    // modifi car il il faut passer par AdjustSize
    // en effet, setBound ne met pas 0
    // et il faut setBound pour placer le composant

    public void addNotify()
    {
       super.addNotify();

//       java.awt.Dimension d = getSize();
//       if( d.width == 0  ||  d.height == 0 )
       setSize( this.adjustSize( this.getGraphics() ) );
    }



    /**
     * Overridden to call repaint in case change of enabled
     * state affects the look of the lightweight component.
     */
    public void setEnabled( boolean state )
    {
        super.setEnabled( state );
        repaint();
    }


    /**
     * Overridden to call paint without clearing the
     * screen so as to reduce flicker.
     */
    public void update( java.awt.Graphics g ) { paint( g ); }

    /** Overridden to return the preferred size of the component  */
    public java.awt.Dimension getMinimumSize()
    {
        return getPreferredSize();
    }


    /**
       Overridden to return either the user-established size
         set by calling "setSize",
       or the size as determined by calling "AdjustSize".
       Must be called after component has had a graphics
       context established for it.

       @return Dimension representing the component's preferred size
     */

    public java.awt.Dimension getPreferredSize()
    {
     if( getSize().width != 0  &&  getSize().height != 0 )
            return getSize();
     else   return adjustSize( getGraphics() );
    }


    /**  Sets the displayed value of the Spinner  */

    public synchronized void setValue( int newValue )
    {
        this.value = newValue;
        this.repaint();
    }

    public synchronized void setLimits(int minimum, int maximum)
    {
        this.minValue = minimum;
        this.maxValue = maximum;
    }


    public int getValue() {  return value;  }
    public synchronized void setMinValue( int value ) { setValue( minValue = value ); }
    public int getMinValue() { return minValue; }
    public synchronized void setMaxValue( int value ) { maxValue = value; }
    public int getMaxValue() { return maxValue; }

    /** Met en mode cyclage (retour au dbut aprs la fin) */
    public void setRollsOver( boolean state ) { rollsOver = state; }
    public boolean getRollsOver() { return rollsOver; }

    public void setActionCommand( String newActionCommand ) { actionCommand = newActionCommand; }
    public String getActionCommand() { return actionCommand;}

    /** Adds the ActionListener as a registered listener of ActionEvents */
    public void addActionListener( java.awt.event.ActionListener al )
    {
        actionListener = java.awt.AWTEventMulticaster.add( actionListener, al );
    }

    /** Removes the ActionListener as a registered listener of ActionEvents */
    public void removeActionListener( java.awt.event.ActionListener al )
    {
        actionListener = java.awt.AWTEventMulticaster.remove( actionListener, al );
    }

    /** Gets the optimum size for this Spinner */
    protected java.awt.Dimension adjustSize( java.awt.Graphics g )
    {
        final int h = 24;
        final int w = 48;
	    final int m = h / 2;

	    // Unpressed up triangle   Hauteur 6 largeur 12
		upArrow.addPoint( w - 16, m - 2 );
		upArrow.addPoint( w - 2,  m - 2 );
		upArrow.addPoint( w - 9,  m - 9 );

		// Unpressed triangle bas
		dnArrow.addPoint( w - 9, m + 7 );
		dnArrow.addPoint( w - 15, m + 1 );
		dnArrow.addPoint( w - 3, m + 1 );

		// Pressed polygons (dcals en bas  droite
		upPressedArrow.addPoint( w - 9, 5 );
		upPressedArrow.addPoint( w - 14, m - 1 );
		upPressedArrow.addPoint( w - 4, m - 1 );

		dnPressedArrow.addPoint( w - 9, h - 4 );
		dnPressedArrow.addPoint( w - 14, m + 2 );
		dnPressedArrow.addPoint( w - 4, m + 2 );

		return (new java.awt.Dimension( w, h ));
    }


	/** Paints the Spinner */

    public void paint( java.awt.Graphics g )
    {
        Dimension mySize = getSize();

        int bw = mySize.width;
        int bh = mySize.height;

		Color borderlight = getBackground().brighter();
		Color borderdark  = getBackground().darker();

		// Bord clair du bouton
		g.setColor(borderlight);
		g.drawLine( 0, 0, bw - 1, 0 );
		g.drawLine( 0, 0, 0, bh );

		// Bord fonc du bouton
		g.setColor( borderdark );
		g.drawLine( 2, bh - 2, bw, bh - 2 );
		g.drawLine( 1, bh - 1, bw, bh - 1 );
		g.drawLine( bw, 0, bw, bh );
		g.drawLine( bw - 1, 1, bw - 1, bh );


        // Dans le bouton,
        // rectangle pour la valeur
		Rectangle rect = new Rectangle( 5, 4, bw - 23, bh - 9 );
		int rx = rect.x;          // gauche
		int ry = rect.y;          // haut
		int rw = rect.width;
		int rh = rect.height;
		int rr = rx + rw - 1;
		int rb = ry + rh - 1;

        // Fond des chiffres
		if( isEnabled() )  g.setColor( Color.black );
		else      	       g.setColor( Color.gray);
		g.fillRect( rx - 1, ry - 1, rw + 2, rh + 2 );

        // Dessine les triangles
		if( !upPressed )  g.fillPolygon( upArrow );
	    else              g.fillPolygon( upPressedArrow );

	    if( !dnPressed )  g.fillPolygon( dnArrow );
	    else              g.fillPolygon( dnPressedArrow );

		// Bord du fond de texte
		g.setColor( borderlight );
		g.drawLine( rx , rb + 1, rr + 1, rb + 1);     // bas
		g.drawLine( rr + 1, ry + 1, rr + 1, rb + 1);  // cot droit

		// Valeur
		g.setFont( new Font( "Dialog", Font.BOLD, 14 ) );
		FontMetrics fm = g.getFontMetrics();
		// Largeur du texte en  pixels
        int textwidth = fm.stringWidth("" + value);
	    int y = ry + (fm.getHeight() / 2) + 4;
	    int x = rx + (rw - textwidth ) / 2 + 1;
	    g.setColor( Color.green.darker() );
	    g.drawString("" + value, x, y );

    }


    /**
      Helper method used for calling actionPerformed
      on the ActionListeners for this Spinner.
    */
    private void notifyListeners()
    {
     if( actionListener != null )
      {
          actionListener.actionPerformed( new java.awt.event.ActionEvent (
                this, java.awt.event.ActionEvent.ACTION_PERFORMED, this.getActionCommand()
            ) );
      }
    }


    /**
     * Increments the value of this Spinner. Tests whether needs to rollover.
     * Notifies listeners when increment occurs.
     */

    public void increment()
    {
       if( value < maxValue )
         {
            ++value;
            notifyListeners();
         }
       else
       if( rollsOver ) { value = minValue; notifyListeners(); }
    }


    /**
     * Decrements the value of this Spinner. Tests whether needs to rollover.
     * Notifies listeners when decrement occurs.
     */

    public void decrement()
    {
       if( value > minValue )
         {
            --value; notifyListeners();
         }
       else
       if( rollsOver ) { value = maxValue; notifyListeners(); }
    }



     // can try to set flag when spinning and use finalize to unset
     // or can try to catch thread death to unset flag

     public class SpinThread extends Thread
      {
        public SpinThread()
        {
          super();
          this.setPriority( Thread.MIN_PRIORITY );
        }

        public void run()
        {
          try
          {
            if( upPressed ) increment();
            if( dnPressed ) decrement();
            repaint();
            Thread.sleep( 1250 );
            while( upPressed || dnPressed )
            {
              if( upPressed ) increment();
              if( dnPressed ) decrement();
              repaint();
              Thread.sleep( 400 );
            }
          } catch( InterruptedException ex ) { ; }
         }
     } // Class

    public void spin()
    {
     new SpinThread().start();
    } // spin



   /**
    * Handle any mouse activity on this Spinner, especially processing mouse
    * downs to spin the Spinner and mouse ups to stop the Spinner.
    * Pass all other activity up the inheritance chain.
    */
	public void processMouseEvent( java.awt.event.MouseEvent evt )
	{
	    if( isEnabled() )
	    {
    	 switch( evt.getID() )
    	 {
           case java.awt.event.MouseEvent.MOUSE_PRESSED:
        		    // Do not let user start up too many spin threads in a short time
        		    long newMouseClickTime = System.currentTimeMillis();
        		    if( newMouseClickTime - oldMouseClickTime > DELAY_PERIOD )
        		    {
            		    if( upArrow.contains( new java.awt.Point(evt.getX(), evt.getY()) ) )
            		    {
            		        upPressed = true;
        		            spin();
            		    }
            		    else if( dnArrow.contains( new java.awt.Point(evt.getX(), evt.getY()) ) )
            		    {
            		        dnPressed = true;
        		            spin();
            		    }
        		    }
        		    oldMouseClickTime = newMouseClickTime;
        		    break;
        	case java.awt.event.MouseEvent.MOUSE_RELEASED:
        	    if( upPressed == true ) upPressed = false;
        	    else
        	    if( dnPressed == true ) dnPressed = false;
        		repaint();
        		break;
    		}
		}

		super.processMouseEvent( evt );
	}



}
