/*
    Advanced Explorer (c) 1999-2006 by D.G. Sureau

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    webmaster@scriptol.org
    http://www.scriptol.org
*/
package common;

import java.awt.*;
import java.awt.event.*;

/**
   Class Help Tip
   utilisable par des composants

   Exige que la Frame de base du programme soit donne
   avant toute instance,
   car elle est le contexte graphique
*/

public class Tip extends Window implements ActionListener
{
  DPanel panelTip = null;   // Panel virtuel
  String tipText = null;
  static Frame parent = null;
  Timer timer;
  Component compo;    // composant associ

  int width;   // taille dfinie par setFont()
  int height;

  final int border = 2;

  static Style defStyle = new Style("yellow", "black", "Dialog", Font.PLAIN, 14);

  public Tip(Component c, String txt)
  {
   super(parent);
   setLayout(new GridLayout());

   tipText = txt;

   setFont(new Font("Dialog", Font.PLAIN, 12));

   panelTip = new DPanel();
   add(panelTip);

   setBackground(Color.yellow);
   setForeground(Color.black);

   compo = c;
 }


 /**
    Frame parente de tous les tips,
    appel  cette fonction prcde les instanciations
    car la classe hrite de Window qui s'appelle avec
    la frame parent en paramtre
 */
 public static void setParent(Frame p) { parent = p; }


 /**
    Dfinit la frame principale
     partir d'un composant
    Appel:
         Tip.makeParent(this);

    A tester
 */

 public static void setParent(Component compo)
 {
   if(parent == null)
   {
    Container par = compo.getParent();
    while(par != null)
     {
       if(par.getParent() == null)
         parent = (Frame) par;
       par = par.getParent();
     }
   }
 }

 /**
    Dessin du message
 */
 public void update(Graphics g) { paint(g); }

 public void paint(Graphics g)
 {
   super.paint(g);

   Rectangle r = panelTip.getBounds();
   int x = r.x;
   int y = r.y;
   int w = r.width - 1;
   int h = r.height - 1;

   g = panelTip.getGraphics();

   g.setColor(getBackground());
   g.fillRect(x, y, w, h);
   g.setColor(getForeground());
   g.drawRect(x, y, w, h);
   g.drawString(tipText, x + 8, y + border * 2 + 10);
 }


   public void setStyle() { setStyle(defStyle); }

   public void setStyle(Style s)
   {
     setBackground(s.background);
     setForeground(s.foreground);
     setFont(s.font);
   }


   public void setFont(Font f)
    {
      super.setFont(f);
      //GraphicsEnvironment ge = java.awt.GraphicsEnvironment.getLocalGraphicsEnvironment();

      FontMetrics FM = getToolkit().getFontMetrics(f);

      int a = FM.getAscent();
      int d = FM.getHeight() - FM.getAscent();

      height = a + d + (border << 1);
      width = FM.stringWidth(tipText) + (d << 1) + (border << 1) + 8;

      setBounds(0, 0, width, height);
    }

   public void setBackground(Color b)
    {
      super.setBackground(b);
      panelTip.setBackground(b);
    }

   public void setForeground(Color f)
    {
      super.setForeground(f);
      panelTip.setForeground(f);
    }



   public void showIt()
   {
    timer = new Timer(750, false);
    timer.addActionListener(this);

    Rectangle r = compo.getBounds();
    int x = r.x;
    int y = r.y;

    Container par = compo.getParent();
    while(par != null)
     {
       Point pp = par.getLocation();
       x += pp.x;
       y += pp.y;
       par = par.getParent();
     }

    Rectangle rp = parent.getBounds();
    int pw = rp.x + rp.width - 8;
    int ph = rp.y + rp.height - 8;

    x = x + r.width / 2;
    y = y + r.height + 8;

    if(x + width > pw) x = pw - width;
    if(y + height > ph) y = ph - height;

    setLocation(x , y);
    timer.start();
   }

   public void hideIt()
   {
    if(timer != null) timer.stop();

    setVisible(false);
    dispose();
   }

  public void actionPerformed(java.awt.event.ActionEvent e)
   {
     setVisible(true);
	 return;
   }

} // class
