/*
    Advanced Explorer (c) 1999-2006 by D.G. Sureau

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    webmaster@scriptol.org
    http://www.scriptol.org
*/
package common;

import java.io.File;
import java.awt.*;
import common.*;

 /**
    Classe qui construit une arborescence spcialise
    dans les rpertoires de disque
 */


public class TreeDir extends DTreeView
 {
  static String letters = "abcdefghijklmnopqrstuvwxyz";
  static VectorString Disks = new VectorString();
  static int maxUnits = 0;

  public TreeDir()
  {
   super();
  }

  public TreeDir(VectorString listOfDrives)
  {
    this();
    Disks = listOfDrives;
  }

  static void setDisks(String[] dlist)
  {
   for(int i = 0; i < dlist.length; i++)
    Disks.addElement(dlist[i]);
  }


  /**
    Etablit la liste des units accessibles
   */

  void driveIdent()
    {
      if(OS.isWindows())
       {

        int i = 0;
        int l = letters.length();
        maxUnits = 0;
        Disks.clear();

        if (Preferences.FLOPPIES == false)
        {
          i = 2;
          l = l - i;
        }

        while(i < l)
        {
         char c = letters.charAt(i);
         String drive = c + ":";
         File test = new File(drive + File.separatorChar);
         if(test.exists())
         {
           Disks.addElement(drive);
           maxUnits++;
         }
         i++;
        }
      } // Windows
    }


  /**
    Teste s'il y a un sous-rp
    et met alors le drapeau child
   */
  void setSub(DTreeNode node, String curDir)
  {
    String l[] = new File(curDir, File.separator).list();

    int m = l.length;
    for(int i = 0; i < m; i++)
     {
      /*
      if (curDir.equals("c:"))
      {
        System.out.println(curDir + ")" + l[i]);
      }
      */
      if((new File(curDir, l[i])).isDirectory())
      {
       node.isDirectory = true;  // c'est une branche non remplie encore
       node.isExpandable = true; // car sera complt ensuite
       return;
      }
     }
  }

  /** Cre l'arborescence de base  */
  public void init()
  {
   DTreeNode node;

   if(OS.isWindows())
   {
    driveIdent();

    for(int i = 0; i < maxUnits; i++)
    {
     String disk = Disks.elementAt(i);
     node = new DTreeNode(disk, DirIcons.IconDrive, DirIcons.IconDrive, this);
     append(node);
     setSub(node, disk);
    }
   }
   else
   {
     node = new DTreeNode("/", this);
     append(node);
     setSub(node, "/");
   }
}


    /**
     Au moment de l'affichage,
     complte l'arborescence d'un node
     On reconstitue le chemin selon les nodes parents

     Surcharge la fonction dans les handles de TreeView
  */

  protected void complete(DTreeNode node, int x, int y)
  {
    if(!node.isIncomplete()) return;
    String path = "";

    Dimension s = getSize();
    if(x == 0 && y == 0)
    {
     x = s.width / 3 - 64;
     y = s.height / 2 - 8;
    }
    if(y + 32 > s.height) y -= 48;

    //Tool.drawTip(getGraphics(), x + 64, y + 8);
    setCursor(new Cursor(Cursor.WAIT_CURSOR));

    DTreeNode tNode = node;
    while(tNode != null)
    {
     path = Path.merge(tNode.getText(), path);
     tNode = tNode.getParent();
    }

    makeBranch(node, path);
    setCursor(Cursor.getDefaultCursor());
  }

  /** Fonction rcursive sur une branche */

  public void makeBranch(DTreeNode oldNode, String branch)
  {
    int mode = 0;
    boolean first = true;
    branch = Path.makeJavaRoot(branch);

    String curlist[] = new File(branch, File.separator).list();   // Liste du contenu
    int l = curlist.length;

    for(int i = 0; i < l; i++)
    {
     String fic = curlist[i];
     try
     {
       // Pour chaque entre dans le rpertoire "branch" teste si...
      if((new File(branch, fic)).isDirectory())
      {
        DTreeNode newNode =
           new DTreeNode(fic, DirIcons.IconDir, DirIcons.Expanded,  this );

       if(first) { mode = DTreeView.CHILD; first = false; }
       else        mode = DTreeView.LAST;

       insert(newNode, oldNode, mode);
       setSub(newNode, branch + File.separator + fic);

       oldNode = newNode;
      }
     } catch( NullPointerException e) { }

    }

  } // make



  /** Lecture du chemin complet du rpertoire slectionn */

  public String getSelectedPath()
   {
	DTreeNode node = getSelectedNode();
	String path = "";

	while(node != null)
	{
	  path = Path.merge(node.getText(), path);
	  node = node.getParent();
	}
    return path;
  }

}
